Package: SeroTrackR
Type: Package
Title: Serology-Based Data Analysis and Visualization
Version: 1.1.1
Authors@R: c(
    person(
      "Dionne", "Argyropoulos",
      email = "argyropoulos.d@wehi.edu.au",
      role = c("aut", "cre")
    )
  )
Description: Data wrangling and cleaning, quality control checks and 
  implementation of machine learning classification algorithm. 
License: CC BY 4.0
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: dplyr, drc, forcats, ggplot2, here, janitor, kableExtra,
        knitr, magrittr, openxlsx, parsnip, purrr, ranger, readxl,
        rmarkdown, stats, stringr, tidyr, tidyselect, utils, workflows
URL: https://github.com/dionnecargy/SeroTrackR,
        https://dionnecargy.github.io/SeroTrackR/
BugReports: https://github.com/dionnecargy/SeroTrackR/issues
Suggests: glue, htmltools, httr, jsonlite, readr, rsconnect,
        shiny.fluent, tidyverse, zoo
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-26 09:30:35 UTC; argyropoulos.d
Author: Dionne Argyropoulos [aut, cre]
Maintainer: Dionne Argyropoulos <argyropoulos.d@wehi.edu.au>
Repository: CRAN
Date/Publication: 2026-05-27 07:20:16 UTC
Built: R 4.5.2; ; 2026-05-27 07:51:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(.check_platform)
export(.clean_bioplex)
export(.clean_luminex)
export(.convert_dilution_to_mfi)
export(.convert_mfi_to_dilution)
export(.convert_mfi_to_dilution_no_bounds)
export(.convert_mfi_to_dilution_no_lower_bound)
export(.extract_luminex_sections)
export(.fit_standard_curve)
export(.magpix_version_config)
export(.merge_mfitorau)
export(.post_process_bioplex)
export(.post_process_luminex)
export(.process_antigen_loglog)
export(.read_luminex_file)
export(.relabel_columns)
export(.setup_mfitorau_inputs)
export(MFItoRAU)
export(MFItoRAU_Adj)
export(MFItoRAU_LDH)
export(MFItoRAU_Pk)
export(MFItoRAU_Plasmo)
export(classifyResults)
export(getAntigenCounts)
export(getCounts)
export(getCountsQC)
export(getGithubRelease)
export(getPlateLayout)
export(getRepeats)
export(getSampleID)
export(makeCard)
export(plotBeadCounts)
export(plotBlanks)
export(plotBoxPlotClassification)
export(plotCounts)
export(plotMFI)
export(plotModel)
export(plotModel_Adj)
export(plotRAU)
export(plotStds)
export(plotStds_PkPfPv)
export(plotStds_all)
export(processCounts)
export(processPkPfPv)
export(readPlateLayout)
export(readSeroData)
export(renderClassificationTable)
export(renderDetailsList)
export(renderQCReport)
export(renderReport)
export(renderTwoCols)
export(runLDHPipeline)
export(runPlasmoPipeline)
export(runPvSeroPipeline)
export(runQC)
import(dplyr)
import(drc)
import(ggplot2)
import(parsnip)
import(ranger)
import(stringr)
import(tidyr)
import(workflows)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,ends_with)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_split)
importFrom(dplyr,if_any)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(drc,drm)
importFrom(forcats,fct_rev)
importFrom(here,here)
importFrom(janitor,clean_names)
importFrom(janitor,row_to_names)
importFrom(kableExtra,kable_styling)
importFrom(knitr,asis_output)
importFrom(knitr,kable)
importFrom(magrittr,"%>%")
importFrom(openxlsx,getSheetNames)
importFrom(openxlsx,read.xlsx)
importFrom(openxlsx,write.xlsx)
importFrom(purrr,imap_dfr)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(purrr,reduce)
importFrom(readxl,read_excel)
importFrom(rmarkdown,render)
importFrom(stats,coef)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(tidyr,as_tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,separate)
importFrom(tidyr,unnest)
importFrom(tidyselect,matches)
importFrom(tools,file_ext)
importFrom(utils,count.fields)
importFrom(utils,data)
importFrom(utils,read.csv)
importFrom(utils,read.csv2)
importFrom(utils,write.csv)

# <img src="man/figures/SeroTrackR_sticker.jpeg" width="25%" height="25%" align="left"/> SeroTrackR

The `{SeroTrackR}` R package compliments the PvSeroApp [see Github here](https://github.com/dionnecargy/PvSeroApp) for experienced users who are comfortable with implementing the code in their own R sessions. 

This package contains helper functions relevant for *Plasmodium* spp. projects for in-house and collaborator needs. 

<p>&nbsp;</p>

## How to install the package:

```{r}
devtools::install_github("dionnecargy/SeroTrackR") # To download the package
library(SeroTrackR) # To load the package 
```

## How can I use the package?

Please see the [tutorial](https://dionnecargy.github.io/SeroTrackR/) or see the vignettes inside the package by running the following commands: 

```{r}
browseVignettes("SeroTrackR") 

# To run a specific vignette: 
vignette("PvSeroApp_R_Tutorial")
vignette("Pk_Pv_Pf_Serology_Tutorial")
```

## -----------------------------------------------------------------------------
# install.packages("tidyverse")

## -----------------------------------------------------------------------------
# library(tidyverse)

## -----------------------------------------------------------------------------
# setup <- function(){
#   needed <- c(
#     # Imports: Required
#     "dplyr", "drc", "forcats", "ggplot2", "here", "janitor",
#     "kableExtra", "knitr", "magrittr", "openxlsx", "parsnip",
#     "purrr", "ranger", "readr", "readxl", "rmarkdown", "stats",
#     "stringr", "tidyr", "tidyselect", "utils", "workflows",
#     # Imports: Suggested
#     "glue", "htmltools", "httr", "jsonlite", "shiny.fluent",
#     "tidyverse", "zoo"
#     )
#   for(package in needed){
#     if(!sum(installed.packages() %in% package)){
#       install.packages(package)
#     }
# 
#     require(package, character.only = TRUE)
#   }
# }
# 
# setup()

## -----------------------------------------------------------------------------
# my_raw_data       <- "data/my_plate1.csv"
# my_plate_layout   <- "data/my_plate_layout.xlsx"

## -----------------------------------------------------------------------------
# getPlateLayout("your/folder/with/plate/layouts/")

## -----------------------------------------------------------------------------
# getPlateLayout("data/")

## -----------------------------------------------------------------------------
# getPlateLayout()

## -----------------------------------------------------------------------------
# getPlateLayout(folder_path = c("plate_layout_1.xlsx", "plate_layout_2.xlsx", "plate_layout_3.xlsx"))

## -----------------------------------------------------------------------------
library(SeroTrackR)
library(tidyverse)

bioplex_raw_plates <- c(
  system.file("extdata", "example_BioPlex_plate1.xlsx", package = "SeroTrackR"),
  system.file("extdata", "example_BioPlex_plate2.xlsx", package = "SeroTrackR")
)
all_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")

## -----------------------------------------------------------------------------
# bioplex_raw_plates <- c(
#   "data/example_BioPlex_plate1.xlsx",
#   "data/example_BioPlex_plate2.xlsx"
# )
# all_plate_layout <- "data/example_platelayout_1.xlsx"

## -----------------------------------------------------------------------------
# magpix_raw_plate     <- system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")

## -----------------------------------------------------------------------------
# magpix_raw_plate      <- "data/example_MAGPIX_plate3.csv"

## -----------------------------------------------------------------------------
# # Serological data
# bioplex_sero_data <- readSeroData(
#   raw_data = bioplex_raw_plates,
#   platform = "bioplex"
# )
# magpix_sero_data <- readSeroData(
#   raw_data = magpix_raw_plate,
#   platform = "magpix",
#   version = "4.2"
# )

## -----------------------------------------------------------------------------
# sero_data_merged <- NULL
# # data_raw
# sero_data_merged$data_raw <- bioplex_sero_data$data_raw %>%
#   bind_rows(magpix_sero_data$data_raw)
# 
# # results
# sero_data_merged$results <- bioplex_sero_data$results %>%
#   bind_rows(magpix_sero_data$results)
# 
# # counts
# sero_data_merged$counts <- bioplex_sero_data$counts %>%
#   bind_rows(magpix_sero_data$counts)
# 
# # blanks
# sero_data_merged$blanks <- bioplex_sero_data$blanks %>%
#   bind_rows(magpix_sero_data$blanks)
# 
# # stds
# sero_data_merged$stds <- bioplex_sero_data$stds %>%
#   bind_rows(magpix_sero_data$stds)
# 
# # run
# sero_data_merged$run <- bioplex_sero_data$run %>%
#   bind_rows(magpix_sero_data$run)

## -----------------------------------------------------------------------------
# plate_list_all  <- readPlateLayout(
#   plate_layout = all_plate_layout,
#   sero_data = sero_data_merged
# )
# 
# qc_results <- runQC(
#   sero_data = sero_data_merged,
#   plate_list = all_plate_layout
# )
# 
# mfi_to_rau_output <- MFItoRAU(
#   sero_data = sero_data_merged,
#   plate_list = all_plate_layout,
#   qc_results = qc_results,
#   std_point = 10
# )
# 
# # etc..

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(rmarkdown.html_vignette.check_title = FALSE)
setup <- function() {
  needed <- c("knitr", "rmarkdown", "tidyverse", "kableExtra")
  
  lapply(needed, function(pkg) {
    if (requireNamespace(pkg, quietly = TRUE)) {
      library(pkg, character.only = TRUE)
    }
  })
}

setup()
library(SeroTrackR)

knitr::opts_chunk$set(
  dpi = 72
)

knitr::opts_chunk$set(
  dpi = 72
)

## ----setup 1------------------------------------------------------------------
library(SeroTrackR)
library(tidyverse)

your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"), 
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")

## ----exec=FALSE, eval=FALSE---------------------------------------------------
# your_raw_data <- c(
#   "PATH/TO/YOUR/FILE/plate1.csv",
#   "PATH/TO/YOUR/FILE/plate2.csv",
#   "PATH/TO/YOUR/FILE/plate3.csv"
# )
# your_plate_layout <- "PATH/TO/YOUR/FILE/plate_layout.xlsx"

## ----runPvSeroPipeline with classification------------------------------------
final_analysis <- runPvSeroPipeline(
  raw_data = your_raw_data, 
  plate_layout = your_plate_layout, 
  platform = "magpix", 
  location = "ETH", 
  experiment_name = "experiment1", 
  classify = "Yes", 
  algorithm_type = "antibody_model", 
  sens_spec = "balanced"
)

## ----classification tab 1-----------------------------------------------------
final_analysis[[1]] %>%
  head() %>% 
  kable()

## ----std curve plot tab 1-----------------------------------------------------
final_analysis[[2]]

## ----bead counts plot tab 1---------------------------------------------------
final_analysis[[3]] # Plot
final_analysis[[4]] # Samples to repeat 

## ----blanks qc plot tab 1-----------------------------------------------------
final_analysis[[5]]

## ----model output plot tab 1--------------------------------------------------
final_analysis[[6]]

## ----runPvSeroPipeline without classification---------------------------------
no_classification_final_analysis <- runPvSeroPipeline(
  raw_data = your_raw_data, 
  plate_layout = your_plate_layout, 
  platform = "magpix", 
  location = "ETH", 
  experiment_name = "experiment1", 
  classify = "No", ########################## key if you do NOT want any classification performed i.e., you do not have PvSeroTaT antigens 
  algorithm_type = "antibody_model", 
  sens_spec = "balanced"
)

## ----mfi and rau tab 2--------------------------------------------------------
no_classification_final_analysis[[1]]  %>%
  head() %>% 
  kable()

## ----std curve plot tab 2-----------------------------------------------------
#### Standard Curve Plot
no_classification_final_analysis[[2]]

#### Bead Counts QC Plot
no_classification_final_analysis[[3]] # Plot
no_classification_final_analysis[[4]] # Samples to repeat 

#### Blanks QC Plot
no_classification_final_analysis[[5]]

#### Model Output Plot
no_classification_final_analysis[[6]]

## ----create pdf output, exec=FALSE, eval=FALSE--------------------------------
# renderQCReport(
#   your_raw_data,
#   your_plate_layout,
#   "magpix",
#   location = "ETH",
#   path = "inst/tutorials/" # defaults to your current working directory
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(rmarkdown.html_vignette.check_title = FALSE)
setup <- function() {
  needed <- c("knitr", "rmarkdown", "tidyverse", "kableExtra")
  
  lapply(needed, function(pkg) {
    if (requireNamespace(pkg, quietly = TRUE)) {
      library(pkg, character.only = TRUE)
    }
  })
}

setup()
library(SeroTrackR)

knitr::opts_chunk$set(
  dpi = 72
)

## ----setup 1, eval=FALSE------------------------------------------------------
# library(SeroTrackR)
# library(tidyverse)
# 
# your_raw_data <- system.file("extdata", "example_BioPlex_PvLDH_plate1.csv", package = "SeroTrackR")
# your_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")

## ----exec=FALSE, eval=FALSE---------------------------------------------------
# your_raw_data <- "PATH/TO/YOUR/FILE/plate1.csv"
# your_plate_layout <- "PATH/TO/YOUR/FILE/plate_layout.xlsx"

## ----example code for LDH, eval=FALSE-----------------------------------------
# runLDHpipeline(
#   raw_data = your_raw_data,
#   plate_layout = your_plate_layout,
#   platform = "bioplex",                               # Defaults to "bioplex" but "magpix" also works
#   file_path = NULL                                    # Defaults to current folder/working directory
# )

#' Create a Fluent UI Table
#'
#' This function makes the table in a Fluent UI format.
#'
#' @param df Any processed data frame
#' @return A table in the Fluent UI format
#' @export
#'
#' @importFrom dplyr as_tibble
#' @author Dionne Argyropoulos
#'
#' @examples
#' # Minimal example using a small data frame.
#' # This example is safe for CRAN because it runs only if
#' # shiny.fluent and htmltools are installed.
#'
#' if (requireNamespace("shiny.fluent", quietly = TRUE) &&
#'     requireNamespace("htmltools", quietly = TRUE)) {
#'
#'   # Tiny example data frame
#'   example_df <- data.frame(
#'     Sample = c("A", "B"),
#'     Value = c(10, 20),
#'     stringsAsFactors = FALSE
#'   )
#'
#'   # Render Fluent UI DetailsList
#'   renderDetailsList(example_df)
#' }
renderDetailsList <- function(df) {

  # Check if shiny.fluent is installed
  if (!requireNamespace("shiny.fluent", quietly = TRUE)) {
    stop("Package 'shiny.fluent' is required for renderDetailsList(). Please install it.", call. = FALSE)
  }

  # Check if htmltools is installed
  if (!requireNamespace("htmltools", quietly = TRUE)) {
    stop("Package 'htmltools' is required for renderDetailsList(). Please install it.", call. = FALSE)
  }

  # Use functions from shiny.fluent with :: prefix
  htmltools::div(
    class = "ms-Grid-row",
    htmltools::div(
      class = "ms-Grid-col ms-sm12",  # Use ms-sm12 for full width on small screens
      shiny.fluent::Stack(
        tokens = list(childrenGap = 10),
        horizontal = TRUE,
        htmltools::div(
          style = "max-height: 600px; overflow: auto; width: 100%;",
          shiny.fluent::DetailsList(
            items = df,
            columns = dplyr::tibble(fieldName = names(df), name = names(df)),
            constrainMode = 0,
            checkboxVisibility = 2,
            styles = list(
              root = list(
                width = "100%",  # Ensure table width is constrained within the available space
                minWidth = "fit-content",  # Allow table to grow to fit content
                overflowX = "auto"  # Enable horizontal scrolling only when necessary
              )
            )
          )
        )
      )
    )
  )
}

#' Run Pk/Pf/Pv Data Analysis Pipeline from Start to End
#'
#' @param raw_data  String with the raw data path.
#' @param platform  "magpix" or "bioplex". Default: "Bioplex"
#' @param plate_layout An ".xlsx" file with sheets labelled plate1, plate2... etc.
#' @param panel Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve. Value is an integer.
#' @param experiment_name User-input experiment name. Default: "experiment1".
#' @param classify "Yes" or "No" depending on whether you would like classification or not. Default = "Yes".
#' @param algorithm_type Algorithm: "antibody_model" (PvSEM algorithm; default)
#' @param sens_spec User-selected Sensitivity/Specificity threshold: "balanced"
#' (default) or "90\% specificity".
#'
#' @return A data frame containing the MFI and RAU Dilution values for each sample, QC plots for standard curve, bead counts and blanks.
#' @export
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Helper to avoid repetition in examples
#' run_example_std <- function(std_point) {
#'   # Load raw data for given standard curve
#'   your_raw_data <- c(
#'     system.file("extdata",
#'                 paste0("example_MAGPIX_pk_", std_point, "std_plate1.csv"),
#'                 package = "SeroTrackR"),
#'     system.file("extdata",
#'                 paste0("example_MAGPIX_pk_", std_point, "std_plate2.csv"),
#'                 package = "SeroTrackR")
#'   )
#'
#'   layout_file <- system.file(
#'     "extdata",
#'     paste0("example_platelayout_pk_", std_point, "std.xlsx"),
#'     package = "SeroTrackR"
#'   )
#'
#'   # Run pipeline
#'   runPlasmoPipeline(
#'     raw_data = your_raw_data,
#'     platform = "magpix",
#'     plate_layout = layout_file,
#'     panel = "panel1",
#'     std_point = std_point,
#'     experiment_name = paste0(std_point, "-point standard curve")
#'   )
#' }
#'
#' # ---- 5-point standard curve ----
#' results_5std <- run_example_std(5)
#'
#' # ---- 10-point standard curve ----
#' results_10std <- run_example_std(10)
#'
#' }
runPlasmoPipeline <- function(
    raw_data,
    platform = "magpix",
    plate_layout,
    panel = "panel1",
    std_point,
    experiment_name = "experiment1",
    classify = "Yes",
    algorithm_type = "antibody_model",
    sens_spec = "balanced"
  ){

  #############################################################
  # Step 1: Reading in Raw Data
  #############################################################
  sero_data                 <- readSeroData(raw_data = raw_data, platform)
  plate_list                <- readPlateLayout(plate_layout = plate_layout, sero_data = sero_data)

  #############################################################
  # Step 2: Quality Control
  #############################################################
  qc_results                <- runQC(sero_data, plate_list)
  message("QC Processes completed.")

  #############################################################
  # Step 3: Plotting
  #############################################################
  stdcurve_plot             <- suppressWarnings(plotStds_PkPfPv(sero_data, experiment_name, panel))
  plateqc_plot              <- plotCounts(qc_results, experiment_name)
  check_repeats_output      <- getRepeats(qc_results, plate_list)
  blanks_plot               <- plotBlanks(sero_data, experiment_name)
  message("QC Plotting completed.")

  #############################################################
  # Step 4: Run new 5-point MFI to RAU
  #############################################################
  mfi_outputs               <- MFItoRAU_Plasmo(
    sero_data = sero_data,
    plate_list = plate_list,
    panel = panel,
    std_point = std_point,
    qc_results = qc_results
  )
  message("MFI to RAU conversion completed.")

  #############################################################
  # Step 5: Perform Pv classification
  #############################################################
  if(classify == "Yes"){
    Pv_classified             <- classifyResults(mfi_outputs, algorithm_type, sens_spec, qc_results, project = "pkpfpv")
    message("Pv classification completed.")
    return(list(
      std_curve = stdcurve_plot,
      bead_counts = plateqc_plot,
      blanks = blanks_plot,
      mfi_outputs = mfi_outputs,
      pv_classification = Pv_classified
    ))
  } else {
    message("No Classification Performed.")
    return(list(
      std_curve = stdcurve_plot,
      bead_counts = plateqc_plot,
      blanks = blanks_plot,
      mfi_outputs = mfi_outputs
    ))
  }

}

#' Make Card in Fluent UI
#'
#' This function imports the makes a card following the Fluent UI format.
#'
#' @param title String with the large title that will be printed in the card.
#' @param id Identifying tag for use to link.
#' @param content A list of content to be rendered.
#' @param size A value from 1 to 12 of the width of the screen (default = 12).
#' @param style Value for any css styling.
#' @return A "card" in the Fluent UI format with content.
#' @export
#' @examples
#' # Minimal example creating a simple Fluent UI card.
#' # Safe for CRAN: runs only if shiny.fluent, htmltools, and glue are installed.
#'
#' if (requireNamespace("shiny.fluent", quietly = TRUE) &&
#'     requireNamespace("htmltools", quietly = TRUE) &&
#'     requireNamespace("glue", quietly = TRUE)) {
#'
#'   # Simple card content
#'   card_content <- list(
#'     htmltools::div("This is some example text inside the card.")
#'   )
#'
#'   # Create a Fluent UI card
#'   makeCard(
#'     title   = "Example Card",
#'     id      = "example-card",
#'     content = card_content,
#'     size    = 6
#'   )
#' }
makeCard <- function(title, id, content, size = 12, style = "") {

  # Check if shiny.fluent is installed
  if (!requireNamespace("shiny.fluent", quietly = TRUE)) {
    stop("Package 'shiny.fluent' is required for makeCard(). Please install it.", call. = FALSE)
  }

  # Check if glue is installed
  if (!requireNamespace("glue", quietly = TRUE)) {
    stop("Package 'glue' is required for makeCard(). Please install it.", call. = FALSE)
  }

  # Check if htmltools is installed
  if (!requireNamespace("htmltools", quietly = TRUE)) {
    stop("Package 'htmltools' is required for makeCard(). Please install it.", call. = FALSE)
  }

  # Use functions from shiny.fluent with :: prefix
  htmltools::div(
    id = id,
    class = glue::glue("card ms-depth-8 ms-sm{size} ms-xl{size}"),
    style = style,
    shiny.fluent::Stack(
      tokens = list(padding = 20, childrenGap = 5),
      shiny.fluent::Text(variant = "large", title, block = TRUE),
      content
    )
  )
}

#' Run PvSero Pipeline from Start to End
#'
#' A master function combining the entire PvSeroApp pipeline into one command to run in R.
#'
#' @param raw_data  String with the raw data path.
#' @param platform  "magpix" or "bioplex". Default = "magpix".
#' @param plate_layout An ".xlsx" file with sheets labelled plate1, plate2... etc.
#' @param location  "PNG" or "ETH" to filter WEHI standard curve data.
#' @param experiment_name User-input experiment name.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#' @param classify "Yes" or "No" depending on whether you would like classification or not. Default = "Yes".
#' @param algorithm_type Algorithm: "antibody_model" (PvSEM algorithm; default)
#' @param sens_spec User-selected Sensitivity/Specificity threshold: "balanced"
#' (default) or "90\% specificity".
#'
#' @returns classifyResults_output, stdcurve_plot, plateqc_plot, check_repeats_output, blanks_plot, model_plot
#' @export
#'
#' @import workflows parsnip  ggplot2 drc dplyr
#' @importFrom janitor row_to_names
#' @importFrom openxlsx getSheetNames read.xlsx
#' @importFrom purrr map
#' @importFrom readxl read_excel
#' @importFrom rmarkdown render
#' @importFrom stringr str_replace  str_detect
#' @importFrom tidyr  pivot_longer  pivot_wider drop_na nest unnest
#' @importFrom tools file_ext
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#'
#' \donttest{
#' # Example data supplied with the package
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' plate_layout <- system.file(
#'   "extdata", "example_platelayout_1.xlsx", package = "SeroTrackR"
#' )
#'
#' # Run full pipeline including classification
#' runPvSeroPipeline(
#'   raw_data = your_raw_data,
#'   plate_layout = plate_layout,
#'   platform = "magpix",
#'   location = "PNG",
#'   experiment_name = "experiment1",
#'   std_point = 10,
#'   algorithm_type = "antibody_model",
#'   sens_spec = "balanced",
#'   classify = "Yes"
#' )
#'
#' # Run processing pipeline only (no classification)
#' runPvSeroPipeline(
#'   raw_data = your_raw_data,
#'   plate_layout = plate_layout,
#'   platform = "magpix",
#'   location = "PNG",
#'   experiment_name = "experiment1",
#'   std_point = 10,
#'   algorithm_type = "antibody_model",
#'   sens_spec = "balanced",
#'   classify = "No"
#' )
#' }
runPvSeroPipeline <- function(
    raw_data,
    plate_layout,
    platform = "magpix",
    location,
    experiment_name = "experiment1",
    std_point = 10,
    classify = "Yes",
    algorithm_type = "antibody_model",
    sens_spec = "balanced"
  ){

  #############################################################
  # Step 1: Reading in Raw Data
  #############################################################
  sero_data                 <- readSeroData(raw_data, platform)
  plate_list                <- readPlateLayout(plate_layout, sero_data)

  #############################################################
  # Step 2: Quality Control and
  #############################################################
  qc_results                <- runQC(sero_data, plate_list)
  message("QC Processes completed.")

  #############################################################
  # Step 3: MFI to RAU Conversion
  #############################################################
  if(location == "ETH"){
    mfi_to_rau_output       <- suppressMessages(MFItoRAU_Adj(sero_data, plate_list, qc_results, std_point, project = NULL))
  } else if(location == "PNG"){
    mfi_to_rau_output       <- suppressWarnings(MFItoRAU(sero_data, plate_list, qc_results, std_point, project = NULL))
  }
  message("MFI to RAU conversion completed.")

  #############################################################
  # Step 4: Plotting
  #############################################################
  stdcurve_plot             <- suppressWarnings(plotStds(sero_data, location, experiment_name))
  plateqc_plot              <- plotCounts(qc_results, experiment_name)
  check_repeats_output      <- getRepeats(qc_results, plate_list)
  blanks_plot               <- plotBlanks(sero_data, experiment_name)
  if(location == "ETH"){
    model_plot              <- plotModel_Adj(mfi_to_rau_output, sero_data)
  } else if(location == "PNG"){
    model_plot              <- plotModel(mfi_to_rau_output, sero_data)
  }
  message("QC Plotting completed.")

  #############################################################
  # Step 5: Classification
  #############################################################
  if(classify == "Yes"){
    classifyResults_output    <- classifyResults(mfi_to_rau_output, algorithm_type, sens_spec, qc_results, project = NULL)
    message("Pv classification completed.")
    return(list(classifyResults_output, stdcurve_plot, plateqc_plot, check_repeats_output, blanks_plot, model_plot))
  } else {
    message("No Classification Performed.")
    return(list(mfi_to_rau_output[[2]], stdcurve_plot, plateqc_plot, check_repeats_output, blanks_plot, model_plot))
  }

}

#' Run Quality Control Pipeline
#'
#' A master function containing each quality control processing step.
#'
#' @param sero_data Output from `readSeroData()`.
#' @param plate_list Output from `readPlateLayout()`.
#'
#' @returns processCounts_output, getCounts_output, sampleid_output, getAntigenCounts_output, getCountsQC_output
#' @export
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#'
#' \donttest{
#' # Example data supplied with the package
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' your_plate_layout <- system.file(
#'   "extdata", "example_platelayout_1.xlsx", package = "SeroTrackR"
#' )
#'
#' # Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Run full pipeline including classification
#' runQC(
#'   sero_data = sero_data,
#'   plate_list = plate_list
#' )
#' }
runQC <- function(sero_data, plate_list){

  # Quality Control and MFI to RAU
  processCounts_output      <- processCounts(sero_data)
  getCounts_output          <- getCounts(processCounts_output)
  sampleid_output           <- getSampleID(processCounts_output, plate_list)
  getAntigenCounts_output   <- getAntigenCounts(processCounts_output, plate_list)
  getCountsQC_output        <- getCountsQC(getAntigenCounts_output, getCounts_output)

  qc_outputs <- list(
    processCounts_output = processCounts_output,
    getCounts_output = getCounts_output,
    sampleid_output = sampleid_output,
    getAntigenCounts_output = getAntigenCounts_output,
    getCountsQC_output = getCountsQC_output
  )

  return(qc_outputs)
}
#' Process Counts from Raw Serological Data file
#'
#' A helper function to process counts data.
#'
#' @param sero_data Output from `readSeroData()`.
#' @return Returns a long table of counts with "Warning" category (<15 == 1 and
#' \eqn{>=}  15 == 0) for downstream wrangling.
#' @export
#' @importFrom dplyr mutate case_when
#' @importFrom tidyr pivot_longer
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example demonstrating how to process bead count data.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
#'
#' # Process counts
#' processed_master <- processCounts(sero_data = sero_data)
#'
#' }
#'
processCounts <- function(sero_data){

  # 1. Store Counts Data
  counts_data <- sero_data$counts

  # 2. Data Wrangling
  counts_data <- counts_data %>%
    dplyr::mutate(Location=gsub(".*,", "", Location)) %>%
    dplyr::mutate(Location=substr(Location, 1, nchar(Location)-1))  %>%
    tidyr::pivot_longer(-c(Sample, Location, Plate), names_to = "Antigen", values_to = "Count") %>%
    dplyr::mutate(Warning = case_when(
      # Bead count below 15: warning
      as.numeric(Count)<15~1,
      # Bead count above or equal to 15: good
      as.numeric(Count)>=15~0,
      # Bead count above 500: warning (impossible: 500 is theoretical maximum per antigen)
      as.numeric(Count)>500~1,
    ))

  return(counts_data)
}
#' Get Count Data from Raw Median Fluorescent Intensity
#'
#' This function obtains the count data from the raw Median Fluorescent
#' Intensity (MFI). This is an interim function used for the plotCounts
#' function. This function relies on the `readAntigens` and `readSeroData` data
#' processing functions.
#'
#' @param processed_counts Output from `processCounts()`.
#' @return (i) Data frame providing bead counts per well per plate. (ii)
#' Designates whether wells should be repeated if there are \eqn{<=}  15 beads (repeat)
#' or if they are sufficient with > 15 beads (sufficient beads).
#' @export
#' @importFrom dplyr select group_by summarise mutate
#' @author Shazia Ruybal-Pesántez, Dionne Argyropoulos
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' counts      <- processCounts(sero_data)
#' counts_raw  <- getCounts(counts)
#' }
getCounts <- function(processed_counts){

  counts <- processed_counts %>%
    dplyr::select(Location, Warning, Plate) %>%
    dplyr::group_by(Location, Plate) %>%
    dplyr::summarise(Sum = sum(Warning)) %>%
    dplyr::mutate(Repeat = case_when(
      Sum>=1 ~ "repeat",
      Sum<1 ~ "sufficient beads"
    )) %>%
    dplyr::mutate(
      Row = as.factor(substr(Location, 1, nchar(Location)-1)),
      Row = gsub("1", "", Row),
      Col = as.numeric(substr(Location, 2, nchar(Location))),
      QC_total = ifelse(Repeat == "sufficient beads", "pass", "fail")
    )

  return(counts)
}
#' Get SampleID from Plate Layout
#'
#' A helper function to extract Sample ID based on plate name and row/col
#'
#' @param processed_counts Output from `processCounts()`.
#' @param plate_list Plate name inside of the plate layout file.
#' @return Returns the corresponding Sample ID for the correct row/column in
#' the plate layout file. Henceforth "Sample ID" refers to the code in the
#' plate layout file, while "Sample" is the code in the Luminex file.
#' @export
#' @importFrom dplyr left_join select mutate bind_rows across
#' @importFrom tidyselect matches
#' @importFrom tidyr pivot_longer
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' counts      <- processCounts(sero_data)
#' counts_raw  <- getCounts(counts)
#' sample_ids  <- getSampleID(counts, plate_list)
#' }
getSampleID <- function(processed_counts, plate_list) {
  plate_layout_longer <- list()

  for (plate_level in seq_along(plate_list)) {

    # Get plate name (or fallback to index)
    plate_name <- names(plate_list)[plate_level]
    if (is.null(plate_name) || plate_name == "") {
      plate_name <- as.character(plate_level)
    }

    # Read and wrangle Plate i
    plate_layout <- plate_list[[plate_level]]
    names(plate_layout)[1] <- "Row"

    plate_layout_level <- plate_layout %>%
      dplyr::mutate(dplyr::across(tidyselect::matches("^[0-9]+$"), as.character)) %>%
      tidyr::pivot_longer(
        cols = tidyselect::matches("^[.x]*[0-9]+$"),
        names_to = "Col",
        values_to = "SampleID"
      ) %>%
      dplyr::mutate(
        Location = paste0(Row, Col),
        Plate = plate_name  # Add Plate info here
      )

    # Save to list
    plate_layout_longer[[plate_level]] <- plate_layout_level
  }

  # Combine all into a single data frame
  plate_layout_longer_df <- dplyr::bind_rows(plate_layout_longer) %>%
    dplyr::mutate(Plate = factor(Plate))  # Make Plate a factor

  # Join to antigen_specific_df
  final_table <- plate_layout_longer_df %>%
    dplyr::left_join(processed_counts, by = c("Location", "Plate")) %>%
    dplyr::select(-c(Row, Col))

  return(final_table)

}
#' Get Count Data for each Antigen from the Raw Median Fluorescent Intensity
#'
#' This function obtains the count data from the raw Median Fluorescent
#' Intensity (MFI). This function relies on the `readAntigens` and
#' `readSeroData` data processing functions.
#'
#' @param processed_counts Output from `processCounts()`.
#' @param plate_list Output from `readPlateLayout()`.
#' @return (i) Data frame providing bead counts per antigen per well per plate.
#' (ii) Designates whether wells should be repeated if there are \eqn{<=}  15 beads
#' (repeat) or if they are sufficient with > 15 beads (sufficient beads).
#' @export
#' @importFrom dplyr select group_by summarise mutate ungroup left_join arrange
#' @author Dionne Argyropoulos
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' counts      <- processCounts(sero_data)
#' counts_raw  <- getCounts(counts)
#' sample_ids  <- getSampleID(counts, plate_list)
#'
#' # Get Antigen Counts:
#' antigen_cts <- getAntigenCounts(counts, plate_list)
#' }
getAntigenCounts <- function(processed_counts, plate_list){
  #############################################################################
  # Data Wrangling
  #############################################################################

  antigen_specific_df <- processed_counts %>%
    dplyr::select(Location, Antigen, Warning, Count, Plate) %>%
    dplyr::group_by(Location, Antigen, Count, Plate) %>%
    dplyr::summarise(Sum = sum(Warning)) %>%
    dplyr::mutate(Repeat = case_when(
      Sum>=1 ~ "repeat",
      Sum<1 ~ "sufficient beads"
    )) %>%
    dplyr::mutate(
      Count = as.numeric(Count),
      Repeat = factor(Repeat, levels = c("sufficient beads", "repeat")),
      QC_antigen = ifelse(Repeat == "sufficient beads", "pass", "fail")
    )

  #############################################################################
  # Create Table Output
  #############################################################################

  table <- getSampleID(processed_counts, plate_list) %>%
    dplyr::ungroup() %>%
    dplyr::select(SampleID, Location, Antigen, Plate, Count) %>%
    dplyr::mutate(Count = as.numeric(Count))
  antigen_specific_df_final <- antigen_specific_df %>%
    dplyr::left_join(table, by = c("Plate", "Count", "Antigen", "Location")) %>%
    dplyr::select(-Sum) %>%
    dplyr::arrange(Location, Antigen, Plate)

  return(antigen_specific_df_final)

}
#' Get All Counts Data
#'
#' This function obtains the count data from the raw Median Fluorescent
#' Intensity (MFI). This function relies on the output of the Antigen-specific
#' counts (`getAntigenCounts`) and the Well or Sample-specific counts
#' (`getCounts`).
#'
#' @param antigen_counts_output Output from `getAntigenCounts`.
#' @param counts_output Output from `getCounts`.
#' @return Joined data frame for all count data.
#' @export
#' @importFrom tidyr pivot_wider
#' @importFrom dplyr select rename_with left_join all_of
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' counts      <- processCounts(sero_data)
#' counts_raw  <- getCounts(counts)
#' sample_ids  <- getSampleID(counts, plate_list)
#' antigen_cts <- getAntigenCounts(counts, plate_list)
#' counts_qc   <- getCountsQC(antigen_cts, counts_raw)
#'
#' }
getCountsQC <- function(antigen_counts_output, counts_output){

  #############################################################################
  # Data Wrangling
  #############################################################################

  # 1. Data Wrangling to store counts per antigen output
  antigen_counts_only <- antigen_counts_output %>%
    tidyr::pivot_wider(id_cols = c(SampleID, Location, Plate), names_from = "Antigen", values_from = "Count") %>% ungroup() %>%
    dplyr::select(Location, SampleID, Plate, everything()) %>%
    dplyr::rename_with(~ paste0(., "_Count"), .cols = where(is.numeric))

  # 2. Data Wrangling to store QC pass/fail per antigen output
  antigen_QC_only <- antigen_counts_output %>%
    tidyr::pivot_wider(id_cols = c(SampleID, Location, Plate), names_from = "Antigen", values_from = "QC_antigen") %>%
    dplyr::rename_with(~ paste0(., "_QC"), .cols = -c(SampleID, Location, Plate))

  # 3. Join both antigen-specific data frames together
  joined_antigen_counts <- antigen_counts_only %>%
    dplyr::left_join(antigen_QC_only, by = c("SampleID", "Location", "Plate"))

  #############################################################################
  # Re-arrange data
  #############################################################################

  # Get all base marker names by stripping _Count
  marker_bases <- names(joined_antigen_counts) %>%
    grep("_Count$", ., value = TRUE) %>%
    sub("_Count$", "", .)

  # Create the desired column order
  new_order <- c(
    "Location", "SampleID", "Plate",
    unlist(lapply(marker_bases, function(x) c(paste0(x, "_Count"), paste0(x, "_QC"))))
  )

  # Reordered data frame
  joined_antigen_counts <- joined_antigen_counts %>%
    dplyr::select(all_of(new_order))

  #############################################################################
  # Add total counts
  #############################################################################

  total_counts_only <- counts_output %>%
    dplyr::select(Location, Plate, QC_total)

  total_counts_final_output <- joined_antigen_counts %>%
    dplyr::left_join(total_counts_only, by = c("Location", "Plate"))

  return(total_counts_final_output)

}

#' Check Beads to Repeat
#'
#' This function gets the count data and outputs a table of the isolates to
#' repeat or a statement to confirm that none need to be repeated.
#'
#' @param qc_results Output from `runQC()`.
#' @param plate_list Output from `readPlateLayout()`.
#' @return A data frame with wells to "fail", OR if no "fail" found will return
#' text "No repeats necessary".
#' @export
#' @importFrom tidyr drop_na
#' @importFrom dplyr left_join select distinct filter
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Step 0: Load example raw data and plate layout
#' raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#' plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")
#'
#' # Step 1: Read data and plate layout
#' sero_data   <- readSeroData(raw_data, platform = "magpix")
#' plate_list  <- readPlateLayout(plate_layout, sero_data)
#'
#' # Step 2: Process counts
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Step 3: Identify samples to repeat
#' repeats_table <- getRepeats(
#'   qc_results = qc_results,
#'   plate_list = plate_list
#' )
#'
#' # View results
#' repeats_table
#' }
getRepeats <- function(qc_results, plate_list) {

  counts_output     <- qc_results$getCounts_output
  sample_id_output  <- qc_results$sampleid_output

  # 1. Filter "Repeats" in Counts Output
  repeats <- counts_output %>% dplyr::filter(QC_total == "fail")
  # 2. If zero "Repeats" found, then write text. If "Repeats" found, then output table.
  if (nrow(repeats) == 0) {
    return("No repeats necessary.")
  } else {
    table <- sample_id_output %>% dplyr::distinct(SampleID, Location, Plate)
    table <- table %>%
      dplyr::left_join(repeats, by = c("Location", "Plate")) %>%
      tidyr::drop_na() %>%
      dplyr::select(Location, SampleID, Plate, QC = QC_total)
    return(table)
  }
}

#' Plot Classification
#'
#' One example of data visualisation to detect the median and interquartile
#' range of the RAU values per antigen for seropositive and seronegative
#' individuals. Please note that the `classifyResults()` function must
#' be run first.
#'
#' @param all_classifications Data frame of `classifyResults()` for all
#' sens_spec thresholds.
#' @param selected_threshold String with the threshold.
#' @return Box plots with RAU values for each protein stratified by
#' classification (ggplot).
#' @export
#'
#' @import ggplot2
#' @importFrom dplyr filter mutate
#' @importFrom tidyr pivot_longer
#' @importFrom purrr map_dfr
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU using ETH beads
#' mfi_to_rau <- MFItoRAU_Adj(
#'   sero_data    = sero_data,
#'   plate_list   = plate_list,
#'   qc_results   = qc_results
#' )
#'
#' # Step 4: Define sens/spec thresholds
#' sens_spec_all <- c(
#'   "balanced", "90% specificity"
#' )
#'
#' # Step 5: Classify results across all thresholds
#' all_classifications <- purrr::map_dfr(sens_spec_all, ~{
#'   classifyResults(
#'     mfi_to_rau_output = mfi_to_rau,
#'     algorithm_type = "antibody_model",
#'     sens_spec = .x,
#'     qc_results = qc_results
#'   ) |>
#'   as.data.frame() |>
#'   dplyr::mutate(sens_spec = .x)
#' })
#'
#' # Plot classification for a single threshold
#' plotBoxPlotClassification(all_classifications, "balanced")
#' }
plotBoxPlotClassification <- function(all_classifications, selected_threshold){

  all_classifications %>%
    dplyr::filter(sens_spec == selected_threshold) %>%
    tidyr::pivot_longer(
      -c(SampleID, Plate, QC_total, pred_class_max, sens_spec),
      names_to = "Antigen",
      values_to = "RAU"
    ) %>%
    dplyr::mutate(
      pred_class_max = factor(pred_class_max, levels = c("seronegative", "seropositive"))
    ) %>%
    ggplot2::ggplot(aes(x = pred_class_max, y = RAU, fill = pred_class_max)) +
    ggplot2::geom_boxplot() +
    ggplot2::scale_y_log10() +
    ggplot2::scale_fill_manual(values = c(seronegative = "#878787", seropositive = "#d6604d")) +
    ggplot2::labs(
      title = paste0("Threshold Chosen: ", as.character(selected_threshold)),
      x = "Classification",
      y = "RAU",
      fill = "Classification"
    ) +
    ggplot2::facet_grid(~Antigen) +
    ggplot2::theme_bw() +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))

}

#' Processing Serological Data for Pk/Pf/Pv MFI to RAU conversion
#'
#' This is a pre-requisite function before running the `MFItoRAU_Plasmo()` so that the
#' appropriate MFI to RAU conversions can be run for the respective antigens.
#'
#' @param sero_data   Output of `readSeroData()`
#' @param plate_list  Output of `readPlateLayout()`
#' @param panel Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.
#'
#' @return A list of two data frames:
#' 1. Data frame with Pk antigens
#' 2. Data frame with Pf/Pv antigens
#' @export
#'
#' @importFrom dplyr mutate across  select  pull filter case_when
#' @importFrom stringr str_extract  str_detect  str_detect
#' @importFrom utils read.csv
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example demonstrating multi-plate 5-standard processing workflow.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data_5std <- c(
#'   system.file("extdata", "example_MAGPIX_pk_5std_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_pk_5std_plate2.csv", package = "SeroTrackR")
#' )
#'
#' your_plate_layout_5std <- system.file(
#'   "extdata", "example_platelayout_pk_5std.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data_5std,
#'   platform = "magpix"
#' )
#'
#' # Read matching plate layout
#' plate_list <- readPlateLayout(
#'   plate_layout = your_plate_layout_5std,
#'   sero_data = sero_data
#' )
#'
#' # Process multi-species panel
#' processed_master <- processPkPfPv(
#'   sero_data = sero_data,
#'   plate_list = plate_list,
#'   panel = "panel1"
#' )
#' }
processPkPfPv <- function(sero_data, plate_list, panel = "panel1"){

  # Check if shiny.fluent is installed
  if (!requireNamespace("zoo", quietly = TRUE)) {
    stop("Package 'zoo' is required for processPkPfPv(). Please install it.", call. = FALSE)
  }

  #############################################################
  # Step 1: Collect Data Inputs for Function
  #############################################################
  master_file               <- sero_data$results
  layout                    <- plate_list
  chosen_panel              <- panel

  #############################################################
  # Interim Pre-Processing Step for Pk project
  #############################################################
  L                         <- master_file %>%
    dplyr::mutate(
      dplyr::across(-c(Location, Sample, Plate), as.numeric),
      # Extract suffix (anything inside parentheses)
      suffix = stringr::str_extract(Sample, "\\s*\\([^\\)]+\\)"),
      # Carry forward last seen suffix
      suffix = zoo::na.locf(suffix, na.rm = FALSE),
      # Add suffix only to S-samples that don't already have one
      Sample = ifelse(stringr::str_detect(Sample, "^S\\d+$"), paste0(Sample, suffix), Sample)
    ) %>%
    dplyr::select(-suffix)

  #############################################################
  # Step 2: Filter for Pv/Pf and Pv Datasets
  #############################################################

  # Extract Reference IDs for Antigens
  if(panel == "panel1"){
    panel <-read.csv(system.file("extdata", "PkPfPv_Panel_1.csv", package = "SeroTrackR"))
    pv_antigens <- panel %>% dplyr::filter(Species == "Pv") %>% dplyr::pull(Antigens)
    pf_antigens <- panel %>% dplyr::filter(Species == "Pf") %>% dplyr::pull(Antigens)
    pk_antigens <- panel %>% dplyr::filter(Species == "Pk") %>% dplyr::pull(Antigens)
  } else {
    panel <- read.csv(panel)
    pv_antigens <- panel %>% dplyr::filter(Species == "Pv") %>% dplyr::pull(Antigens)
    pf_antigens <- panel %>% dplyr::filter(Species == "Pf") %>% dplyr::pull(Antigens)
    pk_antigens <- panel %>% dplyr::filter(Species == "Pk") %>% dplyr::pull(Antigens)
  }

  PfPv <- L %>%
    # Step 1: Filter for only Pv/Pf-relevant standard curve
    dplyr::filter(!str_detect(Sample, "PK")) %>%
    # Step 2: Filter for only Pv/Pf-relevant antigens
    dplyr::select(Location, Sample, Plate, all_of(pv_antigens), all_of(pf_antigens))

  Pk <- L %>%
    # Step 1: Filter for only Pk-relevant standard curve
    dplyr::filter(!str_detect(Sample, "ETH|PNG|Global")) %>%
    # Step 2: Filter for only Pk-relevant antigens
    dplyr::select(Location, Sample, Plate, all_of(pk_antigens))

  return(list(PfPv = PfPv,  Pk = Pk))

}

#' Render Markdown report
#'
#' A short function to render the rmarkdown report on Shiny.
#'
#' @param input Input files
#' @param output Output files
#' @param params Parameters to generate outputs.
#' @return PDF output.
#' @export
#' @importFrom rmarkdown render
#' @author Dionne Argyropoulos
#'
#' @examples
#' # Minimal example that renders a temporary Rmd file.
#' # Safe for CRAN because it only writes to tempdir()
#' \dontrun{
#' if (requireNamespace("rmarkdown", quietly = TRUE) &&
#'     rmarkdown::pandoc_available()) {
#'
#'   # Create a temporary Rmd that declares params in the YAML
#'   rmd_file <- tempfile(fileext = ".Rmd")
#'   writeLines(c(
#'     "---",
#'     "title: \"Test Report\"",
#'     "output: html_document",
#'     "params:",
#'     "  value: 0",
#'     "---",
#'     "",
#'     "This is a test report.",
#'     "",
#'     "Parameter value: `r params$value`"
#'   ), con = rmd_file)
#'
#'   # Output location
#'   out_file <- tempfile(fileext = ".html")
#'
#'   # Example parameters to pass in
#'   example_params <- list(value = 123)
#'
#'   # Render report
#'   renderReport(
#'     input  = rmd_file,
#'     output = out_file,
#'     params = example_params
#'   )
#'
#'   # Optionally inspect the output path
#'   out_file
#' }
#' }
renderReport <- function(input, output, params) {

  # Turn on debugging
  Sys.setenv(TINYTEX_DEBUG = "1")

  # Render without cleaning (keeps .log file)
  rmarkdown::render(
    input = input,
    output_file = output,
    params = params,
    envir = new.env(parent = globalenv()),
    clean = FALSE
  )

  # Locate the .log file
  log_file <- sub("\\.pdf$", ".log", output)

  # If not found, try .tex → .log
  if (!file.exists(log_file)) {
    tex_file <- sub("\\.pdf$", ".tex", output)
    log_file <- sub("\\.tex$", ".log", tex_file)
  }

  # Print log into Shiny logs
  if (file.exists(log_file)) {
    cat("\n===== LATEX LOG START =====\n")
    cat(readLines(log_file), sep = "\n")
    cat("\n===== LATEX LOG END =====\n")
  } else {
    cat("No LaTeX log file found.\n")
  }
}

#' Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' conversion
#'
#' This function fits a 5-parameter logistic standard curve to the dilutions
#' of the positive controls for each protein and converts the MFI values
#' into relative antibody units (RAU) written by Connie Li Wai Suen.
#'
#' @param sero_data Output from `readSeroData()`.
#' @param plate_list Output from `readPlateLayout()`.
#' @param qc_results Output from `runQC()`.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#' @param project Default = NULL. Only write "pkpfpv" if using Pk/Pf/Pv pipeline.
#'
#' @return  A list of three data frames:
#' 1. Data frame with  MFI data, converted RAU data and matched SampleID's.
#' 2. Plot information for `plotModel` function
#' 3. Data frame of RAU data for random forest classification use.
#'
#' @export
#'
#' @import drc
#' @importFrom dplyr distinct select inner_join bind_rows
#' @importFrom tidyselect matches
#' @importFrom purrr imap_dfr
#' @author Dionne Argyropoulos, Connie Li Wai Suen
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU
#' mfi_to_rau <- MFItoRAU(
#'   sero_data = sero_data,
#'   plate_list = plate_list,
#'   qc_results = qc_results
#' )
#'
#' }
MFItoRAU <- function(
    sero_data,
    plate_list,
    qc_results,
    std_point = 10,
    project = NULL
){

  if (is.null(project)) {
    df <- sero_data$results
  } else if (project == "pkpfpv") {
    df <- sero_data
  } else {
    df <- sero_data$results
  }

  setup <- .setup_mfitorau_inputs(
    df = df,
    plate_list = plate_list,
    std_point = std_point
  )

  L        <- setup$L
  layout   <- setup$layout
  antigens <- setup$antigens

  # unpack params
  dilution                      <- setup$params$dilution
  dilution_scaled               <- setup$params$dilution_scaled
  dilution_factor               <- setup$params$dilution_factor
  current_min_relative_dilution <- setup$params$current_min_relative_dilution
  s1_concentration              <- setup$params$s1_concentration
  s_final_concentration         <- setup$params$s_final_concentration

  counts_QC_output              <- qc_results$getCountsQC_output

  # LOG-LOG MODEL
  # Iterate over each level in L$Plate and corresponding layout data frame
  results_all <- list()  # To store results for all plates
  model_results_all <- list()  # To store model results for all plates
  MFI_RAU_results_all <- list() # To store MFI to RAU conversion results for all plates

  for (plate_idx in seq_along(unique(L$Plate))) {
    plate_level <- unique(L$Plate)[plate_idx]
    subset_data <- L[L$Plate == plate_level, ]

    # Fetch the corresponding layout data frame
    current_layout <- layout[[plate_level]]

    # Initialize storage for results
    results.df.wide <- NULL
    model_list <- list()

    # Iterate over antigens
    for (i in antigens) {

      out <- .process_antigen_loglog(
        subset_data            = subset_data,
        antigen                = i,
        dilution               = dilution,
        s1_concentration       = s1_concentration,
        s_final_concentration  = s_final_concentration
      )

      model_list[[i]] <- out$model

      if (is.null(results.df.wide)) {
        results.df.wide <- out$results
      } else {
        results.df.wide <- merge(
          results.df.wide,
          out$results,
          by = c("Location", "Sample", "Plate")
        )
      }
    }

    # MODEL RESULTS AND PLOTS
    # # Plot models with plate in the title
    # model_results <- list()
    # for (i in names(model_list)) {
    #   title <- paste("Plate:", plate_level, "- Protein:", i)  # Combine plate and protein name
    #   model_results[[i]] <- plot(model_list[[i]], main = title)
    # }

    model_results <- list()

    for (i in names(model_list)) {

      model   <- model_list[[i]]
      rng     <- range(model$data$dilution, na.rm = TRUE)
      newx    <- exp(seq(log(rng[1]), log(rng[2]), length.out = 100))
      pred    <- suppressWarnings(as.numeric(predict(model, newdata = data.frame(dilution = newx))))

      model_results[[i]] <- data.frame(
        dilution = newx,
        log.std = pred
      )
    }

    # MERGE DATA
    results.df.wide <- .merge_mfitorau(
      df = results.df.wide,
      layout = layout,
      plate_level = plate_level
    )

    # OUTPUT
    # Save just MFI and RAU for downstream analyses
    col_selection <- grepl("SampleID|Plate|_MFI|\\_Dilution$", colnames(results.df.wide))
    MFI_RAU_results <- results.df.wide[, col_selection]

    # Store results and models for current plate: `results_all` and `model_results_all` store all results and model plots for each plate.
    results_all[[plate_level]] <- results.df.wide
    model_results_all[[plate_level]] <- model_results
    MFI_RAU_results_all[[plate_level]] <- MFI_RAU_results
  }

  # Return the final results tables with QC pass/fail
  counts_data <- counts_QC_output %>%
    ungroup() %>%
    dplyr::select(SampleID, Location.2 = Location, Plate, QC_total)

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")
  name_lookup <- setNames(new_names, old_names)

  # Join all results together and relabel antigen names for export
  final_results <- dplyr::bind_rows(results_all) %>%
    dplyr::inner_join(counts_data, by = c("SampleID", "Plate", "Location.2")) %>%
    dplyr::rename_with(function(col_names) {
      for (old in names(name_lookup)) {
        col_names <- str_replace(
          col_names,
          paste0("^", old, "(?=_)"),  # match only prefix before underscore
          name_lookup[[old]]
        )
      }
      col_names
    })

  final_MFI_RAU_results <- dplyr::bind_rows(MFI_RAU_results_all) %>%
    dplyr::inner_join(counts_data, by = c("SampleID", "Plate")) %>%
    dplyr::rename_with(function(col_names) {
      for (old in names(name_lookup)) {
        col_names <- str_replace(
          col_names,
          paste0("^", old, "(?=_)"),  # match only prefix before underscore
          name_lookup[[old]]
        )
      }
      col_names
    })

  final_model_results_all <- purrr::imap_dfr(
    model_results_all,
    ~ purrr::imap_dfr(.x, ~ dplyr::mutate(.x, Antigen = .y), .id = "Antigen"),
    .id = "Plate"
  ) %>%
  dplyr::mutate(Antigen = dplyr::recode(Antigen, !!!name_lookup))

  # Output
  return(
    list(
      final_results,
      final_MFI_RAU_results,
      final_model_results_all
    )
  )

}
#' Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' conversion based on other standard
#'
#' This function fits a 5-parameter logistic standard curve to the dilutions
#' of the positive controls for each protein and converts the MFI values
#' into relative antibody units (RAU) written by Eamon Conway.
#'
#' @param sero_data Output from `readSeroData()`.
#' @param plate_list Output from `readPlateLayout()`.
#' @param qc_results Output from `runQC()`.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#' @param project Default = NULL. Only write "pkpfpv" if using Pk/Pf/Pv pipeline.
#'
#' @return  A list of three data frames:
#' 1. Data frame with  MFI data, converted RAU data and matched SampleID's.
#' 2. Plot information for `plotModel` function.
#' 3. Data frame of RAU data for random forest classification use.
#'
#' @export
#'
#' @importFrom dplyr group_by mutate across inner_join rowwise summarise right_join select left_join rename_with all_of ungroup
#' @importFrom tidyr nest unnest pivot_wider
#' @importFrom tidyselect matches
#' @importFrom purrr map
#' @author Eamon Conway, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU using ETH beads
#' mfi_to_rau <- MFItoRAU_Adj(
#'   sero_data    = sero_data,
#'   plate_list   = plate_list,
#'   qc_results   = qc_results
#' )
#'
#' }
MFItoRAU_Adj <- function(
    sero_data,
    plate_list,
    qc_results,
    std_point = 10,
    project = NULL
){

  if (is.null(project)) {
    df <- sero_data$results
  } else if (project == "pkpfpv") {
    df <- sero_data
  } else {
    df <- sero_data$results
  }

  setup <- .setup_mfitorau_inputs(
    df = df,
    plate_list = plate_list,
    std_point = std_point
  )

  L        <- setup$L
  layout   <- setup$layout
  antigens <- setup$antigens

  # unpack params
  dilution                      <- setup$params$dilution
  dilution_scaled               <- setup$params$dilution_scaled
  dilution_factor               <- setup$params$dilution_factor
  current_min_relative_dilution <- setup$params$current_min_relative_dilution
  s1_concentration              <- setup$params$s1_concentration
  s_final_concentration         <- setup$params$s_final_concentration

  counts_QC_output              <- qc_results$getCountsQC_output

  # Reference Fit
  refs <-read.csv(system.file("extdata", "png_eth_stds.csv", package = "SeroTrackR"))

  control = list(maxit = 10000, abstol = 1e-8, reltol = 1e-6)
  initial_solution = c(-1.0, 0.0, 10, 0.0, 0.0)

  ref_fit <- refs %>%
    dplyr::group_by(.data$std_plate, .data$antigen) %>%
    tidyr::nest()  %>%
    dplyr::mutate(
      .keep = "none",
      eth_fit = purrr::map(data, ~ {
        suppressMessages(
          suppressWarnings(
            .fit_standard_curve(.x$eth_mfi, .x$dilution, control)
          )
        )
      }),
      png_fit = purrr::map(data, ~ {
        suppressMessages(
          suppressWarnings(
            .fit_standard_curve(.x$png_mfi, .x$dilution, control)
          )
        )
      })
    )

  reference_antigens = unique(ref_fit$antigen)

  # Initialise outputs and prepare function by plate
  # Iterate over each level in L$Plate and corresponding layout data frame
  results_all <- list()  # To store results for all plates
  model_results_all <- list()  # To store model results for all plates
  MFI_RAU_results_all <- list() # To store MFI to RAU conversion results for all plates

  for (plate_idx in seq_along(unique(L$Plate))) {
    plate_level <- unique(L$Plate)[plate_idx]
    subset_data <- L[L$Plate == plate_level, ]

    # Apply conversion
    eth_qa_sc <- subset_data %>%
      dplyr::filter(type.letter == "S") %>%
      tidyr::pivot_longer(-c(Sample, Location, Plate, type.letter), names_to = "antigen", values_to = "mfi") %>%
      dplyr::mutate(dilution = dilution_factor ^ (-as.numeric(gsub(
        "\\D", "", .data$`Sample`
      )) + 1))  %>%
      dplyr::group_by(.data$antigen) %>%
      tidyr::nest()

    eth_qa_mfi <- subset_data %>%
      dplyr::filter(type.letter == "U" | type.letter == "X") %>%
      tidyr::pivot_longer(-c(Sample, Location, Plate, type.letter), names_to = "antigen", values_to = "mfi") %>%
      dplyr::group_by(.data$antigen) %>%
      tidyr::nest()

    qa_fit <- eth_qa_sc %>%
      dplyr::mutate(.keep = "none", new_fit = purrr::map(data, ~ {
        .fit_standard_curve(.x$mfi, .x$dilution, control)
      }))

    # We have the fit for each antigen.
    eth_converted = dplyr::inner_join(ref_fit, qa_fit) %>%
      dplyr::inner_join(eth_qa_mfi) %>%
      dplyr::rowwise() %>%
      dplyr::mutate(.keep = "none", data = list(
        data |> dplyr::mutate(
          .keep = "none",
          mfi = .data$mfi,
          Sample = .data$Sample,
          dilution = .convert_mfi_to_dilution_no_bounds(mfi,new_fit, 0.0), # We do not want the initial conversion to have any bounds. There are some required due to asymptotes in the function however. (Eamon)
          ref_mfi = .convert_dilution_to_mfi(dilution,eth_fit),
          dilution = .convert_mfi_to_dilution(ref_mfi,png_fit, current_min_relative_dilution)
        )
      )) %>%
      tidyr::unnest(cols = data)

    # Take MEAN of these 10 repeats
    estimate_eth <- eth_converted %>%
      dplyr::group_by(antigen, Sample) %>%
      dplyr::summarise(
        dilution = mean(dilution) * s1_concentration,
        mfi = mean(mfi)
      )

    # MODEL RESULTS AND PLOTS
    sc_fit <- eth_qa_sc %>%
      dplyr::mutate(.keep = "none", new_fit = purrr::map(data, ~ {
        suppressMessages(
          suppressWarnings(
            .fit_standard_curve(.x$mfi, .x$dilution, control)
          )
        )
      }))

    qa_converted <- dplyr::inner_join(sc_fit, eth_qa_sc) |>
      dplyr::rowwise() |>
      dplyr::mutate(.keep = "none", data = list(
        data |> dplyr::mutate(
          .keep = "none",
          Sample = .data$Sample,
          dilution = .data$dilution,
          mfi = .data$mfi,
          mfi_pred = .convert_dilution_to_mfi(.data$dilution, new_fit)
        )
      )) |>
      tidyr::unnest(cols = data)

    model_results <- qa_converted

    # MERGE DATA: Relabel Sample Names with Plate Layout
    # Bind plate-subset data with RAU-converted data
    eth_converted_locations <- subset_data %>%
      dplyr::select(Location, Sample, Plate) %>%
      dplyr::right_join(estimate_eth, by = "Sample")

    # Pivot wider
    eth_converted_wide.1 <- eth_converted_locations %>%
      # Pivot wider: All MFI values
      dplyr::select(-dilution) %>%
      tidyr::pivot_wider(names_from = "antigen", values_from = "mfi") %>%
      dplyr::rename_with(~paste0(.x, "_MFI"), -c(Location, Sample, Plate))

    eth_converted_wide.2 <- eth_converted_locations %>%
      # Pivot wider: All Dilution values
      dplyr::select(-mfi) %>%
      tidyr::pivot_wider(names_from = "antigen", values_from = "dilution") %>%
      dplyr::rename_with(~paste0(.x, "_Dilution"), -c(Location, Sample, Plate))

    eth_converted_wide <- dplyr::left_join(
      eth_converted_wide.1,
      eth_converted_wide.2,
      by = c("Location", "Sample", "Plate")
    )

    eth_converted_wide <- .merge_mfitorau(
      df = eth_converted_wide,
      layout = layout,
      plate_level = plate_level
    )

    # Create output dataframes
    # Save just MFI and RAU for downstream analyses
    col_selection <- grepl("SampleID|Location.2|Plate|_MFI|\\_Dilution$", colnames(eth_converted_wide))
    MFI_RAU_results <- eth_converted_wide[, col_selection]

    # Store results and models for current plate: `results_all` and `model_results_all` store all results and model plots for each plate.
    results_all[[plate_level]] <- eth_converted_wide
    model_results_all[[plate_level]] <- model_results
    MFI_RAU_results_all[[plate_level]] <- MFI_RAU_results

  }

  # Joining all plate data
  counts_data <- counts_QC_output %>%
    dplyr::ungroup() %>%
    dplyr::select(SampleID, Location.2 = Location, Plate, QC_total)

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")
  name_lookup <- setNames(new_names, old_names)

  final_results <- dplyr::bind_rows(results_all) %>%
    dplyr::inner_join(counts_data, by = c("SampleID", "Location.2", "Plate"))

  final_model_results_all <- dplyr::bind_rows(model_results_all, .id = "Plate") %>%
    dplyr::mutate(antigen = dplyr::recode(antigen, !!!name_lookup))

  final_MFI_RAU_results <- dplyr::bind_rows(MFI_RAU_results_all) %>%
    dplyr::inner_join(counts_data, by = c("SampleID", "Location.2", "Plate"))

  # Re-arrange data for final outputs
  # Get all base marker names by stripping _Count
  marker_bases <- names(final_results) %>%
    grep("_MFI$", ., value = TRUE) %>%
    sub("_MFI$", "", .)

  # Create the desired column order
  final_results_order <- c(
    "SampleID", "Location.2", "Location", "Sample", "Plate", "QC_total",
    unlist(lapply(marker_bases, function(x) c(paste0(x, "_MFI"), paste0(x, "_Dilution"))))
  )
  final_MFI_RAU_order <- c(
    "SampleID", "Plate", "QC_total",
    unlist(lapply(marker_bases, function(x) c(paste0(x, "_MFI"), paste0(x, "_Dilution"))))
  )

  # Reordered data frame
  final_results <- final_results %>%
    dplyr::select(all_of(final_results_order)) %>%
    dplyr::rename_with(function(col_names) {
      for (old in names(name_lookup)) {
        col_names <- str_replace(
          col_names,
          paste0("^", old, "(?=_)"),  # match only prefix before underscore
          name_lookup[[old]]
        )
      }
      col_names
    })

  final_MFI_RAU_results <- final_MFI_RAU_results %>%
    dplyr::select(all_of(final_MFI_RAU_order)) %>%
    dplyr::rename_with(function(col_names) {
      for (old in names(name_lookup)) {
        col_names <- str_replace(
          col_names,
          paste0("^", old, "(?=_)"),  # match only prefix before underscore
          name_lookup[[old]]
        )
      }
      col_names
    })

  return(
    list(
      final_results,
      final_MFI_RAU_results,
      final_model_results_all
    )
  )
}
#' Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' conversion for LDH
#'
#' This function fits a 5-parameter logistic standard curve to the dilutions
#' of the positive controls for each protein and converts the MFI values
#' into relative antibody units (RAU).
#'
#' @param sero_data Output from `readSeroData()` or `readSeroData()`.
#' @param plate_list Output from `readPlateLayout()`.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = "PvLDH".
#' @param file_path A file path to write the .csv final file. Default: Current working directory.
#'
#' @return A data frame containing the MFI and RAU Dilution values for each sample
#' @export
#'
#' @import drc
#' @import dplyr
#' @importFrom tidyr pivot_wider  drop_na
#' @importFrom stringr str_split  str_sub str_remove
#' @importFrom purrr map  map_chr
#' @importFrom here here
#' @importFrom utils write.csv
#'
#' @author Connie Li Wai Suen, Caitlin Bourke, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example demonstrating multi-plate processing workflow.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data <- system.file("extdata", "example_BioPlex_PvLDH_plate1.xlsx", package = "SeroTrackR")
#'
#' your_plate_layout <- system.file(
#'   "extdata", "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Read in raw BioPlex data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "bioplex"
#' )
#'
#' # Read matching plate layout
#' plate_list <- readPlateLayout(
#'   plate_layout = your_plate_layout,
#'   sero_data = sero_data
#' )
#'
#' # Run MFI to RAU conversion
#' mfi_outputs <- MFItoRAU_LDH(
#'   sero_data = sero_data,
#'   plate_list = plate_list
#' )
#'
#' # View All Outputs
#' mfi_outputs
#' }
MFItoRAU_LDH <- function(
    sero_data,
    plate_list,
    std_point = "PvLDH",
    file_path = NULL
){

  # Step 1: Read raw serology data and plate layout
  setup <- .setup_mfitorau_inputs(
    df = sero_data$results,
    plate_list = plate_list,
    std_point = "PvLDH"
  )

  L        <- setup$L
  layout   <- setup$layout
  antigens <- setup$antigens

  # unpack params
  dilution                      <- setup$params$dilution
  s1_concentration              <- setup$params$s1_concentration
  s_final_concentration         <- setup$params$s_final_concentration

  results.df.wide <- NULL

  # Step 2: Perform LOG-LOG MODEL
  # Iterate over each level in L$Plate and corresponding layout data frame
  results_all <- list()  # To store results for all plates

  for (plate_idx in seq_along(unique(L$Plate))) {
    plate_level <- unique(L$Plate)[plate_idx]
    subset_data <- L[L$Plate == plate_level, ]

    # Fetch the corresponding layout data frame
    current_layout <- layout[[plate_level]]

    results.df.wide <- NULL
    model_list <- list()

    for (i in antigens) {

      out <- .process_antigen_loglog(
        subset_data           = subset_data,
        antigen               = i,
        dilution              = dilution,
        s1_concentration      = s1_concentration,
        s_final_concentration = s_final_concentration,
        unknown_letters       = c("U", "X")
      )

      model_list[[i]] <- out$model

      if (is.null(results.df.wide)) {
        results.df.wide <- out$results
      } else {
        results.df.wide <- merge(
          results.df.wide,
          out$results,
          by = c("Location", "Sample", "Plate")
        )
      }
    }

    # STEP 4: MERGE DATA with plate layout
    # Create "Location.2" Variable which has the row and column ID only
    results.df.wide <- results.df.wide %>%
      as.data.frame() %>%
      dplyr::mutate(
        Location.2 = Location %>%
          as.character() %>%
          str_split(",") %>%                  # split on ","
          map_chr(~ .x[2]) %>%                # take the 2nd element
          str_sub(1, -2)                      # drop the last character
      ) %>%
      dplyr::relocate(Location.2, .before = 1)       # put Location.2 in front

    # Create plate layout to bind to "results.df.wide"
    # 1. Parse L$Location into well IDs
    location.2 <- dplyr::tibble(Location = subset_data$Location) %>%
      dplyr::mutate(
        Location.2 = str_split(Location, ",", simplify = TRUE)[,2] %>%
          str_sub(1, -2),
        alpha   = str_remove(Location.2, "[0-9]+"),
        numeric = str_remove(Location.2, "[^0-9]")
      )
    # 2. Get current plate layout, standardise colnames
    plate_layout_current <- layout[[plate_level]] %>% dplyr::rename(Plate = 1)  # relabel first column to "Plate"
    # 3. Reshape layout to long form (row = Plate, column = numeric, value = SampleID)
    plate_layout_long <- plate_layout_current %>%
      tidyr::pivot_longer(
        -Plate,
        names_to = "numeric",
        values_to = "SampleID"
      )
    # 4. Join Location info with plate layout to attach SampleIDs
    row_to_match <- location.2 %>%
      dplyr::left_join(plate_layout_long, by = c("alpha" = "Plate", "numeric" = "numeric")) %>%
      dplyr::select(Location.2, SampleID) %>%
      dplyr::distinct(SampleID, Location.2, .keep_all = TRUE) %>%
      tidyr::drop_na()

    # Using left_join() to add SampleID information to results.df.wide
    # Define column names to remain as characters
    character_columns <- c("SampleID", "Location", "Location.2", "Sample", "Plate")
    # Join
    results.df.wide <- results.df.wide %>%
      dplyr::left_join(row_to_match, by = "Location.2") %>%
      dplyr::select(SampleID, everything()) %>%                               # Move SampleID to first column
      dplyr::mutate(across(all_of(character_columns), as.character)) %>%      # Convert specified columns to character
      dplyr::mutate(mutate(across(-all_of(character_columns), as.numeric)))   # Convert all other columns to numeric

    # Store results and models for current plate: `results_all` and `model_results_all` store all results and model plots for each plate.
    results_all[[plate_level]] <- results.df.wide
  }

  final_results <- dplyr::bind_rows(results_all)

  # Return final file
  return(final_results)

}
#' Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' conversion for Pk/Pf/Pv Master Function
#'
#' This function leverages `MFItoRAU_Pk()` and `MFItoRAU()` to create a final MFI to RAU
#' output for Pk/Pf/Pv analyses.
#'
#' @param sero_data   Output of `readserodata_output()`
#' @param plate_list  Output of `readPlateLayout()`
#' @param panel Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#' @param qc_results Output from `runQC()`.
#'
#' @return A list of three data frames:
#' 1. Data frame with MFI data, converted RAU data, matched SampleID's, all intermediate dilution conversion factors
#' 2. Data frame with only SampleID's, MFI and RAU data
#' 3. Data frame #2 in long-format
#'
#' @importFrom dplyr select rename_with left_join ends_with right_join
#' @importFrom tidyr pivot_longer separate
#' @importFrom stringr str_replace
#' @importFrom utils read.csv
#'
#' @export
#' @author Dionne Argyropoulos, Caitlin Bourke
#'
#' @examples
#' \donttest{
#' # Example demonstrating multi-plate 5-standard processing workflow.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data_5std <- c(
#'   system.file("extdata", "example_MAGPIX_pk_5std_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_pk_5std_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout_5std <- system.file(
#'   "extdata", "example_platelayout_pk_5std.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data_5std,
#'   platform = "magpix"
#' )
#'
#' # Read matching plate layout
#' plate_list <- readPlateLayout(
#'   plate_layout = your_plate_layout_5std,
#'   sero_data = sero_data
#' )
#' # Quality control
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Run MFI to RAU conversion
#' mfi_outputs               <- MFItoRAU_Plasmo(
#'   sero_data = sero_data,
#'   plate_list = plate_list,
#'   panel = "panel1",
#'   std_point = 5,
#'   qc_results = qc_results
#' )
#'
#' # View All Outputs
#' mfi_outputs
#' }
MFItoRAU_Plasmo <- function(
    sero_data,
    plate_list,
    panel = "panel1",
    std_point,
    qc_results
  ){

  processed_master    <- processPkPfPv(sero_data, plate_list, panel = panel)
  processed_PfPv      <- processed_master$PfPv
  processed_Pk        <- processed_master$Pk

  counts_QC_output    <- qc_results$getCountsQC_output

  # Pfk MFI to RAU processing pipeline
  Pk_Final            <- MFItoRAU_Pk(
      processed_Pk = processed_Pk,
      plate_list = plate_list,
      std_point = std_point,
      qc_results = qc_results
    )

  # Pf/Pv MFI to RAU processing pipeline
  PfPv_Final          <- suppressWarnings(
    MFItoRAU(
      sero_data = processed_PfPv,
      plate_list = plate_list,
      project = "pkpfpv",
      std_point = std_point,
      qc_results = qc_results
    )
  )
  PfPv_Adj_Final      <- suppressMessages(
    MFItoRAU_Adj(
      sero_data = processed_PfPv,
      plate_list = plate_list,
      project = "pkpfpv",
      std_point = std_point,
      qc_results = qc_results
    )
  )

  # Relabel _Dilution to specify
  PfPv_Final[[1]] <- PfPv_Final[[1]] %>% rename_with(~ gsub("_Dilution$", "_loglog_Dilution", .x))
  PfPv_Final[[2]] <- PfPv_Final[[2]] %>% rename_with(~ gsub("_Dilution$", "_loglog_Dilution", .x))

  PfPv_Adj_Final[[1]] <- PfPv_Adj_Final[[1]] %>% rename_with(~ gsub("_Dilution$", "_Adjloglog_Dilution", .x)) %>%
    dplyr::mutate(across(contains("Adjloglog"), ~if_else(.x<1.95e-05, 1.95e-05, .x))) # catch any values below 1.95e-05 (S11 min)
  PfPv_Adj_Final[[2]] <- PfPv_Adj_Final[[2]] %>% rename_with(~ gsub("_Dilution$", "_Adjloglog_Dilution", .x)) %>%
    dplyr::mutate(across(contains("Adjloglog"), ~if_else(.x<1.95e-05, 1.95e-05, .x)))# catch any values below 1.95e-05 (S11 min)


  # Join Dataframes Together
  pk_final_results            <- Pk_Final
  pfpv_final_results          <- PfPv_Final[[1]]
  pfpv_Adj_final_results      <- PfPv_Adj_Final[[1]]

  PkPfPv_Final <- suppressMessages(
    pk_final_results %>%
      left_join(pfpv_final_results, by = c("SampleID", "Location.2", "Location", "Sample", "Plate", "QC_total")) %>%
      left_join(pfpv_Adj_final_results))
  PkPfPv_Final_MFI_RAU <- PkPfPv_Final %>%
    dplyr::select(SampleID, Plate, ends_with("_MFI", ignore.case = FALSE), ends_with("_Dilution", ignore.case = FALSE))

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")
  name_lookup <- setNames(new_names, old_names)

  names(pfpv_Adj_final_results) <- vapply(
    names(pfpv_Adj_final_results),
    function(col) {
      for (old in names(name_lookup)) {
        col <- sub(paste0("^", old), name_lookup[[old]], col)
      }
      col
    },
    character(1)
  )

  # Add panel
  if(panel == "panel1"){
    panel <-read.csv(system.file("extdata", "PkPfPv_Panel_1.csv", package = "SeroTrackR"))
    panel <- panel %>%
      dplyr::mutate(Antigens = dplyr::recode(Antigens, !!!name_lookup))
  } else {
    panel <- read.csv(panel)
  }

  # Create long df for downstream analyses (clean)
  PkPfPv_long_mfi <- PkPfPv_Final_MFI_RAU %>%
    dplyr::select(-ends_with("_Dilution")) %>%
    dplyr::rename_with(~str_replace(., "_MFI", ""), ends_with("_MFI")) %>%
    tidyr::pivot_longer(-c(SampleID, Plate), names_to = "Antigens", values_to = "MFI") %>%
    dplyr::left_join(panel, by = "Antigens")
  PkPfPv_long_rau <- suppressWarnings(
    PkPfPv_Final_MFI_RAU %>%
      dplyr::select(-ends_with("_MFI")) %>%
      dplyr::rename_with(~str_replace(., "_Dilution", ""), ends_with("_Dilution")) %>%
      tidyr::pivot_longer(-c(SampleID, Plate), names_to = "Antigens", values_to = "RAU") %>%
      tidyr::separate(Antigens, c("Antigens", "Beads"), "_") %>%
      dplyr::left_join(panel, by = "Antigens")) %>%
    dplyr::mutate(RAU_Method = case_when(
      Beads == "loglog" ~ "loglog",
      Beads == "Adjloglog" ~ "Adj_loglog",
      .default = "loglog")
    ) %>% dplyr::select(-Beads)

  PkPfPv_long_mfi_rau <- suppressWarnings(
    PkPfPv_long_mfi %>%
      right_join(PkPfPv_long_rau, by = c("SampleID", "Plate", "Antigens", "Species")) %>%
      dplyr::mutate(Species = case_when(
        is.na(Species) & stringr::str_detect(Antigens, "Pv") ~ "Pv",
        is.na(Species) & stringr::str_detect(Antigens, "Pf") ~ "Pf",
        is.na(Species) & stringr::str_detect(Antigens, "Pk") ~ "Pk",
        T ~ Species

      ))
  ) %>%
    dplyr::select(SampleID, Plate, Antigens, Species, MFI, RAU, RAU_Method)

  return(list(All_Results = PkPfPv_Final, MFI_RAU = PkPfPv_Final_MFI_RAU, MFI_RAU_long = PkPfPv_long_mfi_rau))

}
#' Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' conversion for Pk proteins
#'
#' This function is utilised in the master function `MFItoRAU_Plasmo()`.
#'
#' @param processed_Pk  df$Pk of output `processPkPfPv()`
#' @param plate_list  Output of `readPlateLayout()`
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#' @param qc_results Output from `runQC()`.
#'
#' @return Data frame with MFI data, converted RAU data and matched SampleID's.
#' @export
#'
#' @importFrom dplyr filter mutate  select  group_split arrange rename inner_join
#' @importFrom tidyr pivot_longer as_tibble pivot_wider
#' @importFrom drc drm
#' @importFrom purrr  reduce  map
#'
#' @author Dionne Argyropoulos, Caitlin Bourke
MFItoRAU_Pk <- function(processed_Pk, plate_list, std_point, qc_results){

  setup <- .setup_mfitorau_inputs(
    df = processed_Pk,
    plate_list = plate_list,
    std_point = std_point
  )

  L        <- setup$L
  layout   <- setup$layout
  antigens <- setup$antigens

  # unpack params
  dilution                      <- setup$params$dilution
  dilution_scaled               <- setup$params$dilution_scaled
  dilution_factor               <- setup$params$dilution_factor
  current_min_relative_dilution <- setup$params$current_min_relative_dilution
  s1_concentration              <- setup$params$s1_concentration
  s_final_concentration         <- setup$params$s_final_concentration

  counts_QC_output              <- qc_results$getCountsQC_output

  # Iterate over each level in L$Plate and corresponding layout data frame
  stds_mod <- list()
  model_catch <- list()
  antigens_split_rau <- list()

  for (plate_idx in seq_along(unique(L$Plate))) {
    plate_level <- unique(L$Plate)[plate_idx]
    subset_data <- L[L$Plate == plate_level, ]

    # Fetch the corresponding layout data frame
    current_layout <- layout[[plate_level]] ######## when the plate tab name == the plate level defined in the plate column from the file name

    # Get standard curves for this plate
    stds <- subset_data %>%
      dplyr::filter(type.letter == "S") %>%
      dplyr::mutate(standard_type = as.factor(sub(".*?(\\d+).*", "\\1", Sample))) %>%
      tidyr::pivot_longer(-c(Sample, Location, Plate, type.letter, standard_type), names_to = "antigens", values_to = "mfi") %>%
      dplyr::mutate(log_mfi = log(mfi)) %>%
      dplyr::group_split(antigens)

    # Get sample data for all antigens
    # Creating an additional list of dfs is mfi reading for each of the antigens - and adding another column for log_mfi which is needed by the 5PL
    antigens_split <- subset_data %>%
      tidyr::pivot_longer(-c(Sample, Location, Plate, type.letter), names_to = "antigens", values_to = "mfi") %>%
      dplyr::mutate(log_mfi = log(mfi)) %>%
      dplyr::group_split(antigens)

    # Extract the number of antigens
    nprot <- length(antigens_split)
    # Extract the names of the antigens from the list of df
    named_prot <- NULL
    for (i in 1:length(antigens_split)) {
      named_prot[[i]] <- unique(antigens_split[[i]]$antigens)
    }
    named_prot <- unlist(named_prot)

    # Fit models per standard_type/antigens
    stds_mod_plate <- list()
    model_catch_plate <- list()

    suppressWarnings({
      for (i in 1:length(stds)) {
        stds_mod[[i]] <- stds[[i]] %>%
          tidyr::as_tibble() %>%
          dplyr::arrange(standard_type) %>%
          dplyr::mutate(
            dilution = dilution,
            dilution_scaled = dilution_scaled
          )
        model_catch[[i]] <- drc::drm(
          stds_mod[[i]]$log_mfi ~ stds_mod[[i]]$dilution,
          fct = LL.5(names = c("slope", "low_asym", "upp_asym", "ED50", "asym_par"))
        )
      }
    })
    names(model_catch) <- named_prot
    names(stds_mod) <- named_prot

    for (i in 1:length(antigens_split)) {
      df <- antigens_split[[i]]
      antigens <- unique(df$antigens)

      std_tbl <- stds_mod[[antigens]]
      mdl     <- model_catch[[antigens]]

      rau_df <- df %>%
        dplyr::mutate(
          max_s1 = std_tbl[std_tbl$standard_type=="1", ]$log_mfi,
          max_dil = std_tbl[std_tbl$standard_type=="1", ]$dilution,
          slope = mdl$fit$par[1],
          low_asym = mdl$fit$par[2],
          upp_asym = mdl$fit$par[3],
          ed50 = mdl$fit$par[4],
          asym_par = mdl$fit$par[5]
        ) %>%
        dplyr::mutate(rau = case_when(
          log_mfi>=max_s1 ~ max_dil,
          log_mfi<max_s1 ~ ed50*((((upp_asym-low_asym)/(log_mfi-low_asym))^(1/asym_par) - 1 )^(1/slope))
        )
        ) %>%
        dplyr::mutate(rau = ifelse(is.na(rau), s_final_concentration, rau)) %>%
        dplyr::mutate(rau = case_when(
          rau<s_final_concentration   ~ s_final_concentration,
          rau>s1_concentration        ~ s1_concentration,
          TRUE ~ rau
        )
        ) %>%
        dplyr::rename(MFI = mfi, Dilution = rau)

      rau_df_wide <- rau_df %>%
        tidyr::pivot_wider(
          id_cols = c(Location, Sample, Plate, type.letter),
          names_from = antigens,
          values_from = c(
            MFI, log_mfi, max_s1, max_dil, slope,
            low_asym, upp_asym, ed50, asym_par,
            Dilution
          ),
          names_glue = "{antigens}_{.value}"
        ) %>%
        dplyr::filter(!(type.letter == "S" | type.letter == "B")) %>%
        dplyr::select(-type.letter) %>%
        dplyr::mutate(Location.2 = stringr::str_extract(Location, "(?<=,)\\w+\\d+(?=\\))"))

      # Join to plate layout names
      names(current_layout)[1] <- "col" # Relabel first column to be "Plate"
      current_layout_final <- current_layout %>%
        tidyr::pivot_longer(-1) %>%
        dplyr::select(col, row = name, SampleID = value) %>%
        dplyr::mutate(Location.2 = paste0(col, row)) %>%
        dplyr::select(-c(col, row))

      rau_df_final <- rau_df_wide %>%
        dplyr::left_join(current_layout_final, by = "Location.2") %>%
        dplyr::select(SampleID, Location, Location.2, Sample, Plate, everything())

      # Save as list
      antigens_split_rau[[paste0(plate_level, "_", antigens)]] <- rau_df_final

    }
  }

  # Step 4: Final Results
  # Define ID columns (common to all dfs)
  id_cols <- c("SampleID", "Location", "Location.2", "Sample", "Plate")
  # Split by prefix before "_"
  by_prefix <- split(antigens_split_rau, sub("_.*", "", names(antigens_split_rau)))
  # Left join within each prefix group
  joined_list <- map(by_prefix, ~ reduce(.x, left_join, by = id_cols))
  # Bind lists
  rau_combined <- bind_rows(joined_list)
  # Join to counts_QC_output
  counts_data <- counts_QC_output %>%
    ungroup() %>%
    dplyr::select(SampleID, Location.2 = Location, Plate, QC_total)

  final_results <- rau_combined %>%
    dplyr::inner_join(counts_data, by = c("SampleID", "Plate", "Location.2"))

  return(final_results)
}
#' Helper function to set up MFI to RAU function
#'
#' @param df  Output from `readSeroData()`.
#' @param plate_list Output from `readPlateLayout()`.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#'
#' @returns A list of processed sero_data, processed plate layout, antigen names, and parameters for standard curve.
#' @export
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Setup MFI to RAU
#' setup <- .setup_mfitorau_inputs(
#'   df = sero_data$results,
#'   plate_list = plate_list,
#'   std_point = 10
#' )
#'
#' }
#'
#' @author Dionne Argyropoulos
.setup_mfitorau_inputs <- function(df, plate_list, std_point) {

  # Ensure MFI columns are numeric
  L <- df %>% dplyr::mutate(dplyr::across(-c(Location, Sample, Plate), as.numeric))
  # Load plate list
  layout <- plate_list

  # Identify antigens
  remaining_cols <- setdiff(colnames(L), c("Location", "Sample", "Plate"))
  antigens <- remaining_cols[remaining_cols != ""]

  # Categorises into "B" = "Blank", "S" = "Standards", "U" or "X" = "Samples"
  L$type.letter <- substr(L$Sample, start=1, stop=1)

  # Magic Parameters for 5-point and 10-point standard curve
  params <- switch(
    as.character(std_point),

    "5" = {
      dilution <- c(1/50, 1/(50*5^1), 1/(50*5^2), 1/(50*5^3), 1/(50*5^4))

      list(
        dilution = dilution,
        dilution_scaled = dilution * (50*5^4),
        dilution_factor = 5,
        current_min_relative_dilution = 5.0^-5,
        s1_concentration = 1/50,
        s_final_concentration = 1/51200
      )
    },
    "10" = {
      dilution <- c(1/50, 1/100, 1/200, 1/400, 1/800, 1/1600, 1/3200, 1/6400, 1/12800, 1/25600)

      list(
        dilution = dilution,
        dilution_scaled = dilution * 25600,
        dilution_factor = 2,
        current_min_relative_dilution = 2.0^-10,
        s1_concentration = 1/50,
        s_final_concentration = 1/51200
      )
    },
    "PvLDH" = {
      dilution <- c(1000000, 333333.33, 111111.11, 37037.04, 12345.68, 4115.23, 1371.74, 457.25, 152.42, 50.81)

      list(
        dilution = dilution,
        dilution_scaled = NULL,
        dilution_factor = NULL,
        current_min_relative_dilution = NULL,
        s1_concentration = 1000000, # Set observations with very high MFI to 1,000,000 pg/ml.
        s_final_concentration = 16.94 # Set observations with very low MFI to 16.94 pg/ml.
      )
    },
    stop("No standard curve points provided.")
  )

  # Return everything
  list(
    L = L,
    layout = layout,
    antigens = antigens,
    params = params
  )
}
#' Helper function to fit a 5-parameter logistic standard curve to dilutions
#'
#' @param subset_data Data for one plate.
#' @param antigen Data for one antigen.
#' @param dilution Set of five or ten.
#' @param s1_concentration  Concentration of highest dilution.
#' @param s_final_concentration Concentration lowest dilution.
#' @param unknown_letters Bioplex, Magpix or Intelliflex known unknown letters (Default = U and X).
#'
#' @returns A list of the model results data frame and model.
#' @export
#'
#' @author Connie Li Wai Suen, Dionne Argyropoulos
.process_antigen_loglog <- function(
    subset_data,
    antigen,
    dilution,
    s1_concentration,
    s_final_concentration,
    unknown_letters = c("U", "X")
) {

  ## Extract standards
  std <- subset_data %>%
    filter(type.letter == "S") %>%
    pull({{ antigen }})

  std <- as.numeric(std)
  std[is.na(std) | std == 0] <- 1
  log.std <- log(std)

  ## Fit 5PL model
  model <- suppressWarnings(drc::drm(
    log.std ~ dilution,
    fct = drc::LL.5(names = c("b", "c", "d", "e", "f"))
  ))

  coefs <- coef(model)
  b <- coefs[1]
  c <- coefs[2]
  d <- coefs[3]
  e <- coefs[4]
  f <- coefs[5]

  ## Process unknowns
  results.df <- NULL

  for (r in seq_len(nrow(subset_data))) {

    if (subset_data$type.letter[r] %in% unknown_letters) {

      mfi.X <- as.numeric(subset_data[r, antigen])
      y <- log(mfi.X)

      dil.X <- if (y > max(log.std)) {
        max(dilution)
      } else {
        e * ((((d - c) / (y - c))^(1 / f) - 1)^(1 / b))
      }

      ## Bounds
      dil.X <- ifelse(dil.X > s1_concentration, s1_concentration, dil.X)
      dil.X <- ifelse(is.na(dil.X) & y > log.std[2], s1_concentration, dil.X)
      dil.X <- ifelse(dil.X < s_final_concentration, s_final_concentration, dil.X)
      dil.X <- ifelse(is.na(dil.X) & y < max(log.std), s_final_concentration, dil.X)

      results <- data.frame(
        Location = subset_data[r, "Location"],
        Sample   = subset_data[r, "Sample"],
        Plate    = subset_data[r, "Plate"],
        MFI      = mfi.X,
        Dilution = dil.X,
        DilutionReciprocal = 1 / dil.X,
        MinStd       = min(std),
        MaxDilution  = min(dilution),
        MaxStd       = max(std),
        MinDilution  = max(dilution),
        stringsAsFactors = FALSE
      )

      colnames(results) <- c(
        "Location", "Sample", "Plate",
        paste0(
          antigen, "_",
          c(
            "MFI", "Dilution", "DilutionReciprocal",
            "MinStd", "MaxDilution", "MaxStd", "MinDilution"
          )
        )
      )

      results.df <- rbind(results.df, results)
    }
  }

  list(
    results = results.df,
    model = model
  )
}
#' Helper function to add Sample IDs to output.
#'
#' @param df Data frame following 5-parameter logistic function applied.
#' @param layout Output from `readPlateLayout()`.
#' @param plate_level Specific plate of interest.
#'
#' @returns Processed data frame with correct Sample IDs.
#' @export
#'
#' @author Dionne Argyropoulos
.merge_mfitorau <- function(df, layout, plate_level) {

  # Add Location.2
  df <- df %>%
    dplyr::mutate(
      Location.2 = stringr::str_split_fixed(as.character(Location), ",", 2)[, 2],
      Location.2 = stringr::str_trim(Location.2),
      Location.2 = stringr::str_sub(Location.2, 1, -2)
    ) %>%
    dplyr::select(Location.2, dplyr::everything())

  # Prepare plate layout
  plate_layout_current <- layout[[plate_level]] %>%
    dplyr::rename(Plate = 1) %>%
    dplyr::mutate(
      dplyr::across(tidyselect::matches("^[0-9]+$"), as.character)
    ) %>%
    tidyr::pivot_longer(
      cols = tidyselect::matches("^[.x]*[0-9]+$"),
      names_to = "numeric",
      values_to = "SampleID"
    ) %>%
    dplyr::rename(alpha = Plate) %>%
    tidyr::unite("Location.2", alpha:numeric, sep = "", na.rm = TRUE)

  # Join SampleID
  df <- df %>%
    dplyr::left_join(plate_layout_current, by = "Location.2") %>%
    dplyr::distinct(SampleID, Location.2, .keep_all = TRUE) %>%
    tidyr::drop_na() %>%
    dplyr::select(SampleID, dplyr::everything())

  # Coerce column types
  character_columns <- c("SampleID", "Location", "Location.2", "Sample", "Plate")

  df <- df %>%
    dplyr::mutate(
      dplyr::across(dplyr::all_of(character_columns), as.character),
      dplyr::across(!dplyr::all_of(character_columns), as.numeric)
    )

  return(df)
}
#' Convert known dilution to mfi from fitted standard curve
#' @description
#' Convert dilution to predicted mfi using known standard curve fit.
#'
#' @param dilution Known dilution of samples
#' @param params Known parameters for five parameter logistic fit.
#' @return Returns the predicted mfi of a sample with known dilution.
#' @export
#' @author Eamon Conway
#'
#' @examples
#' # This function is typically called internally by higher-level workflows.
#' # Below is a minimal runnable example using dummy parameters.
#'
#' # Five-parameter logistic model typically expects parameters in the order:
#' # a, b, c, d, e  (e often log-transformed)
#' dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01))
#'
#' # Example dilution value
#' dilution_example <- 0.1
#'
#' # Predict MFI from the dummy standard curve
#' .convert_dilution_to_mfi(dilution_example, dummy_params)
#'
.convert_dilution_to_mfi <- function(dilution, params) {
  if (is.null(dilution) || is.null(params)) {
    error("Require both mfi and params to run.")
  }
  exp(.log_logistic_5p(dilution, params[1], params[2], params[3], params[4], exp(params[5])))
}
#' Convert mfi to dilution using known standard curve fit and no bounds
#' @description
#' Convert mfi to dilution using known standard curve fit and no bounds unless you are below the asymptote of the standard curve.
#' In this situation we set your value to min_relative_dilution. I dunno argue?
#' @param mfi Known mfi of samples
#' @param params Known parameters for five parameter logistic fit.
#' @param min_relative_dilution Known minimum value of dilution in the standard curve. Relative means setting S1 to a dilution/RAU/concentration of 1.
#' @return Returns the dilution of each sample in mfi.
#' @export
#' @author Eamon Conway
#' @examples
#' # This function is generally called inside higher-level analysis workflows.
#' # Below is a minimal self-contained example using dummy values.
#'
#' # Dummy five-parameter logistic fit parameters:
#' # a, b, c, d, e  (with e typically supplied on the log scale)
#' dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01))
#'
#' # Example MFI value
#' mfi_example <- 1500
#'
#' # Minimum relative dilution from the standard curve
#' min_rel_dil <- 1
#'
#' # Convert MFI to dilution without bounds
#' .convert_mfi_to_dilution_no_bounds(mfi_example, dummy_params, min_rel_dil)
.convert_mfi_to_dilution_no_bounds <- function(mfi, params, min_relative_dilution) {
  if (is.null(mfi) | is.null(params)) {
    error("Require both mfi and params to run.")
  }
  y <- log(mfi)
  y[y >= (params[2] + params[3])] <- 0.999*(params[2] + params[3])
  result <- .inverse_log_logistic_5p(
    y,
    params[1],
    params[2],
    params[3],
    params[4],
    exp(params[5])
  )
  result[y < params[2]] <- min_relative_dilution
  return(result)
}
#' Convert mfi to dilution using known standard curve fit and no lower bound
#' @description
#' Convert mfi to dilution using known standard curve fit and no lower bound unless you are below the asymptote of the standard curve.
#' In this situation we set your value to min_relative_dilution. I dunno argue?
#' @param mfi Known mfi of samples
#' @param params Known parameters for five parameter logistic fit.
#' @param min_relative_dilution Known minimum value of dilution in the standard curve. Relative means setting S1 to a dilution/RAU/concentration of 1.
#' @return Returns the dilution of each sample in mfi.
#' @export
#' @author Eamon Conway
#' @examples
#' # This function is usually called inside higher-level analysis steps.
#' # Below is a minimal runnable example using dummy values.
#'
#' # Dummy five-parameter logistic fit parameters:
#' # a, b, c, d, e  (with e typically on the log scale)
#' dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01), f = 0, g = 5)
#'
#' # Example MFI value
#' mfi_example <- 1500
#'
#' # Minimum relative dilution from the standard curve
#' min_rel_dil <- 1
#'
#' # Convert MFI to dilution without applying a lower bound
#' .convert_mfi_to_dilution_no_lower_bound(mfi_example, dummy_params, min_rel_dil)
.convert_mfi_to_dilution_no_lower_bound <- function(mfi, params, min_relative_dilution) {
  if (is.null(mfi) | is.null(params)) {
    error("Require both mfi and params to run.")
  }
  y <- log(mfi)
  result <- .inverse_log_logistic_5p(
    y,
    params[1],
    params[2],
    params[3],
    params[4],
    exp(params[5])
  )
  result[y > (params[2] + params[3])] <- 1.0
  result[y < params[2]] <- min_relative_dilution
  result[y > params[7]] <- 1.0
  # I dont think this will happen - Eamon (ask if needed)
  result[result > 1.0] <- 1.0
  return(result)
}
#' Convert mfi to dilution using known standard curve fit.
#' @description
#' Convert mfi to dilution using known standard curve fit.
#'
#' @param mfi Known mfi of samples
#' @param params Known parameters for five parameter logistic fit.
#' @param min_relative_dilution Known minimum value of dilution in the standard curve. Relative means setting S1 to a dilution/RAU/concentration of 1.
#' @return Returns the dilution of each sample in mfi.
#' @export
#' @author Eamon Conway
#' @examples
#' # This function is typically used within larger analysis pipelines.
#' # Below is a minimal runnable example using dummy values.
#'
#' # Dummy five-parameter logistic fit parameters:
#' # a, b, c, d, e  (with e on the log scale)
#' # Additional placeholders (f, g) included so params[6] and params[7] exist.
#' dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01),
#'                   f = -5, g = 5)
#'
#' # Example MFI value
#' mfi_example <- 1500
#'
#' # Minimum relative dilution allowed
#' min_rel_dil <- 1
#'
#' # Convert MFI to dilution
#' .convert_mfi_to_dilution(mfi_example, dummy_params, min_rel_dil)
.convert_mfi_to_dilution <- function(mfi, params, min_relative_dilution) {
  if (is.null(mfi) | is.null(params)) {
    error("Require both mfi and params to run.")
  }
  y <- log(mfi)
  result <- .inverse_log_logistic_5p(
    y,
    params[1],
    params[2],
    params[3],
    params[4],
    exp(params[5])
  )
  result[y > (params[2] + params[3])] <- 1.0
  result[y < params[2]] <- min_relative_dilution
  result[y < params[6]] <- min_relative_dilution
  result[y > params[7]] <- 1.0
  # I dont think this will happen - Eamon (ask if needed)
  result[result > 1.0] <- 1.0
  return(result)
}
#' Fit a standard curve to known mfi and dilution values.
#' @description
#' We wish to convert the standard curve samples to a five parameter logistic curve.
#' This function takes those values and calls optim to determine the fit.
#'
#' @param mfi Known mfi of samples
#' @param dilution Known dilution of samples
#' @param control Optional list of control parameters for the underlying call to optim.
#'
#' @return standard curve log logistic
#' @export
#' @author Eamon Conway
#' @examples
#' # This function is typically called within data-processing workflows.
#' # Workflow-style example (not run on CRAN)
#'
#' \donttest{
#'
#' # This block demonstrates how .fit_standard_curve() is typically used
#' # inside the MFItoRAU_Adj-conversion pipeline.
#'
#' # Step 1 — Prepare master file (normally from readSeroData)
#' master_file <- data.frame(
#'   Location = c("A1","A2","A3"),
#'   Sample   = c("S1","S2","S3"),
#'   Plate    = c("Plate1","Plate1","Plate1"),
#'   Ag1 = c(12000, 8000, 4000),
#'   Ag2 = c(9000,  5000, 2500)
#' )
#'
#' # Convert antigen columns to numeric
#' L <- master_file |>
#'   dplyr::mutate(dplyr::across(-c(Location, Sample, Plate), as.numeric))
#'
#' # Fake plate layout (normally from readPlateLayout)
#' layout <- list(Plate1 = data.frame(Location = c("A1","A2","A3"), WellType = "STD"))
#'
#'
#' # Step 2 — Load reference standard curve MFI values (dummy data)
#' refs <- data.frame(
#'   std_plate = rep("StdPlate1", 5),
#'   antigen   = rep("Ag1", 5),
#'   dilution  = c(1, 1/2, 1/4, 1/8, 1/16),
#'   eth_mfi   = c(14000, 7000, 3500, 1800, 900),
#'   png_mfi   = c(15000, 7600, 3800, 1900, 950)
#' )
#'
#'
#' # Step 3 — Define optimisation settings
#' control <- list(
#'   maxit  = 10000,
#'   abstol = 1e-8,
#'   reltol = 1e-6
#' )
#'
#'
#' # Step 4 — Fit ETH and PNG curves per standard-plate × antigen
#' ref_fit <- refs |>
#'   dplyr::group_by(.data$std_plate, .data$antigen) |>
#'   tidyr::nest() |>
#'   dplyr::mutate(
#'     eth_fit = purrr::map(data, ~ .fit_standard_curve(.x$eth_mfi, .x$dilution, control)),
#'     png_fit = purrr::map(data, ~ .fit_standard_curve(.x$png_mfi, .x$dilution, control))
#'   )
#'
#' ref_fit
#' }
.fit_standard_curve <- function(mfi, dilution, control = NULL) {
  if (is.null(mfi) | is.null(dilution)) {
    error("Require both mfi and dilution to run.")
  }

  y1 <- log(mfi)
  initial_solution <- c(-1.0, 0.0, max(y1), 0.0, 0.0)

  .error_func <- function(x) {
    f1 <- .log_logistic_5p(dilution, x[1], x[2], x[3], x[4], exp(x[5]))
    sum((y1 - f1)^2.0)
  }

  solution <- optim(par = initial_solution, fn = .error_func, control = control)
  if (solution$convergence != 0) {
    stop("Standard curve failed to converge. Look at data and possibly change control parameters from default.")
  }
  c(solution$par, min(y1), max(y1))
}

.inverse_log_logistic_5p <- function(y,b,c,d,e,f){
  A <- (d/(y-c))^(1/f)-1
  return(exp(-e) *A^(1/b))
}

.log_logistic_5p <- function(x, b, c, d, e, f) {
  return(c + d / (1.0 + exp(b * (log(x) + e)))^f)
}

#' Plot Bead Count Data
#'
#' This function gets the count data and plots the plate image, creating a new
#' facet (i.e., panel) for each antigen and each line represents the
#' different plates so that they can be visualised.
#'
#' @param qc_results Output from `runQC()`.
#' @param experiment_name User-input experiment name.
#' @return Tile Plot showing binary result of "sufficient beads" with cut-off
#' >15 beads and "repeat" less than or equal to 15 beads (ggplot).
#' @export
#' @import ggplot2
#' @importFrom forcats fct_rev
#' @author Shazia Ruybal-Pesántez, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Step 3: Plot Counts
#' plotCounts(qc_results, "experiment1")
#' }
plotCounts <- function(qc_results, experiment_name){

  bead_counts <- qc_results$getCounts_output
  bead_counts$Plate <- factor(
    bead_counts$Plate,
    levels = unique(bead_counts$Plate[order(as.numeric(str_extract(bead_counts$Plate, "\\d+")))])
  ) # reorder by plate number
  bead_counts %>%
    ggplot2::ggplot(mapping = aes(x = Col, y = forcats::fct_rev(Row), fill = Repeat)) +
    ggplot2::geom_tile(aes(height = 0.90, width = 0.90)) +
    ggplot2::scale_x_continuous(breaks = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12), position = "top") +
    ggplot2::scale_fill_manual(values = c("sufficient beads" = "#91bfdb", "repeat" = "#d73027"), drop=FALSE) +
    ggplot2::theme_bw() +
    ggplot2::labs(x = "", y = "", title = experiment_name , fill = "") +
    ggplot2::facet_wrap( ~ Plate, ncol = 3, scales = "free_y")  # This will create separate facets for each level of 'Plate'
}

#' Plot Raw Median Fluorescent Intensity of Standard Curve Data
#'
#' This function gets the standards data and plots the standard curves.
#'
#' @param sero_data Output from `readSeroData()`.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve. Default = 10. Value is an integer.
#' @param location "PNG" or "ETH" to filter WEHI standard curve data.
#' @param experiment_name User-input experiment name.
#' @return
#' - Dot and line plot of standard curves (S1-S10) with PNG or Ethiopia stds
#' underneath (ggplot).
#' - WEHI-acceptable standard curve data on background of plot with user data.
#' @importFrom dplyr select mutate filter
#' @importFrom tidyr  pivot_longer
#' @import ggplot2
#' @export
#' @author Dionne Argyropoulos, Shazia Ruybal-Pesantez
#'
#' @examples
#' \donttest{
#' # Example demonstrating how to process bead count data.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
#'
#' # Plot Standards
#' plotStds(
#'   sero_data = sero_data,
#'   location = "ETH",
#'   experiment_name = "experiment1"
#' )
#'
#' }
#'
plotStds <- function(sero_data, std_point = 10, location, experiment_name){

  # stratify data
  master_file <- sero_data
  stds <- master_file$stds

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19", "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  stds_1 <- stds %>%
    dplyr::select(-Location) %>%
    tidyr::pivot_longer(-c(Sample, Plate), names_to = "Antigen", values_to = "MFI") %>%
    dplyr::mutate(
      Plate = factor(Plate, levels = unique(Plate[order(as.numeric(str_extract(Plate, "\\d+")))])), # reorder by plate number
      Antigen = dplyr::recode(Antigen, !!!name_lookup),
      MFI = as.numeric(MFI)
    )

  if(std_point == 10){

    stds_1 <- stds_1 %>% dplyr::mutate(Sample = factor(Sample, c("S1", "S2", "S3", "S4", "S5", "S6", "S7", "S8", "S9", "S10")))

    location_1 <- ifelse(location == "ETH", "ETH", "PNG")
    wehi_stds <- read.csv(system.file("extdata", "all_stds_MFI.csv", package = "SeroTrackR"))
    wehi_stds <- wehi_stds %>%
      dplyr::filter(Location==location_1) %>%
      mutate(Antigen = dplyr::recode(Antigen, !!!name_lookup))

    suppressMessages(
      suppressWarnings(
        ggplot2::ggplot() +
          ggplot2::geom_point(data = wehi_stds, aes(x = Sample, y = MFI), colour = "grey", alpha = 0.25) +
          ggplot2::geom_point(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate,
                                                 text = paste("Sample:", Sample, "<br>MFI:", MFI, "<br>Plate:", Plate))) +
          ggplot2::geom_line(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate)) +
          ggplot2::scale_y_log10(breaks = c(0, 10, 100, 1000, 10000)) +
          ggplot2::labs(
            x = "Standard Curve",
            y = "MFI",
            title = experiment_name
          ) +
          ggplot2::facet_wrap(~Antigen) +
          ggplot2::theme_bw() +
          ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))
      )
    )

  } else if (std_point == 5){

    stds_1 <- stds_1 %>% dplyr::mutate(Sample = factor(Sample, c("S1", "S2", "S3", "S4", "S5")))

    suppressMessages(
      suppressWarnings(
        ggplot2::ggplot() +
        ggplot2::geom_point(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate,
                                               text = paste("Sample:", Sample, "<br>MFI:", MFI, "<br>Plate:", Plate))) +
        ggplot2::geom_line(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate)) +
        ggplot2::scale_y_log10(breaks = c(0, 10, 100, 1000, 10000)) +
        ggplot2::labs(
          x = "Standard Curve",
          y = "MFI",
          title = experiment_name
        ) +
        ggplot2::facet_wrap(~Antigen) +
        ggplot2::theme_bw() +
        ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))
      )
    )

  } else {
    message("Please write a standard point curve.")
  }

}
#' Plot Raw Median Fluorescent Intensity of Standard Curve Data
#'
#' This function gets the standards data and plots the standard curves for any antigens (i.e., non-PvSeroTaT specific).
#'
#' @param sero_data Output from `readSeroData()`.
#' @param experiment_name User-input experiment name.
#' @return
#' - Dot and line plot of standard curves (S1-S10)
#' - WEHI-acceptable standard curve data on background of plot with user data.
#' @importFrom dplyr select mutate filter
#' @importFrom tidyr  pivot_longer
#' @import ggplot2
#' @export
#' @author Shazia Ruybal-Pesantez, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example demonstrating how to process bead count data.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data <- c(
#'    system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'    system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'    system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
#'
#' # Plot Standards
#' plotStds_all(
#'   sero_data = sero_data,
#'   experiment_name = "experiment1"
#' )
#'
#' }
#'
plotStds_all <- function(sero_data, experiment_name){
  master_file <- sero_data
  stds <- master_file$stds

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  stds_1 <- stds %>%
    dplyr::select(-Location) %>%
    tidyr::pivot_longer(-c(Sample, Plate), names_to = "Antigen", values_to = "MFI") %>%
    dplyr::mutate(
      Plate = factor(Plate, levels = unique(Plate[order(as.numeric(str_extract(Plate, "\\d+")))])), # reorder by plate number
      Antigen = dplyr::recode(Antigen, !!!name_lookup),
      Sample = factor(Sample, levels = unique(Sample[order(as.numeric(str_extract(Sample, "\\d+")))])), # reorder by standard curve number
      MFI = as.numeric(MFI)
    )


  suppressMessages(
    suppressWarnings(
      ggplot() +
      ggplot2::geom_point(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate)) +
      ggplot2::geom_line(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate)) +
      ggplot2::scale_y_log10(breaks = c(0, 10, 100, 1000, 10000)) +
      ggplot2::labs(
        x = "Standard Curve",
        y = "MFI",
        title = experiment_name
      ) +
      ggplot2::facet_wrap(~Antigen) +
      ggplot2::theme_bw() +
      ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))
    )
  )

}
#' Plot Raw Median Fluorescent Intensity of Pk/Pf/Pv Standard Curve Data
#'
#' This function gets the standards data and plots the standard curves for antigens in the Pk/Pf/Pv panel.
#'
#' @param sero_data Output from `readSeroData()`.
#' @param experiment_name User-input experiment name.
#' @param panel Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.
#'
#' @return
#' - Dot and line plot of standard curves (S1-S10)
#' - WEHI-acceptable standard curve data on background of plot with user data.
#'
#' @import ggplot2
#' @importFrom dplyr select mutate filter left_join across
#' @importFrom tidyr  pivot_longer  separate
#' @importFrom stringr str_detect str_extract
#' @importFrom utils read.csv
#'
#' @export
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example demonstrating how to process bead count data.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data <- c(
#'    system.file("extdata", "example_MAGPIX_pk_5std_plate1.csv", package = "SeroTrackR"),
#'    system.file("extdata", "example_MAGPIX_pk_5std_plate2.csv", package = "SeroTrackR")
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
#'
#' # Plot Standards
#' plotStds_PkPfPv(
#'   sero_data = sero_data,
#'   experiment_name = "experiment1",
#'   panel = "panel1"
#' )
#'
#' }
#'
plotStds_PkPfPv <- function(
    sero_data,
    experiment_name,
    panel = "panel1"
   ){

  # Check if shiny.fluent is installed
  if (!requireNamespace("zoo", quietly = TRUE)) {
    stop("Package 'zoo' is required for plotStds_PkPfPv(). Please install it.", call. = FALSE)
  }

  #panel 1 is default

  if(panel == "panel1"){
    panel <-read.csv(system.file("extdata", "PkPfPv_Panel_1.csv", package = "SeroTrackR"))
  } else {
    panel <- read.csv(panel)
  }


  master_file <- sero_data
  stds <- master_file$stds

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  stds <- stds %>%
    .relabel_columns()

  stds_1 <- stds %>%
  ################################################################
  # Code to be replaced in future when labelling Stds not an issue
  ################################################################
  dplyr::mutate(
    dplyr::across(-c(Location, Sample, Plate), as.numeric),
    suffix = str_extract(Sample, "\\s*\\([^\\)]+\\)"),
    suffix = zoo::na.locf(suffix, na.rm = FALSE),
    Sample = ifelse(str_detect(Sample, "^S\\d+$"), paste0(Sample, suffix), Sample)
  ) %>%
    dplyr::select(-suffix) %>%
  ################################################################
  # Code to keep
  ################################################################
  dplyr::select(-Location) %>%
    tidyr::separate(Sample, c("Sample", "Beads"), sep = "[ -]") %>%
    tidyr::pivot_longer(-c(Sample, Beads, Plate), names_to = "Antigen", values_to = "MFI") %>%
    dplyr::mutate(
      Plate = factor(Plate, levels = unique(Plate[order(as.numeric(str_extract(Plate, "\\d+")))])), # reorder by plate number
      Beads = stringr::str_extract(Beads, "[A-Z]+"), # ETH (ETH) may sometimes be entered differently (in brackets or not) - this should just keep the letters PK or ETH that we want
      Sample = factor(Sample, levels = unique(Sample[order(as.numeric(str_extract(Sample, "\\d+")))])), # reorder by standard curve number
      Antigen = dplyr::recode(Antigen, !!!name_lookup),
      MFI = as.numeric(MFI)
    ) %>%
    dplyr::left_join(panel, by = c("Antigen" = "Antigens")) %>%
    dplyr::mutate(Species = case_when(
      is.na(Species) & stringr::str_detect(Antigen, "Pv") ~ "Pv",
      is.na(Species) & stringr::str_detect(Antigen, "Pf") ~ "Pf",
      is.na(Species) & stringr::str_detect(Antigen, "Pk") ~ "Pk",
      T ~ Species
    )) %>%
    dplyr::mutate(stds_to_keep = case_when(
      Species=="Pk" & Beads == "PK" ~ "keep",
      Species=="Pf" & Beads == "ETH" ~ "keep",
      Species=="Pv" & Beads == "ETH" ~ "keep",
      .default = "remove"
    )) %>%
    dplyr::filter(stds_to_keep == "keep") %>%
    dplyr::select(-stds_to_keep)


  suppressMessages(
    suppressWarnings(
      ggplot() +
      ggplot2::geom_point(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate)) +
      ggplot2::geom_line(data = stds_1, aes(x = Sample, y = MFI, color = Plate, group = Plate)) +
      ggplot2::scale_y_log10(breaks = c(0, 10, 100, 1000, 10000)) +
      ggplot2::labs(x = "Standard Curve", y = "MFI", title = experiment_name) +
      ggplot2::facet_wrap(~Antigen) +
      ggplot2::theme_bw() +
      ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))
    )
  )

}

#' Read Plate Layout/s
#'
#' This function imports the plate layout. Each sheet of the plate layout
#' ".xlsx" file must contain 13 columns (labelled
#' Plate, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12) (columns A-M) and 9 rows
#' (Plate, A, B, C, D, E, F, G, H) (rows 1-9). *Note that the first row/column
#' i.e., the A1 cell in excel is called "Plate". This function also checks that
#' the plate sheet labels are consistent with the MAGPIX file input names, as a
#' check prior to merging downstream.
#'
#' @param plate_layout An ".xlsx" file with sheets labelled plate1, plate2...
#' etc..
#' @param sero_data Output from `readSeroData()`.
#' @return A list of data frames, with each one representing an individual plate.
#' @export
#' @importFrom openxlsx getSheetNames read.xlsx
#' @author Shazia Ruybal-Pesántez, Dionne Argyropoulos
#'
#' @examples
#'
#' # Example input files
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read and combine serological data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
#
#' # Step 2: Read plate layout
#' plate_list <- readPlateLayout(
#'   plate_layout = your_plate_layout,
#'   sero_data = sero_data
#' )
readPlateLayout <- function(plate_layout, sero_data) {

  if (is.null(plate_layout) || !file.exists(plate_layout)) {
    stop("ERROR: Invalid plate layout file provided.")
  }

  sheet_names <- tryCatch({
    openxlsx::getSheetNames(plate_layout)
  }, error = function(e) {
    stop("ERROR: Failed to read sheet names. Ensure the file is a valid Excel file.")
  })

  # Step 1: Get the sheet names to confirm
  sheet_names <- openxlsx::getSheetNames(plate_layout)

  # Step 2: Read all sheets into plate_list using indices
  plate_list <- lapply(1:length(sheet_names), function(i) {
    openxlsx::read.xlsx(plate_layout, sheet = i)
  })

  # Step 3: Name each element in the list after the corresponding sheet name
  names(plate_list) <- sheet_names

  # Step 4: Check if 'Plate' column exists in sero_data$results
  antigen_output_results <- sero_data$results

  if (!"Plate" %in% colnames(antigen_output_results)) {
    stop("ERROR: 'Plate' column is missing from sero_data$results.")
  }

  # Step 5: Extract levels from 'Plate' column
  antigen_output_levels <- unique(as.character(sero_data$results$Plate))  # Convert factor to character

  # Step 6: Compare plate names
  if (all(antigen_output_levels %in% sheet_names)) {
    message("Plate layouts correctly identified!")
  } else {
    stop("Plate layout sheets and plates labeled in raw data file names do not match. Ensure plate sheets are correctly labeled.")
  }

  return(plate_list)
}

#' Find and create a master plate layout file
#'
#' Join multiple a plate layout files into one master file with multiple tabs
#'
#' @param folder_path A string containing your main folder for your project or the plate layout files. Default = current working directory.
#' @param output_file A string for the path for your output master file.
#'
#' @returns An .xlsx file saved to your current working directory with multiple tabs, one tab for each plate layout.
#' @export
#'
#' @importFrom openxlsx getSheetNames read.xlsx write.xlsx
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example 1: Create two example 96-well plates in-memory
#' create_plate <- function(plate_name) {
#'   rows <- LETTERS[1:8]
#'   cols <- 1:12
#'   df <- data.frame(plate = rows)
#'   for (col in cols) {
#'     df[[as.character(col)]] <- paste0(rows, col)
#'   }
#'   df$plate_id <- plate_name
#'   df
#' }
#'
#' plate1 <- create_plate("Plate1")
#' plate2 <- create_plate("Plate2")
#'
#' # Combine plates into a list to simulate getPlateLayout() output
#' master_layout <- list(
#'   path = tempfile(fileext = ".xlsx"),  # placeholder path
#'   data = list(Plate1 = plate1, Plate2 = plate2)
#' )
#'
#' # The returned list contains:
#' # 1. path: the file path to the (simulated) master Excel file
#' # 2. data: a list of data.frames, one per plate
#' names(master_layout$data)  # View sheet names
#'
#' # Example 2: Access individual plates directly
#' layout_files <- list(plate1, plate2)  # simulate individual Excel sheets
#'
#' master_layout2 <- list(
#'   path = tempfile(fileext = ".xlsx"),  # placeholder path
#'   data = setNames(layout_files, c("Plate1", "Plate2"))
#' )
#'
#' # View the resulting plate names
#' names(master_layout2$data)
#' }
getPlateLayout <- function(folder_path = getwd(), output_file = NULL) {

  # Case 1: folder_path is length == 1 and is a folder
  if (length(folder_path) == 1 && dir.exists(folder_path)) {
    layout_files <- list.files(
      path = folder_path,
      pattern = "layout.*\\.xlsx$",
      recursive = TRUE,
      full.names = TRUE,
      ignore.case = TRUE
    )
    if (length(layout_files) == 0) {
      stop("No layout Excel files found in the specified folder.")
    }
  } else {
    layout_files <- folder_path
  }

  # Case 2: folder_path is length > 1 and contains file names
  plate_list_all <- list()

  for (file in layout_files) {
    sheet_names <- openxlsx::getSheetNames(file)

    for (sheet in sheet_names) {
      df <- openxlsx::read.xlsx(file, sheet = sheet)

      # Instead of renaming duplicates, enforce identical names to Antigen data
      if (sheet %in% names(plate_list_all)) {
        stop(sprintf(
          "Duplicate plate name detected: '%s' in file '%s'.
           Please rename sheets so each plate name is unique across all files.",
          sheet, file
        ))
      }

      plate_list_all[[sheet]] <- df
    }
  }

  if (is.null(output_file)) {
    output_file <- tempfile(fileext = ".xlsx")
  }

  openxlsx::write.xlsx(plate_list_all, file = output_file, colNames = TRUE)

  final_plate_list <- list(
    path = output_file,
    data = plate_list_all
  )

  return(final_plate_list)
}

#' 96 Well Plate Example Layout
#'
#' 96 well plate map in a wide format used in the lab. Contains information of actual Sample ID names in each well.
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 9 rows and 13 variables.
#' \describe{
#'   \item{Plate}{Contains rows labelled "A" to "H"}
#'   \item{1-12}{Contains columns labelled "1" to "12"}
#' }
#' @name example_platelayout_1_xlsx
NULL

#' 5-Point Standard Curve Example Plate Layout
#'
#' 96 well plate map in a wide format used in the lab, used for when 5-point standard curves are required for pk/pf/pv analysis. Contains information of actual Sample ID names in each well.
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 9 rows and 13 variables.
#' \describe{
#'   \item{Plate}{Contains rows labelled "A" to "H"}
#'   \item{1-12}{Contains columns labelled "1" to "12"}
#' }
#' @name example_platelayout_pk_5std_xlsx
NULL

#' 10-Point Standard Curve Example Plate Layout
#'
#' 96 well plate map in a wide format used in the lab, used for when 10-point standard curves are required for pk/pf/pv analysis. Contains information of actual Sample ID names in each well.
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 9 rows and 13 variables.
#' \describe{
#'   \item{Plate}{Contains rows labelled "A" to "H"}
#'   \item{1-12}{Contains columns labelled "1" to "12"}
#' }
#' @name example_platelayout_pk_10std_xlsx
NULL

#' Example Serological Dataset: Bioplex Plate 1
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (Bioplex).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 103 rows and 15 columns.
#' @source Randomised data
#' @name example_BioPlex_plate1_xlsx
NULL

#' Example Serological Dataset: Bioplex Plate 2
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (Bioplex).
#'
#' @format A data frame with 103 rows and 15 columns.
#' @source Randomised data
#' @name example_BioPlex_plate2_xlsx
NULL

#' Example Serological Dataset: Bioplex Plate 3
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (Bioplex).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 104 rows and 8 columns.
#' @source Randomised data
#' @name example_BioPlex_plate3_xlsx
NULL

#' Example Serological Dataset: Bioplex PvLDH Plate 1
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (Bioplex).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 103 rows and 15 columns.
#' @source Randomised data
#' @name example_BioPlex_PvLDH_plate1_xlsx
NULL

#' Example Serological Dataset: MAGPIX Plate 1

#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_plate1_csv
NULL

#' Example Serological Dataset: MAGPIX Plate 2
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_plate2_csv
NULL

#' Example Serological Dataset: MAGPIX Plate 3
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_plate3_csv
NULL

#' Example Serological Dataset: MAGPIX Plate 1 Pk Analysis 5-Point Standard Curve

#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_pk_5std_plate1_csv
NULL

#' Example Serological Dataset: MAGPIX Plate 2 Pk Analysis 5-Point Standard Curve
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_pk_5std_plate2_csv
NULL

#' Example Serological Dataset: MAGPIX Plate 1 Pk Analysis 10-Point Standard Curve

#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_pk_10std_plate1_csv
NULL

#' Example Serological Dataset: MAGPIX Plate 2 Pk Analysis 10-Point Standard Curve
#'
#' A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).
#'
#' This file is stored in inst/extdata
#'
#' @format A data frame with 614 rows and 17 columns.
#' @source Randomised data
#' @name example_MAGPIX_pk_10std_plate2_csv
NULL

#' Plot the Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' Results Data
#'
#' This function gets the Median Fluorescent Intensity (MFI) to Relative
#' Antibody Units (RAU) model results data and plots the model fits based on
#' `MFItoRAU`.
#'
#' @param mfi_to_rau_output Output from `MFItoRAU()`.
#' @param sero_data Output from `readSeroData()`.
#' @return List of dot and line plots of MFI to RAU model standard curve,
#' with each one representing an individual plate (ggplot).
#' @export
#' @import ggplot2
#' @importFrom dplyr mutate across
#' @importFrom tidyr pivot_longer
#' @author Shazia Ruybal-Pesantez, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU using ETH beads
#' mfi_to_rau <- MFItoRAU(
#'   sero_data = sero_data,
#'   plate_list = plate_list,
#'   qc_results = qc_results
#' )
#'
#' # Step 4: Plot Model Results
#' plotModel(mfi_to_rau, sero_data)
#' }
plotModel <- function(mfi_to_rau_output, sero_data){

  model_results <- mfi_to_rau_output[[3]]

  combined_data <- model_results %>%
    dplyr::mutate(
      Plate = gsub("^plate", "", Plate)
    )

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  ### Get Standards for points
  stds_file <- sero_data$stds
  stds_log <-
    stds_file %>%
    dplyr::mutate(across(-c(Location, Sample, Plate), ~ as.numeric(.))) %>%
    tidyr::pivot_longer(-c(Location, Sample, Plate), names_to = "Antigen", values_to = "stdcurve") %>%
    dplyr::mutate(
      dilution = case_when(
        Sample == "S1" ~ 1/50,
        Sample == "S2" ~ 1/100,
        Sample == "S3" ~ 1/200,
        Sample == "S4" ~ 1/400,
        Sample == "S5" ~ 1/800,
        Sample == "S6" ~ 1/1600,
        Sample == "S7" ~ 1/3200,
        Sample == "S8" ~ 1/6400,
        Sample == "S9" ~ 1/12800,
        Sample == "S10" ~ 1/25600,
        .default = NA
      ),
      Antigen = dplyr::recode(Antigen, !!!name_lookup)
    )

  # Generate plots for each plate, grouping proteins together
  plots_model <- lapply(unique(combined_data$Plate), function(plate_name) {
    ggplot2::ggplot() +  # Use 'protein' to differentiate lines
      ggplot2::geom_line(data = subset(combined_data, Plate == plate_name), aes(x = dilution, y = exp(log.std), color = Antigen)) +
      ggplot2::geom_point(data = subset(stds_log, Plate == plate_name), aes(x = dilution, y = stdcurve, color = Antigen)) +
      ggplot2::scale_x_log10(breaks = c(1e-5, 1e-4, 1e-3, 1e-2, 0.03),
                             labels = c("0.00001", "0.0001", "0.001", "0.01", "0.03")) +
      ggplot2::scale_y_log10(breaks = c(0, 10, 100, 1000, 10000)) +
      ggplot2::labs(
        x = "Antibody Dilution",
        y = "Standard Curve (log(MFI))",
        title = paste("Standard Curves for Plate:", plate_name)
      ) +
      ggplot2::theme_bw() +
      ggplot2::facet_wrap(~ Antigen, scales = "free")  # Create a separate plot for each antigen
  })


  # Assign names to the list of plots for clarity
  names(plots_model) <- unique(combined_data$Plate)

  return(plots_model)
}
#' Plot the Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
#' Results Data based on ETH standard
#'
#' This function gets the Median Fluorescent Intensity (MFI) to Relative
#' Antibody Units (RAU) model results data and plots the model fits based on
#' `MFItoRAU_Adj.`
#'
#' @param mfi_to_rau_output Output from `MFItoRAU_Adj()`.
#' @param sero_data Output from `readSeroData()`.
#' @return List of dot and line plots of MFI to RAU model standard curve,
#' with each one representing an individual plate (ggplot).
#' @export
#' @import ggplot2
#' @importFrom dplyr bind_rows
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU using ETH beads
#' mfi_to_rau <- MFItoRAU_Adj(
#'   sero_data = sero_data,
#'   plate_list = plate_list,
#'   qc_results = qc_results
#' )
#'
#' # Step 4: Plot Model Results
#' plotModel_Adj(mfi_to_rau, sero_data)
#' }
plotModel_Adj <- function(mfi_to_rau_output, sero_data){

  # Load model results
  model_results <- mfi_to_rau_output[[3]]

  # Generate plots for each plate, grouping antigens together
  plots_model <- lapply(unique(model_results$Plate), function(plate_name) {
    ggplot2::ggplot(
      data = subset(model_results, Plate == plate_name),
      aes(x = dilution, y = mfi_pred, color = antigen)
    ) +  # Use 'Antigen' to differentiate lines
      ggplot2::geom_line() +
      ggplot2::scale_x_log10() +
      ggplot2::scale_y_log10(breaks = c(0, 10, 100, 1000, 10000)) +
      ggplot2::geom_point(data = subset(model_results, Plate == plate_name), aes(x = dilution, y = mfi, color = antigen)) +
      ggplot2::labs(
        x = "Antibody Dilution",
        y = "Standard Curve (MFI)",
        fill = "Antigen",
        title = paste("Standard Curves for Plate:", plate_name)
      ) +
      ggplot2::theme_bw() +
      ggplot2::facet_wrap(~ antigen, scales = "free_y")  # Create a separate plot for each Antigen
  })

  # Assign names to the list of plots for clarity
  names(plots_model) <- unique(model_results$Plate)

  return(plots_model)
}

#' Generate QC PDF Report
#'
#' @param raw_data A string with the raw data path.
#' @param platform A string: "magpix", "intelliflex", or "bioplex".
#' @param plate_layout A string with the plate layout path.
#' @param experiment_name A string for experiment name.
#' @param date A string or Date. Defaults to today's date.
#' @param experiment_notes A string of notes. Default is "no notes".
#' @param location A string for experiment location: "ETH" or "PNG" accepted.
#' @param std_point Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.
#' @param path Output path for the PDF file. Defaults to current working directory.
#'
#' @return Rendered PDF report.
#' @export
#'
#' @importFrom dplyr filter select all_of rename
#' @importFrom knitr kable asis_output
#' @importFrom rmarkdown render
#' @importFrom here here
#' @importFrom kableExtra kable_styling
#' @importFrom janitor clean_names
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' ## Not run on CRAN because it requires interactive rendering and can be slow:
#' \dontrun{
#'   # Example raw data files (MAGPIX platform)
#'   your_raw_data <- c(
#'     system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'     system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'     system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#'   )
#'
#'   # Example plate layout file
#'   your_plate_layout <- system.file(
#'     "extdata",
#'     "example_platelayout_1.xlsx",
#'     package = "SeroTrackR"
#'   )
#'
#'   # Generate the QC PDF report
#'   renderQCReport(
#'     raw_data     = your_raw_data,
#'     plate_layout = your_plate_layout,
#'     platform     = "magpix",
#'     location     = "ETH"
#'   )
#' }
renderQCReport <- function(
    raw_data,
    plate_layout,
    platform,
    experiment_name = "experiment1",
    date = format(Sys.Date(), "%Y%m%d"), # default to today's date
    experiment_notes = "no notes",
    location,
    std_point = 10,
    path = "." # Default to current working directory
  ) {

  ###############################################################################
  # ----- Load Data functions -----
  ###############################################################################

  sero_data                 <- readSeroData(raw_data, platform)
  raw_data_info             <- sero_data$data_raw %>% janitor::clean_names()
  raw_data_filename         <- tolower(basename(raw_data))
  plate_list                <- readPlateLayout(plate_layout, sero_data)
  version                   <- getGithubRelease("dionnecargy", "PvSeroApp")

  qc_results                <- runQC(sero_data, plate_list)
  if(location == "ETH"){
    mfi_to_rau_output       <- suppressMessages(MFItoRAU_Adj(sero_data, plate_list, qc_results, std_point, project = NULL))
  } else if(location == "PNG"){
    mfi_to_rau_output       <- suppressWarnings(MFItoRAU(sero_data, plate_list, qc_results, std_point, project = NULL))
  }

  stdcurve_plot             <- suppressWarnings(plotStds(sero_data, location, experiment_name))
  plateqc_plot              <- plotCounts(qc_results, experiment_name)
  check_repeats_output      <- getRepeats(qc_results, plate_list)
  blanks_plot               <- plotBlanks(sero_data, experiment_name)
  if(location == "ETH"){
    model_plot              <- plotModel_Adj(mfi_to_rau_output, sero_data)
  } else if(location == "PNG"){
    model_plot              <- plotModel(mfi_to_rau_output, sero_data)
  }

  ###############################################################################
  # ----- Create helper functions -----
  ###############################################################################

  operator_output <- function() {
    if (platform %in% c("magpix", "intelliflex")) {
      op <- raw_data_info %>%
        dplyr::filter(program == "Operator") %>%
        dplyr::select(plate, Operator = x_ponent)
      paste(paste0(op$plate, ": ", op$Operator), collapse = ", ")
    } else {
      return("Information not available for bioplex machine run.")
    }
  }

  volume_output <- function() {
    if (platform == "magpix") {
      vol <- raw_data_info %>%
        dplyr::filter(program == "SampleVolume") %>%
        dplyr::select(plate, `Acquisition Volume` = x_ponent)
      paste(paste0(vol$plate, ": ", vol$`Acquisition Volume`), collapse = ", ")
    } else if (platform == "intelliflex") {
      vol <- raw_data_info %>%
        dplyr::filter(program == "MaxSampleUptakeVolume") %>%
        dplyr::select(plate, `Acquisition Volume` = x_ponent)
      paste(paste0(vol$plate, ": ", vol$`Acquisition Volume`), collapse = ", ")
    } else {
      return("Information not available for bioplex machine run.")
    }
  }

  calibration_output <- function() {
    if (platform == "magpix") {
      calib <- raw_data_info %>%
        dplyr::filter(program %in% c("Last CAL Calibration", "Last VER Verification", "Last Fluidics Test")) %>%
        dplyr::select(plate, program, Result = x_ponent)
      paste(paste0(calib$plate, ": ", calib$program, ": ", calib$Result), collapse = ", ")
    } else if (platform == "intelliflex") {
      calib <- raw_data_info %>%
        dplyr::filter(program %in% c("Last Calibration", "Last Verification", "Last Fluidics Test")) %>%
        dplyr::select(plate, program, Result = x_ponent)
      paste(paste0(calib$plate, ": ", calib$program, ": ", calib$Result), collapse = ", ")
    } else {
      return("Information not available for bioplex machine run.")
    }
  }

  machine_output <- function() {
    if (platform == "magpix") {

      # String to search for
      search_str <- "MachineSerialNo"
      # Find cols with search string in any row
      matching_cols <- names(raw_data_info)[sapply(raw_data_info, function(col) any(grepl(search_str, col)))]
      # Filter the data frame to include only those cols
      filtered_df <- raw_data_info %>% dplyr::select(dplyr::all_of(matching_cols))
      # Get the unknown column name
      col_name <- names(filtered_df)[1]
      # Find row indices where the string appears
      matching_indices <- which(filtered_df[[col_name]] == search_str)
      # Get indices of the rows BELOW the matching rows
      below_indices <- matching_indices + 1
      # Remove indices that are out of bounds (i.e., last row has no row below it)
      below_indices <- below_indices[below_indices <= nrow(filtered_df)]
      # Filter the data frame for these rows
      machine <- filtered_df[below_indices, , drop = FALSE] %>% dplyr::rename(`Machine Serial Number` = col_name)
      machine_levels <- unique(raw_data_info$plate)
      paste(paste0(machine_levels, ": ", machine$`Machine Serial Number`), collapse = ", ")

    } else if (platform == "bioplex") {

      machine <- raw_data_info %>%
        dplyr::filter(str_detect(run_column, "Reader Serial Number")) %>%
        dplyr::mutate(run_column = gsub("Reader Serial Number: ", "", run_column)) %>%
        dplyr::select(run_column)
      machine_levels <- unique(raw_data_info$plate)
      paste(paste0(machine_levels, ": ", machine$run_column), collapse = ", ")

    } else if (platform == "intelliflex") {
      machine <- raw_data_info %>%
        dplyr::filter(program == "SN") %>%
        dplyr::select(plate, `Machine Serial Number` = x_ponent)
      paste(paste0(machine$plate, ": ", machine$`Machine Serial Number`), collapse = ", ")

    }
  }

  check_repats_table_pdf <- function(check_repeats_output) {
    if (is.data.frame(check_repeats_output)) {
      return(check_repeats_output)
    } else {
      return(NULL)
    }
  }

  plate_list_output <- function() {
    tables_output <- lapply(seq_along(plate_list), function(i) {
      table_header <- paste0("##### Plate: ", i, "\n\n")
      table_content <- knitr::kable(plate_list[[i]], format = "latex", booktabs = TRUE)
      paste0(table_header, table_content)
    })
    knitr::asis_output(paste(tables_output, collapse = "\n\n"))
  }

  ###############################################################################
  # ----- Generate PDF Document -----
  ###############################################################################

  suppressWarnings(
    rmarkdown::render(
      input = system.file("rmd/template.Rmd", package="SeroTrackR"),
      output_file = paste0(experiment_name, "_", date, "_", location, "_", version, "_QCreport.pdf"),
      output_dir = here::here(path),
      params = list(
        raw_data_filename        = raw_data_filename,
        experiment_name          = experiment_name,
        date                     = date,
        experiment_notes         = experiment_notes,
        platform                 = platform,
        stdcurve_plot            = stdcurve_plot,
        plateqc_plot             = plateqc_plot,
        blanks_plot              = blanks_plot,
        check_repeats_output     = check_repeats_output,
        check_repats_table_pdf   = check_repats_table_pdf(check_repeats_output),
        model_plot               = model_plot,
        operator_output          = operator_output(),
        volume_output            = volume_output(),
        calibration_output       = calibration_output(),
        machine_output           = machine_output(),
        plate_list_output        = plate_list_output()
      )
    )
  )

}

#' All Classification Data
#'
#' This function runs the classification algorithm for all possible sensitivity
#' and specificity options.
#'
#' @param mfi_to_rau_output Output from `MFItoRAU()` or `MFItoRAU_Adj()`.
#' @param algorithm_type Algorithm: "antibody_model" (PvSEM algorithm; default)
#' @param qc_results Output from `runQC()`.
#' @return A table of all classification outputs.
#' @export
#' @importFrom dplyr group_by summarise select mutate
#' @importFrom tidyr pivot_wider
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load in Raw Data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")
#'
#' # Step 1: Reading in Raw Data
#' sero_data     <- readSeroData(raw_data = your_raw_data, "magpix")
#' plate_list    <- readPlateLayout(
#'   plate_layout = your_plate_layout,
#'   sero_data = sero_data
#' )
#'
#' # Step 2: Quality Control and MFI to RAU
#' qc_results <- runQC(sero_data, plate_list)
#'
#' # Step 4: Run MFI to RAU (e.g., using ETH beads)
#' mfi_to_rau_output  <- MFItoRAU_Adj(sero_data, plate_list, qc_results)
#'
#' # Step 5: Render classification table
#' renderClassificationTable(
#'   mfi_to_rau_output = mfi_to_rau_output,
#'   algorithm_type = "antibody_model",
#'   qc_results = qc_results
#' )
#'
#'}
#'
renderClassificationTable <- function(mfi_to_rau_output, algorithm_type, qc_results){
  # Load All sens_spec possibilities to cycle through
  sens_spec_all <- c("balanced", "90% specificity")

  # Run classify_final_results
  all_classifications <- purrr::map_dfr(sens_spec_all, ~{
    classifyResults(
      mfi_to_rau_output = mfi_to_rau_output,
      algorithm_type = algorithm_type,
      sens_spec = .x,
      qc_results = qc_results
    ) %>%
      as.data.frame() %>%  # Ensure it's a data frame
      dplyr::mutate(sens_spec = .x)  # Add the sens_spec column
  })
  # Return the combined data frame
  all_classifications %>%
    dplyr::group_by(sens_spec, pred_class_max) %>%
    dplyr::summarise(n = n()) %>%
    tidyr::pivot_wider(names_from = pred_class_max, values_from = n) %>%
    dplyr::select(`Sensitivity/Specificity` = sens_spec,
                  Seropositive = seropositive,
                  Seronegative = seronegative)
}

#' Run LDH Pipeline from Start to End
#'
#' A master function combining the entire LDH pipeline into one command to run in R.
#'
#' @param raw_data  String with the raw data path.
#' @param plate_layout An ".xlsx" file with sheets labelled plate1, plate2... etc.
#' @param platform  "magpix" or "bioplex". Default: "Bioplex"
#' @param dilution  A list of numbers ranging from S1 to S10. Default: 1000000, 333333.33, 111111.11, 37037.04, 12345.68, 4115.23, 1371.74, 457.25, 152.42, 50.81.
#' @param experiment_name User-input experiment name. Default: "experiment1".
#' @param file_path A file path to write the .csv final file. Default: Current working directory.
#'
#' @return A data frame containing the MFI and RAU Dilution values for each sample, QC plots for standard curve, bead counts and blanks.
#' @export
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Example input files
#' your_raw_data <- system.file(
#'   "extdata",
#'   "example_BioPlex_PvLDH_plate1.xlsx",
#'   package = "SeroTrackR"
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Run full LDH processing pipeline
#' runLDHPipeline(
#'   raw_data      = your_raw_data,        # Vector of raw data files
#'   plate_layout  = your_plate_layout,    # Plate layout file
#' )
#'
#' }
runLDHPipeline <- function(
    raw_data,
    plate_layout,
    platform = "bioplex",
    dilution = c(1000000, 333333.33, 111111.11, 37037.04, 12345.68, 4115.23, 1371.74, 457.25, 152.42, 50.81),
    experiment_name = "experiment1",
    file_path = NULL
){

  #############################################################
  # Step 1: Reading in Raw Data
  #############################################################
  sero_data                 <- readSeroData(raw_data, platform)
  plate_list                <- readPlateLayout(plate_layout, sero_data)

  #############################################################
  # Step 2: Quality Control
  #############################################################
  qc_results                <- runQC(sero_data, plate_list)

  #############################################################
  # Step 3: Plotting
  #############################################################
  stdcurve_plot             <- suppressWarnings(plotStds_all(sero_data, experiment_name))
  plateqc_plot              <- plotCounts(qc_results, experiment_name)
  check_repeats_output      <- getRepeats(qc_results, plate_list)
  blanks_plot               <- plotBlanks(sero_data, experiment_name)

  #############################################################
  # Step 4: MFI to RAU Conversion
  #############################################################
  mfitorau_output           <- MFItoRAU_LDH(sero_data, plate_list, dilution, file_path)

  return(list(stdcurve_plot, plateqc_plot, check_repeats_output, blanks_plot, mfitorau_output))
}

#' Median Fluorescent Intensity (MFI) Box Plots
#'
#' Boxplot of the MFI values.
#'
#' @param mfi_to_rau_output Output from `MFItoRAU()` or `MFItoRAU_Adj()`
#'.
#' @param location "PNG" or "ETH".
#' @return Box plots with MFI values for each protein (ggplot).
#' @export
#' @importFrom dplyr select rename_with ends_with mutate
#' @importFrom tidyr pivot_longer
#' @importFrom stringr str_replace
#' @import ggplot2
#' @author Dionne Argyropoulos
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU using ETH beads
#' mfi_to_rau <- MFItoRAU_Adj(
#'   sero_data   = sero_data,
#'   plate_list  = plate_list,
#'   qc_results  = qc_results
#' )
#'
#' # Step 4: Plot MFI values
#' plotMFI(mfi_to_rau, "MFI")
#' }
plotMFI <- function(mfi_to_rau_output, location){

  df_results <- mfi_to_rau_output[[2]]

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  df_results <- df_results %>%
    dplyr::select(SampleID, Plate, ends_with("_MFI")) %>%
    dplyr::rename_with(~str_replace(., "_MFI", ""), ends_with("_MFI")) %>%
    tidyr::pivot_longer(-c(SampleID, Plate), names_to = "Antigen", values_to = "MFI") %>%
    dplyr::mutate(
      Plate = factor(Plate, levels = unique(Plate[order(as.numeric(str_extract(Plate, "\\d+")))])), # Reorder by plate number
      MFI = as.numeric(MFI)
    )

  df_wehi <- read.csv(system.file("extdata", "longitudinal_MFI.csv", package = "SeroTrackR")) %>%
    dplyr::mutate(Antigen = dplyr::recode(Antigen, !!!name_lookup))

  plot <- df_results %>%
    ggplot2::ggplot(aes(x= Antigen, y = MFI)) +
    ggplot2::geom_boxplot(data = df_wehi, aes(x = Antigen, y = MFI), fill = "grey", colour = "darkgrey") +
    ggplot2::geom_boxplot(aes(fill = Antigen)) +
    ggplot2::scale_y_log10(breaks = c(10, 100, 1000, 10000), limits = c(10, 10000), labels = c("10", "100", "1,000", "10,000")) +
    ggplot2::scale_fill_brewer(palette = "Paired", type = "qual") +
    ggplot2::labs(x = "Antigen", y = "Antibody MFI") +
    ggplot2::facet_wrap( ~ Plate) +
    ggplot2::theme_bw() +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1), legend.position = "none")

  return(plot)

}

utils::globalVariables(c(
  ".", ".pred_new", "1", "12",
  "Antigen", "Antigens", "Beads", "Col", "Content", "Count", "df",
  "Dilution", "Location", "Location.2", "MFI", "Plate", "Program",
  "QC_total", "RAU", "RAU_Method", "Repeat", "Row", "Run", "Sample",
  "SampleID", "Sens_Spec", "Species", "Sum", "Text", "Type", "Warning",
  "antigen", "asym_par", "content", "data_type", "delete", "dilution",
  "div", "ed50", "error", "eth_fit", "fct_rev", "file_name", "fromJSON",
  "glue", "log_mfi", "log.std", "low_asym", "max_dil", "max_s1", "mfi",
  "mfi_pred",  "name", "name_lookup", "new_fit", "plate",  "platemap",
  "plate_list", "png_fit", "pred_class_max", "prefix", "program", "protein",
  "rau", "rau_restricted", "raw_data_filenames", "ref_mfi", "RunColumn",
  "run_column", "sample_id", "sens_spec", "seronegative", "seropositive",
  "slope", "species", "standard_cat", "standard_dil", "standard_type",
  "status_code", "std_sample", "stdcurve", "stds_to_keep", "suffix",
  "to_remove", "type.letter", "upp_asym", "value", "well", "xPONENT",
  "x_ponent"
))

#' Create two columns in Fluent UI
#'
#' This function creates two columns in the Fluent UI format.
#'
#' @param first_col A list of content for the first column.
#' @param second_col A list of content for the second column.
#' @param first_width Percent width of the column space (default: 50\%).
#' @param second_width Percent width of the column space (default: 50\%).
#' @return Fluent UI window with two columns.
#' @export
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' # Minimal example using htmltools elements.
#' # This example runs without starting a Shiny app and is safe for CRAN.
#'
#' if (requireNamespace("shiny.fluent", quietly = TRUE) &&
#'     requireNamespace("htmltools", quietly = TRUE)) {
#'
#'   # Create simple content for each column
#'   col1 <- list(htmltools::div("First column content"))
#'   col2 <- list(htmltools::div("Second column content"))
#'
#'   # Render two columns with default widths
#'   renderTwoCols(first_col = col1, second_col = col2)
#' }
renderTwoCols <- function(
    first_col,
    second_col,
    first_width = "50%",
    second_width = "50%"
) {

  # Check if shiny.fluent is installed
  if (!requireNamespace("shiny.fluent", quietly = TRUE)) {
    stop("Package 'shiny.fluent' is required for renderTwoCols(). Please install it.", call. = FALSE)
  }

  # Check if htmltools is installed
  if (!requireNamespace("htmltools", quietly = TRUE)) {
    stop("Package 'htmltools' is required for renderTwoCols(). Please install it.", call. = FALSE)
  }

  # Use functions from shiny.fluent with :: prefix
  shiny.fluent::Stack(
    horizontal = TRUE,
    tokens = list(childrenGap = 40),
    children = list(
      # First Column
      htmltools::div(
        tokens = list(childrenGap = 15),
        style = list(width = first_width),
        children = first_col  # First column content
      ),
      # Second Column
      htmltools::div(
        tokens = list(childrenGap = 15),
        style = list(width = second_width),
        children = second_col  # Second column content
      )
    )
  )
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import dplyr
#' @import tidyr
#' @import ggplot2
#' @import stringr
#' @importFrom stats coef na.omit optim predict setNames
#' @importFrom utils read.csv data
#' @importFrom magrittr %>%
## usethis namespace: end
NULL

#' Plot Raw Median Fluorescent Intensity Blanks Data
#'
#' This function gets the blank sample data and plots the blank sample Median
#' Fluorescent Intensity (MFI) values.
#'
#' @param sero_data Output from `readSeroData()`.
#' @param experiment_name User-input experiment name.
#' @return Bar plot showing whether MFI values for the blanks for each antigen
#' per plate is above or below the threshold MFI = 50 (ggplot).
#' @export
#' @import ggplot2
#' @importFrom dplyr select mutate
#' @importFrom tidyr pivot_longer
#' @author Shazia Ruybal-Pesantez, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' # Example demonstrating how to process bead count data.
#' # These files are included in the SeroTrackR package under inst/extdata.
#'
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' # Read in raw MAGPIX data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
#'
#' # Plot blanks
#' plotBlanks(
#'   sero_data = sero_data,
#'   experiment_name = "experiment1"
#' )
#'
#' }
#'
plotBlanks <- function(sero_data, experiment_name){
  master_file <- sero_data
  blanks <- master_file$blanks

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  blanks %>%
    dplyr::select(-Location) %>%
    tidyr::pivot_longer(-c(Sample, Plate), names_to = "Antigen", values_to = "MFI") %>%
    dplyr::mutate(
      Plate = factor(Plate, levels = unique(Plate[order(as.numeric(str_extract(Plate, "\\d+")))])),  # Reorder by plate number
      Antigen = dplyr::recode(Antigen, !!!name_lookup)
    ) %>%
    ggplot2::ggplot(aes(x = factor(Antigen), y = as.numeric(MFI), fill = Sample)) +
    ggplot2::geom_bar(stat = "identity", position = "dodge") +
    ggplot2::geom_hline(yintercept = 50, linetype = "dashed", color = "grey") +
    ggplot2::labs(
      x = "Antigen",
      y = "MFI",
      title = experiment_name
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1)) +
    ggplot2::facet_wrap(~ Plate)  # Create separate facets for each 'plate'
}

#' Read Raw Serological Data
#'
#' This function imports the raw data from the Magpix or Bioplex machine
#' and matches the sample names from the plate layout based on their plate/well
#' location.
#'
#' @param raw_data String with the raw data path.
#' @param raw_data_filenames String with the raw data filename path.
#' Default is NA as it can be deduced from raw_data. Needs to be a parameter for the PvSeroApp.
#' @param version xPONENT software version. For "magpix" can be 4.2 or 4.3. Default: 4.2.
#' @param platform "magpix", "bioplex" or "intelliflex".
#'
#' @return List of data frames: (i) raw data output, (ii) cleaned all results
#' (iii) count data, (iv) blanks only, (v) standards only, (vi) run
#' information.
#'
#' @export
#'
#' @author Dionne Argyropoulos, Shazia Ruybal-Pesantez
#'
#' @examples
#' # Example raw data files (MAGPIX platform)
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
#' )
#'
#' # Read and combine raw serology data
#' sero_data <- readSeroData(
#'   raw_data = your_raw_data,
#'   platform = "magpix"
#' )
readSeroData <- function(raw_data, platform, version = "4.2", raw_data_filenames = NULL){
  platemap_file <- system.file("extdata", "platemap.csv", package = "SeroTrackR")
  platemap <- read.csv(platemap_file)

  raw_data_filenames <- tolower(
    if (is.null(raw_data_filenames)) basename(raw_data) else raw_data_filenames
  )

  # Initialise master list to store files
  master_list <- list(
    data_raw  = NULL,  # Placeholder for raw data combined across files
    results   = NULL,  # Placeholder for processed results combined
    counts    = NULL,  # Placeholder for any count data combined
    blanks    = NULL,  # Placeholder for any blanks data combined
    stds      = NULL,  # Placeholder for any stds data combined
    run       = NULL   # Placeholder for any run data combined
  )

  # Loop through each file and process accordingly
  for (i in seq_along(raw_data)) {
    file <- raw_data[i]
    file_name <- raw_data_filenames[i]

    if (.check_platform(file, platform, file_name) == TRUE) {
      message("PASS: File ", file_name, " successfully validated.")
    }

    if (platform == "magpix") {

      cfg         <- .magpix_version_config(version)
      df          <- .read_luminex_file(file)
      sections    <- .extract_luminex_sections(df, cfg, "magpix")
      master_list <- .post_process_luminex(sections, file_name, master_list)

    } else if (platform == "bioplex"){

      df          <- .read_luminex_file(file)
      sections    <- .post_process_bioplex(df)
      master_list <- .post_process_luminex(sections, file_name, master_list)

    } else if (platform == "intelliflex"){

      df          <- .read_luminex_file(file)
      sections    <- .extract_luminex_sections(df, cfg, "intelliflex")
      master_list <- .post_process_luminex(sections, file_name, master_list)

    } else {
      stop(
        paste0(
          "Unsupported platform: '", platform, "'.\n",
          "  Supported platforms are: 'magpix', 'bioplex', or 'intelliflex'.\n",
          "  Please check your platform argument and try again."
        ),
        call. = FALSE
      )
    }

  }

  return(master_list)

}
#' Check Platform
#'
#' This function checks the platform the user has input and whether it aligns
#' with the correct format as expected. Will report error if NOT aligned.
#'
#' @param raw_data String with the raw data path.
#' @param platform "magpix", "bioplex" or "intelliflex".
#' @param file_name String with the raw data filename (for error messaging).
#'
#' @return TRUE: if platform == file format, ERROR message when platform does
#' not equal file format.
#' @export
#'
#' @importFrom readxl read_excel
#' @importFrom janitor row_to_names
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
#' .check_platform(raw_data = your_raw_data, platform = "magpix", file_name = basename(your_raw_data))
.check_platform <- function(raw_data, platform, file_name) {

  if (length(raw_data) == 0) {
    stop("No raw data files were provided.", call. = FALSE)
  }

  # Read file
  df <- .read_luminex_file(raw_data)

  # Extract the first two column names
  col_names <- colnames(df)
  if (all(grepl("^X\\d+$", col_names))) {
    df <- suppressWarnings(df %>% janitor::row_to_names(row_number = 1))
  }
  first_two_cols <- colnames(df)[1:2]

  # Detect if the file is Magpix based on column names
  is_magpix <- any(grepl("Program|row", first_two_cols, ignore.case = TRUE)) ||
    any(grepl("xPonent|col", first_two_cols, ignore.case = TRUE))

  # Detect if file is Bioplex (inverse of Magpix detection)
  is_bioplex <- !is_magpix

  # User selected "magpix" but the file does not have "Program" or "xPonent"
  if (platform == "magpix" && !is_magpix) {
    stop(
      paste0(
        "Platform mismatch for file '", file_name, "':\n",
        "  You specified platform = 'magpix', but this file appears to be 'bioplex' or 'intelliflex'.\n",
        "  Please try platform = 'bioplex' or platform = 'intelliflex'.\n",
        "  MagPix files contain columns like 'xPONENT', 'Program', or 'row'."
      ),
      call. = FALSE
    )
  }

  # User selected "bioplex" but the file contains "Program" or "xPonent"
  if (platform == "bioplex" && is_magpix) {
    stop(
      paste0(
        "Platform mismatch for file '", file_name, "':\n",
        "  You specified platform = 'bioplex', but this file appears to be 'magpix'.\n",
        "  Please try platform = 'magpix'.\n",
        "  Bioplex files do not contain 'xPONENT' or 'Program' columns."
      ),
      call. = FALSE
    )
  }

  # User selected "intelliflex" but file is Magpix
  if (platform == "intelliflex" && is_magpix) {
    stop(
      paste0(
        "Platform mismatch for file '", file_name, "':\n",
        "  You specified platform = 'intelliflex', but this file appears to be 'magpix'.\n",
        "  Please try platform = 'magpix'.\n",
        "  Intelliflex files have a different structure than Magpix files."
      ),
      call. = FALSE
    )
  }

  return(TRUE)

}
#' Helper function to read raw luminex files
#'
#' @param file String with the raw data path.
#'
#' @returns raw data frame
#' @export
#'
#' @importFrom tools file_ext
#' @importFrom readxl read_excel
#' @importFrom dplyr filter
#' @importFrom utils read.csv2 count.fields
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
#' df <- .read_luminex_file(your_raw_data)
.read_luminex_file <- function(file) {
  ext <- tools::file_ext(file)

  if (ext == "xlsx") {
    df <- suppressMessages(readxl::read_excel(file)) %>% as.data.frame()
  } else if (ext == "csv") {
    first_lines <- readLines(file, n = 5)
    df <- if (any(grepl(";", first_lines))) {
      suppressWarnings(read.csv2(file, header = F, col.names = paste0("x", 1:max(count.fields(file, sep = ";"),na.rm = T)), fill = T)%>%
                         janitor::row_to_names(1))
    } else {
      suppressWarnings(read.csv(file, header = F, col.names = paste0("x", 1:max(count.fields(file, sep = ","),na.rm = T)), fill = T) %>%
                         janitor::row_to_names(1))
    }
    colnames(df) <- make.names(colnames(df), unique = T)
    df <- dplyr::filter(df, rowSums(is.na(df)) != ncol(df))
  } else {
    stop("Unsupported file format! Please use .csv or .xlsx", call. = FALSE)
  }

  df
}
#' Relabel column names to Standardised Naming Convention
#'
#' This is a helper function to be used inside `readSeroData()` to relabel
#' columns for each plate.
#'
#' @param df  Data frame from `readSeroData()` processing.
#'
#' @returns A data fame with columns renamed
#' @export
#'
#' @importFrom stringr str_detect
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#' your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
#'
#' if (
#'  requireNamespace("dplyr", quietly = TRUE) &&
#'  requireNamespace("janitor", quietly = TRUE)
#' ) {
#'
#'   # Read in raw luminex file
#'   df <- .read_luminex_file(your_raw_data)
#'
#'   # Get the start and end rows of the data section: start = "Median", end = "Net MFI"
#'   row1    <- which(df$xPONENT == "Median")
#'   row2    <- which(df$xPONENT == "Net MFI")
#'
#'   # Apply data processing pipeline, including .relabel_columns()
#'   df |>
#'     dplyr::slice((row1 + 1):(row2 - 1)) |>
#'     janitor::row_to_names(row_number = 1) |>
#'     janitor::clean_names() |>
#'     dplyr::select(dplyr::where(~ !all(is.na(.x)))) |>
#'     dplyr::filter(dplyr::if_any(dplyr::everything(), ~ !is.na(.x))) |>
#'     dplyr::mutate(dplyr::across(everything(), ~ gsub("NaN", 0, .))) |>
#'     .relabel_columns()
#' }
#'}
.relabel_columns <- function(df) {
  colnames(df) <- dplyr::case_when(
    stringr::str_detect(colnames(df), regex("EBP", ignore_case = TRUE)) ~ "EBP",
    stringr::str_detect(colnames(df), regex("(LF005|Pv.fam.a|fam.a|Pv-fam-a)", ignore_case = TRUE)) ~ "LF005",
    stringr::str_detect(colnames(df), regex("(LF010|MSP5)", ignore_case = TRUE)) ~ "LF010",
    stringr::str_detect(colnames(df), regex("(LF016|PvMSP1-19|PvMSP1.19)", ignore_case = TRUE)) ~ "LF016",
    stringr::str_detect(colnames(df), regex("(MSP8|L34)", ignore_case = TRUE)) ~ "MSP8",
    stringr::str_detect(colnames(df), regex("(P87|RBP2b-P87|RBP2b|PvRBP2b)", ignore_case = TRUE)) ~ "RBP2b.P87",
    stringr::str_detect(colnames(df), regex("(PTEX|PTEX150|L18)", ignore_case = TRUE)) ~ "PTEX150",
    stringr::str_detect(colnames(df), regex("PkTRAMPCSS|PkTRAMP-CSS|PkPC", ignore_case = TRUE)) ~ "PkTRAMP-CSS",
    stringr::str_detect(colnames(df), regex("CSS", ignore_case = TRUE)) ~ "PvCSS",
    stringr::str_detect(colnames(df), regex("(PfMSP1-19|PfMSP1|PfMSP1.19)", ignore_case = TRUE)) ~ "PfMSP1-19",
    stringr::str_detect(colnames(df), regex("PfAMA1", ignore_case = TRUE)) ~ "PfAMA1",
    stringr::str_detect(colnames(df), regex("Pfetramp5Ag1|Pfetramp", ignore_case = TRUE)) ~ "Pfetramp5Ag1",
    stringr::str_detect(colnames(df), regex("HSP40Ag1", ignore_case = TRUE)) ~ "PfHSP40Ag1",
    stringr::str_detect(colnames(df), regex("PfGexp18", ignore_case = TRUE)) ~ "PfGexp18",
    stringr::str_detect(colnames(df), regex("PkSSP2", ignore_case = TRUE)) ~ "PkSSP2",
    stringr::str_detect(colnames(df), regex("PkMSP10", ignore_case = TRUE)) ~ "PkMSP10",
    stringr::str_detect(colnames(df), regex("PkRIPR", ignore_case = TRUE)) ~ "PkRIPR",
    stringr::str_detect(colnames(df), regex("Sera3ag1", ignore_case = TRUE)) ~ "PkSERA3ag1",
    stringr::str_detect(colnames(df), regex("Pk8", ignore_case = TRUE)) ~ "Pk8",
    stringr::str_detect(colnames(df), regex("SERA3Ag2", ignore_case = TRUE)) ~ "PkSERA3Ag2",
    TRUE ~ colnames(df) # Keep unmatched names as-is
  )
  return(df)
}
#' Helper function to identify Magpix version
#'
#' @param version String with the raw data path.
#'
#' @returns specific column names for filtering for xPONENT software v4.2 and v4.3
#' @export
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' version = "4.2"
#' .magpix_version_config(version)
#'
.magpix_version_config <- function(version) {
  switch(
    version,
    "4.2" = list(end_count = "Avg Net MFI"),
    "4.3" = list(end_count = "Result"),
    stop("Unsupported MagPix version", call. = FALSE)
  )
}
#' Helper function to process luminex sections
#'
#' @param df  String with the raw data path.
#' @param cfg Magpix version output of .magpix_version_config().
#' @param plt Platform (magpix, intelliflex)
#'
#' @returns List of data_raw, results, counts, blanks, stds, run
#' @export
#'
#' @importFrom dplyr slice select filter if_any everything  where as_tibble
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
#' df            <- .read_luminex_file(your_raw_data)
#' cfg           <- .magpix_version_config("4.2")
#' section       <- .extract_luminex_sections(df, cfg, "magpix")
.extract_luminex_sections <- function(df, cfg, plt) {

  if (plt == "magpix") {

    median_row_number    <- which(df$xPONENT == "Median")
    endmedian_row_number <- which(df$xPONENT == "Net MFI")
    count_row_number     <- which(df$xPONENT == "Count")
    endcount_row_number  <- which(df$xPONENT == cfg$end_count)

  } else if (plt == "intelliflex") {

    median_row_number     <- which(df$xPONENT == "Median")
    endmedian_row_number  <- which(df$xPONENT == "Mean")
    count_row_number      <- which(df$xPONENT == "Count")
    endcount_row_number   <- which(df$xPONENT == "%CV")
  }

  results <- .clean_luminex(df, median_row_number, endmedian_row_number)
  counts  <- .clean_luminex(df, count_row_number, endcount_row_number)

  run <- df %>%
    dplyr::slice(1:median_row_number) %>%
    dplyr::select(dplyr::where(~ !all(is.na(.x)))) %>%
    dplyr::filter(dplyr::if_any(dplyr::everything(), ~ !is.na(.x)))

  blanks <- results %>%
    dplyr::filter(grepl("Blank|^B$", Sample, ignore.case = TRUE))

  stds <- results %>%
    dplyr::filter(grepl("^S", Sample, ignore.case = TRUE))

  if (nrow(blanks) == 0) stop("No blanks were found.", call. = FALSE)
  if (nrow(stds) == 0)   stop("No standards were found.", call. = FALSE)

  list(
    data_raw = df,
    results  = results,
    counts   = dplyr::as_tibble(counts),
    blanks   = blanks,
    stds     = stds,
    run      = run
  )
}
#' Helper function to process luminex (Magpix/Intelliflex) data
#'
#' @param df Raw luminex file
#' @param row1 Leading row to subset
#' @param row2 Final row to subset
#'
#' @returns Cleaned data fame
#' @export
#'
#' @importFrom dplyr slice select filter if_any across everything across any_of mutate
#' @importFrom stringr str_remove
#' @importFrom janitor row_to_names
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
#' df            <- .read_luminex_file(your_raw_data)
#' cfg           <- .magpix_version_config("4.2")
#'
#' row1         <- which(df$xPONENT == "Median")
#' row2         <- which(df$xPONENT == "Net MFI")
#'
#' results      <- .clean_luminex(df, row1, row2)
#'
#' }
.clean_luminex <- function(df, row1, row2){

  df2 <- df %>%
    # Filter to correct section of df
    dplyr::slice((row1 + 1):(row2 - 1)) %>%
    #make blank cells NA so they are dropped in next line
    mutate(across(.cols = where(is.character), .fns = ~na_if(.x, ""))) %>%
    # Drop all-NA columns
    dplyr::select(dplyr::where(~ !all(is.na(.x)))) %>%
    # Drop all-NA rows
    dplyr::filter(dplyr::if_any(dplyr::everything(), ~ !is.na(.x))) %>%
    # Change all NaNs to 0s
    dplyr::mutate(dplyr::across(everything(), ~ gsub("NaN", 0, .))) %>%
    # Remove first row
    janitor::row_to_names(row_number = 1) %>%
    # Make all numeric columns numeric
    dplyr::mutate(
      dplyr::across(
        dplyr::where(~ all(is.na(.x) | grepl("^[-+]?[0-9]*\\.?[0-9]+$", .x))),
        as.numeric
      )
    ) %>%
    # Use custom function to ensure columns are SeroTrackR friendly
    .relabel_columns() %>%
    # Remove Total Events column
    dplyr::select(-dplyr::any_of("Total Events")) %>%
    # Relabel Blanks and Samples
    dplyr::mutate(
      # Sequentially relabel Blank rows and keep other Sample values unchanged
      Sample = ifelse(Sample == "Blank", paste0("Blank", row_number()),
                      ifelse(Sample == "B", paste0("Blank", row_number()), Sample)),
      # Sequentially relabel Sample rows and keep other Sample values
      Sample = ifelse(Sample == "S", paste0("S", cumsum(Sample == "S")), Sample)
    )

  return(df2)
}
#' Helper function to process bioplex data
#'
#' @param df Output from `.read_luminex.file()`
#'
#' @returns Cleaned data fame
#' @export
#'
#' @importFrom dplyr rename rename_with slice select filter across any_of bind_rows left_join arrange mutate if_else
#' @importFrom stringr str_remove str_replace_all str_remove_all str_sub
#' @importFrom janitor row_to_names
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' your_raw_data <- system.file("extdata", "example_BioPlex_plate1.xlsx", package = "SeroTrackR")
#' df            <- .read_luminex_file(your_raw_data)
#' results       <- .clean_bioplex(df)
.clean_bioplex <- function(df){

  platemap      <- read.csv(system.file("extdata", "platemap.csv", package = "SeroTrackR"))
  col_name <- colnames(df)[2]  # second column
  df2 <- df %>%
    # Rename first column
    dplyr::rename(Run = 1) %>%
    # Drop rows before "Type"
    dplyr::slice(which(.data[[col_name]] == "Type") : dplyr::n()) %>%
    # Promote first row to header
    janitor::row_to_names(row_number = 1) %>%
    # Clean column names (remove "(...)")
    dplyr::rename_with(~ stringr::str_remove(.x, "\\s*\\(.*\\)")) %>%
    # Helper function to relabel to be compatible with SeroTrackR
    .relabel_columns() %>%
    # Recode + ordering helpers so that standards and blanks are at the top
    dplyr::mutate(
      Type   = dplyr::if_else(Type == "B", "Blank", Type),
      suffix = as.numeric(stringr::str_remove_all(Type, "\\D")),
      prefix = stringr::str_sub(Type, 1, 1)
    ) %>%
    # Order so that standards and blanks are at the top
    dplyr::arrange(prefix, suffix) %>%
    # Join platemap
    dplyr::left_join(platemap, by = "Well") %>%
    # Drop unwanted columns if present
    dplyr::select(
      -dplyr::any_of(c(
        "prefix", "suffix", "Region", "Gate", "Total",
        "% Agg Beads", "Sampling Errors", "Well", "Description"
      ))
    ) %>%
    # Align with MagPix schema
    dplyr::select(Location, Sample = Type, everything()) %>%
    # Clean bad values + relabel blanks
    dplyr::mutate(
      # change "NaN" to 0s and "***" to 0s
      dplyr::across(dplyr::everything(), ~ stringr::str_replace_all(.x, c("NaN" = "0", "\\*\\*\\*" = "0"))),
      # Sequentially relabel Blank rows and keep other
      Sample = dplyr::if_else(
        Sample == "Blank",
        paste0("Blank", row_number()),
        Sample
      )
    )
  return(df2)
}
#' Helper function to process bioplex sections
#'
#' @param df Output from `.read_luminex_file()`
#'
#' @returns List of data_raw, results, counts, blanks, stds, run
#' @export
#'
#' @importFrom dplyr mutate across  where filter slice select
#'
#' @author Dionne Argyropoulos
#'
#' your_raw_data <- system.file("extdata", "example_BioPlex_plate1.xlsx", package = "SeroTrackR")
#' df            <- .read_luminex_file(your_raw_data)
#' sections      <- .post_process_bioplex(df)
.post_process_bioplex <- function(df){

  results <- .clean_bioplex(df) %>%
    dplyr::mutate(dplyr::across(-c(Location, Sample), ~ gsub("\\s*\\(.*\\)", "", .))) %>%
    dplyr::mutate(
      dplyr::across(
        dplyr::where(~ all(is.na(.x) | grepl("^[-+]?[0-9]*\\.?[0-9]+$", .x))),
        as.numeric
      )
    )
  counts  <- .clean_bioplex(df) %>%
    dplyr::mutate(dplyr::across(-c(Location, Sample), ~ gsub(".*\\((.*)\\).*", "\\1", .))) %>%
    dplyr::mutate(
      dplyr::across(
        dplyr::where(~ all(is.na(.x) | grepl("^[-+]?[0-9]*\\.?[0-9]+$", .x))),
        as.numeric
      )
    )

  blanks <- results %>% dplyr::filter(grepl("Blank", Sample, ignore.case = TRUE))

  stds <- results %>% dplyr::filter(grepl("^S", Sample, ignore.case = TRUE))

  colnames(df)[1] <- "RunColumn"
  well_row <- which(df$RunColumn == "Well")[1]
  run <- df %>%
    dplyr::slice(1:(well_row - 2)) %>%
    dplyr::select(RunColumn)


  if (nrow(blanks) == 0) stop("No blanks were found.", call. = FALSE)
  if (nrow(stds) == 0)   stop("No standards were found.", call. = FALSE)

  list(
    data_raw = df,
    results  = results,
    counts   = dplyr::as_tibble(counts),
    blanks   = blanks,
    stds     = stds,
    run      = run
  )

}
#' Helper function to process luminex into master_list
#'
#' @param sections Output from `.post_process_bioplex()`.
#' @param file_name User input file name.
#' @param master_list Intermediary df from `readSeroData()`.
#'
#' @returns List of data_raw, results, counts, blanks, stds, run
#' @export
#'
#' @importFrom stringr str_extract
#' @importFrom dplyr bind_rows mutate
#'
#' @author Dionne Argyropoulos
.post_process_luminex <- function(sections, file_name, master_list) {

  plate <- stringr::str_extract(file_name, "(?i)(repeat)?plate\\d+(?=[._-]|$)")

  add_plate <- function(x) dplyr::mutate(x, Plate = plate)

  master_list$data_raw <- dplyr::bind_rows(master_list$data_raw, add_plate(sections$data_raw))
  master_list$results  <- dplyr::bind_rows(master_list$results,  add_plate(sections$results))
  master_list$counts   <- dplyr::bind_rows(master_list$counts,   add_plate(sections$counts))
  master_list$blanks   <- dplyr::bind_rows(master_list$blanks,   add_plate(sections$blanks))
  master_list$stds     <- dplyr::bind_rows(master_list$stds,     add_plate(sections$stds))
  master_list$run      <- dplyr::bind_rows(master_list$run,      add_plate(sections$run))

  return(master_list)
}

#' Random Forest Classification
#'
#' This function classifies unknown samples as recently exposed or not
#' (Note: MFItoRAU() or MFItoRAU_Adj() needs to be run first to convert to
#' RAU).
#'
#' @param mfi_to_rau_output Output from `MFItoRAU()` or `MFItoRAU_Adj()`.
#' @param algorithm_type Algorithm: "antibody_model" (PvSEM algorithm; default)
#' @param sens_spec User-selected Sensitivity/Specificity threshold: "balanced"
#' (default) or "90\% specificity".
#' @param qc_results Output from `runQC()`.
#' @param project Default = NULL. Only write "pkpfpv" if using Pk/Pf/Pv pipeline.
#'
#' @return
#' - Data frame with exposure status for every sample.
#' - Summary table with positive/negative results for each threshold.
#' @export
#'
#' @import workflows parsnip ranger
#' @importFrom dplyr select mutate rename_with ends_with bind_cols ungroup inner_join
#' @importFrom stringr str_replace
#'
#' @author Lauren Smith, Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Step 3: Convert MFI to RAU using ETH beads
#' mfi_to_rau <- MFItoRAU_Adj(
#'   sero_data   = sero_data,
#'   plate_list  = plate_list,
#'   qc_results  = qc_results
#' )
#'
#' # Step 4: Perform Pv classification
#' pv_classified <- classifyResults(
#'   mfi_to_rau_output = mfi_to_rau,
#'   algorithm_type    = "antibody_model",
#'   sens_spec         = "balanced",
#'   qc_results        = qc_results
#' )
#' }
classifyResults <- function(
    mfi_to_rau_output,
    algorithm_type = "antibody_model",
    sens_spec = "balanced",
    qc_results,
    project = NULL
  ) {

  #############################################################################
  # Data wrangling
  #############################################################################
  rau_data <- mfi_to_rau_output[[1]]

  if (is.null(project)) {
    rau_data <- rau_data %>%
      dplyr::select(SampleID, Plate, Location.2, ends_with("_Dilution")) %>%
      dplyr::mutate(across(ends_with("_Dilution"), as.numeric)) %>%    # Convert only "_Dilution" columns to numeric
      dplyr::rename_with(~ str_replace(., "_Dilution$", ""), ends_with("_Dilution")) # Remove the "_Dilution" suffix
  } else if (project == "pkpfpv") {
    rau_data <- rau_data %>%
      dplyr::select(SampleID, Plate, Location.2, ends_with("_Adjloglog_Dilution")) %>%
      dplyr::mutate(across(ends_with("_Adjloglog_Dilution"), as.numeric)) %>%    # Convert only "_Adjloglog_Dilution" columns to numeric
      dplyr::rename_with(~ str_replace(., "_Adjloglog_Dilution$", ""), ends_with("_Adjloglog_Dilution")) # Remove the "_Adjloglog_Dilution" suffix
  } else {
    rau_data <- rau_data %>%
      dplyr::select(SampleID, Plate, Location.2, ends_with("_Dilution")) %>%
      dplyr::mutate(across(ends_with("_Dilution"), as.numeric)) %>%    # Convert only "_Dilution" columns to numeric
      dplyr::rename_with(~ str_replace(., "_Dilution$", ""), ends_with("_Dilution")) # Remove the "_Dilution" suffix
  }

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19", "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")
  new_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  name_lookup <- setNames(new_names, old_names)

  rau_data <- rau_data %>%
    rename_with(
      ~ name_lookup[.x],
      .cols = any_of(old_names)
    )

  counts_QC_output <- qc_results$getCountsQC_output

  #############################################################################
  # Load files from package
  #############################################################################
  # Check if local file exists
  local_file <- system.file("extdata", "PvSeroTaTmodel.rds", package = "SeroTrackR")

  # Check if this is a CRAN submission
  # Step 1. Reads in serostatus using the trained random forest
  if (file.exists(local_file)) {
    # Master version: use local file
    antibody_model    <- readRDS(system.file("extdata", "PvSeroTaTmodel.rds", package = "SeroTrackR"))
  } else {
    # CRAN: use URL (requires internet)
    antibody_model    <- readRDS(url("https://raw.githubusercontent.com/dionnecargy/SeroTrackR/master/inst/extdata/PvSeroTaTmodel.rds"))
  }

  # Step 2: Read in the random forest votes threshold values
  threshold_table   <- read.csv(system.file("extdata", "threshold_values.csv", package = "SeroTrackR"))

  #############################################################################
  # Model-specific functions
  #############################################################################

  # Step 3: Determine random forest votes threshold based on the algorithm_type string
  threshold <- if (sens_spec == "balanced") {
    threshold_table %>% filter(sens_spec == "max_sens_spec") %>% pull(threshold)
  } else if (sens_spec == "90% specificity") {
    threshold_table %>% filter(sens_spec == "90_spec") %>% pull(threshold)
  } else {
    stop("Invalid sensitivity/specificity type provided.")
  }

  # Step 4: Run the model
  # Retrieve the model based on the algorithm_type string
  model <- get(algorithm_type)

  #############################################################################
  # Model outputs
  #############################################################################
  # Classify rau_data using the specified model and determine seropositive / seronegative based on selected threshold
  sero_status <- predict(model, new_data = rau_data, type = "prob") |>
    dplyr::mutate(
      .keep = "none",
      pred_class_max = ifelse(.pred_new > threshold, "seropositive", "seronegative"),
      pred_class_max = as.factor(pred_class_max)
    )

  final_results <- rau_data %>%
    dplyr::bind_cols(sero_status)

  #############################################################################
  # Return the table of prediction classes and QC pass/fail
  #############################################################################
  name_lookup_2 <- setNames(old_names, new_names)

  final_classification_qc <- counts_QC_output %>%
    dplyr::ungroup() %>%
    dplyr::select(SampleID, Plate, Location.2 = Location, QC_total) %>%
    dplyr::inner_join(final_results, by = c("SampleID", "Plate", "Location.2")) %>%
    dplyr::select(-Location.2) %>%
      rename_with(
        ~ name_lookup_2[.x],
        .cols = any_of(new_names)
      )

  return(final_classification_qc)
}

#' Plot Bead Counts per Plate per Antigen
#'
#' Enhances the `plotCounts()` output by providing greater resolution,
#' displaying antigens per plate, and enabling SampleID name visibility via
#' hover (transformed to Plotly in server.R)
#'
#' @param qc_results Output from `runQC()`.
#' @return Dot plot with values > 15 threshold coloured in blue (sufficient
#' beads) and less than or equal to 15 beads coloured in red (repeat)
#' faceted by each antigen
#' (ggplot).
#' @export
#' @import ggplot2
#'
#' @author Dionne Argyropoulos
#'
#' @examples
#' \donttest{
#'
#' # Step 0: Load example raw data
#' your_raw_data <- c(
#'   system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
#'   system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
#' )
#' your_plate_layout <- system.file(
#'   "extdata",
#'   "example_platelayout_1.xlsx",
#'   package = "SeroTrackR"
#' )
#'
#' # Step 1: Read serology data and plate layout
#' sero_data  <- readSeroData(your_raw_data,"magpix")
#' plate_list <- readPlateLayout(your_plate_layout, sero_data)
#'
#' # Step 2: Process counts and perform quality control
#' qc_results  <- runQC(sero_data, plate_list)
#'
#' # Step 3: Plot Bead Counts
#' plotBeadCounts(qc_results)
#' }
plotBeadCounts <- function(qc_results){

  antigen_counts_output <- qc_results$getAntigenCounts_output

  # relabel antigen names from lab codes to proper antigen names
  old_names <- c("EBP", "LF005", "LF010", "LF016", "MSP8", "RBP2b.P87", "PTEX150", "PvCSS")
  new_names <- c("PvEBP", "Pv-fam-a", "PvMSP5", "PvMSP1-19",  "PvMSP8", "PvRBP2b", "PvPTEX150", "PvCSS")

  name_lookup <- setNames(new_names, old_names)

  antigen_counts_output <- antigen_counts_output %>%
    dplyr::mutate(
      Plate = factor(Plate, levels = unique(Plate[order(as.numeric(str_extract(Plate, "\\d+")))])), # reorder by plate number,
      Antigen = dplyr::recode(Antigen, !!!name_lookup)
    )

  antigen_counts_output %>%
    ggplot2::ggplot(
      aes(Plate, Count, colour = Repeat, alpha = Repeat, size = Repeat,
          text = paste("Sample:", SampleID, "<br>Bead Count:", Count, "<br>Location:", Location,"<br>Plate:", Plate))) +
    ggplot2::geom_hline(yintercept = 15, linetype = "dashed", colour = "#861e18") +
    ggplot2::geom_point() +
    ggplot2::scale_y_continuous(breaks = c(0, 15, 50, 100, 150, 200)) +
    ggplot2::scale_colour_manual(values = c("sufficient beads" = "#91bfdb", "repeat" = "#d73027"), drop=FALSE) +
    ggplot2::scale_alpha_manual(values = c("sufficient beads" = 0.5, "repeat" = 1)) +
    ggplot2::scale_size_manual(values = c("sufficient beads" = 1, "repeat" = 3)) +
    ggplot2::labs(x = "Plate", y = "Bead Counts", alpha = "", colour = "", size = "") +  # Add legend title
    ggplot2::facet_grid(~ Antigen) +
    ggplot2::theme_bw() +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1), legend.position = "right") + # Show legend
    ggplot2::guides(alpha = "none") +
    ggplot2::guides(size = "none")

}

#' Get GitHub Version
#'
#' A short function to obtain the github version from the repository. This is
#' a generalisable function that can be used for any version tags on a repo.
#'
#' @param repo_owner GitHub Username
#' @param repo_name GitHub Repository Name
#' @return Version tag string
#' @export
#' @author Dionne Argyropoulos
#'
#' @examples
#'
#' getGithubRelease(
#'  repo_owner = "dionnecargy",
#'  repo_name = "SeroTrackR"
#' )
#'
getGithubRelease <- function(repo_owner, repo_name) {

  # Check if httr is installed
  if (!requireNamespace("httr", quietly = TRUE)) {
    stop("Package 'httr' is required for getGitHubRelease(). Please install it.", call. = FALSE)
  }
  # Check if jsonlite is installed
  if (!requireNamespace("jsonlite", quietly = TRUE)) {
    stop("Package 'jsonlite' is required for getGitHubRelease(). Please install it.", call. = FALSE)
  }

  url <- paste0("https://api.github.com/repos/", repo_owner, "/", repo_name, "/releases/latest")
  response <- httr::GET(url)

  if (httr::status_code(response) == 200) {
    release_info <- jsonlite::fromJSON(httr::content(response, "text"))
    return(release_info$tag_name)  # Extracts the tag name (release version)
  } else {
    return(NULL)
  }
}

---
title: "PvSeroApp in R Tutorial"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{PvSeroApp in R Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(rmarkdown.html_vignette.check_title = FALSE)
setup <- function() {
  needed <- c("knitr", "rmarkdown", "tidyverse", "kableExtra")
  
  lapply(needed, function(pkg) {
    if (requireNamespace(pkg, quietly = TRUE)) {
      library(pkg, character.only = TRUE)
    }
  })
}

setup()
library(SeroTrackR)

knitr::opts_chunk$set(
  dpi = 72
)

knitr::opts_chunk$set(
  dpi = 72
)
```

## Data Analysis: `runPvSeroPipeline()` 

Run this global function `runPvSeroPipeline()` embedded within the `{SeroTrackR}` R package! This function contains all of the steps in order of how to perform the *Plasmodium vivax* serology test and treat protocol as found in our [application](https://dionnecargy.shinyapps.io/PvSeroApp/)! 

### Visualisation of the PvSeroApp Pipeline

![](../man/figures/SeroTrackR_PvSeroApp.jpeg)

### Using Tutorial Dataset: Load the Data

We will be using the build-in files in the R package for this tutorial, as shown below. 

```{r setup 1}
library(SeroTrackR)
library(tidyverse)

your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"), 
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")
```

To run your OWN data, follow the code below and uncomment (i.e., remove the hashtags): 

```{r, exec=FALSE, eval=FALSE}
your_raw_data <- c(
  "PATH/TO/YOUR/FILE/plate1.csv",
  "PATH/TO/YOUR/FILE/plate2.csv",
  "PATH/TO/YOUR/FILE/plate3.csv"
)
your_plate_layout <- "PATH/TO/YOUR/FILE/plate_layout.xlsx"
```

### Run Classification: Yes

```{r runPvSeroPipeline with classification}
final_analysis <- runPvSeroPipeline(
  raw_data = your_raw_data, 
  plate_layout = your_plate_layout, 
  platform = "magpix", 
  location = "ETH", 
  experiment_name = "experiment1", 
  classify = "Yes", 
  algorithm_type = "antibody_model", 
  sens_spec = "balanced"
)
```

#### Classification 

This is a table containing the classification results (seropositive or seronegative) for each `SampleID`. In this case, the classification results are stored in the `pred_class_max` column as we chose the `sens_spec = "balanced"`. If you change it to another type of threshold, then the suffix of that column will change accordingly. 

You will also see the relative antibody unit (RAU) values (columns with antigen names), whether the sample passed QC check (`QC_total`) and the `plate` that they were run on. 

```{r classification tab 1}
final_analysis[[1]] %>%
  head() %>% 
  kable()
```

#### Standard Curve Plot

The standard curve plots are generated from the antibody data from the standards you indicated in your plate layout (e.g. S1-S10) and Median Fluorescent Intensity (MFI) units are displayed in log10-scale. In the case of the PvSeroTaT multi-antigen panel, the antigens will be displayed and in general your standard curves should look relatively linear (only when the y-axis is on logarithmic scale). 

```{r std curve plot tab 1}
final_analysis[[2]]
```

#### Bead Counts QC Plot

A summary of the bead counts for each plate well are displayed, with blue indicating there are sufficient beads (≥15) or red when there are not enough. If any of the wells are red, they should be double-checked manually and re-run on a new plate if required.

The function will inform you whether there are "No repeats necessary" or provide a list of samples to be re-run. In the example data, the beads in plate 2 wells A1 and A2 will need to be repeated

```{r bead counts plot tab 1}
final_analysis[[3]] # Plot
final_analysis[[4]] # Samples to repeat 
```

#### Blanks QC Plot

The Median Fluorescent Intensity (MFI) units for each antigen is displayed for your blank samples. In general, each blank sample should have ≤50 MFI for each antigen, if they are higher they should be cross-checked manually.

In the example data, blank samples recorded higher MFI values for LF005 on plate 1 and should be checked to confirm this is expected from the assay.

```{r blanks qc plot tab 1}
final_analysis[[5]]
```

#### Model Output Plot

The automated data processing in this app allows you to convert your Median Fluorescent Intensity (MFI) data into Relative Antibody Units (RAU) by fitting a 5-parameter logistic function to the standard curve on a per-antigen level. The results from this log-log conversion should look relatively linear for each antigen.

```{r model output plot tab 1}
final_analysis[[6]]
```

### Run Classification: No

```{r runPvSeroPipeline without classification}
no_classification_final_analysis <- runPvSeroPipeline(
  raw_data = your_raw_data, 
  plate_layout = your_plate_layout, 
  platform = "magpix", 
  location = "ETH", 
  experiment_name = "experiment1", 
  classify = "No", ########################## key if you do NOT want any classification performed i.e., you do not have PvSeroTaT antigens 
  algorithm_type = "antibody_model", 
  sens_spec = "balanced"
)
```


#### MFI and RAU Data 

```{r mfi and rau tab 2}
no_classification_final_analysis[[1]]  %>%
  head() %>% 
  kable()
```

#### QC Plots

Repeat the same steps as above to find the QC plots! 

```{r std curve plot tab 2}
#### Standard Curve Plot
no_classification_final_analysis[[2]]

#### Bead Counts QC Plot
no_classification_final_analysis[[3]] # Plot
no_classification_final_analysis[[4]] # Samples to repeat 

#### Blanks QC Plot
no_classification_final_analysis[[5]]

#### Model Output Plot
no_classification_final_analysis[[6]]
```


### Create a PDF Report

```{r create pdf output, exec=FALSE, eval=FALSE}
renderQCReport(
  your_raw_data, 
  your_plate_layout, 
  "magpix", 
  location = "ETH",
  path = "inst/tutorials/" # defaults to your current working directory
)
```

---
title: "FAQs"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{FAQs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## How do I install R packages other than `SeroTrackR`? 

The key package that you will need to be able to run the `SeroTrackR` R package and perform any other subsequent analyses is the `tidyverse` R packages. 

```{r}
#| exec: false
#| eval: false
install.packages("tidyverse")
```

Then once you have installed the tidyverse meta-package, then you load the library in as below: 

```{r}
#| exec: false
#| eval: false
library(tidyverse)
```

There **are many** dependencies that the `SeroTrackR` R package relies on. These packages help us to wrangle our data, process our MFI data into RAU, apply the machine learning classification algorithm, and create PDF reports. 

If you are using R via RStudio, then when you load the SeroTrackR package it may prompt you to install all of these other files. Select yes if that is the case. 

If it does not prompt you automatically, or if you are using another platform to use R, then use the code below: 

```{r}
#| exec: false
#| eval: false
setup <- function(){
  needed <- c(
    # Imports: Required 
    "dplyr", "drc", "forcats", "ggplot2", "here", "janitor", 
    "kableExtra", "knitr", "magrittr", "openxlsx", "parsnip", 
    "purrr", "ranger", "readr", "readxl", "rmarkdown", "stats", 
    "stringr", "tidyr", "tidyselect", "utils", "workflows", 
    # Imports: Suggested
    "glue", "htmltools", "httr", "jsonlite", "shiny.fluent", 
    "tidyverse", "zoo"
    )
  for(package in needed){
    if(!sum(installed.packages() %in% package)){
      install.packages(package)
    }
    
    require(package, character.only = TRUE)
  }
}

setup()
```

## How do I organise my files? 

It is best practice to create an `R project (.Rproj)` file and store all of your files in there. A comprehensive tutorial on how to create an R project can be found [here](https://kzeglinski.github.io/new_wehi_r_course/session_1.html). 

You can then create a Quarto Markdown Document `data_processing.qmd` inside your R project where you can process all of your serological data. 

```bash
my_R_project
└── data/
    ├── raw_data_plate1.csv
    ├── raw_data_plate2.csv
    ├── raw_data_plate3.csv
    ├── platelayout.xlsx
└── results/
└── data_processing.qmd
```

## How can I read in my data?

Once you have an `R project (.Rproj)` setup, you can save all of your files into a new folder called "data". Then you can read in your data as follows: 

```{r}
#| exec: false
#| eval: false

my_raw_data       <- "data/my_plate1.csv"
my_plate_layout   <- "data/my_plate_layout.xlsx"
```

## Is there capability to run this without internet? 

You will need internet to initially download the R package and to process certain files which call upon the internet. 
<!-- However, once this R package is downloaded, you will only ever need internet if you are applying the classification algorithm.  -->

The developers are currently working on how to leverage a safe no-internet option that allows for the incorporation of the classification algorithm. The reason we use the internet at the moment is because the files containing the algorithm are quite **large**. 

## I have multiple plate layout files. How can I input them?

Use the `getPlateLayout()` function to create a master plate layout file to then input into the other functions in the package!

```{r}
#| exec: false 
#| eval: false
getPlateLayout("your/folder/with/plate/layouts/")
```

Here replace "your/folder/with/plate/layouts/" with the main file that contains your folders. For example, if your folder looks like this:

``` bash
my_R_project/
└── data/
    ├── plate_1/
    │   ├── raw_magpix_data_plate1.csv
    │   └── plate_layout_1.xlsx
    ├── plate_2/
    │   ├── raw_magpix_data_plate2.csv
    │   └── plate_layout_2.xlsx
    └── plate_3/
        ├── raw_magpix_data_plate3.csv
        └── plate_layout_3.xlsx
```

you would write:

```{r}
#| exec: false 
#| eval: false
getPlateLayout("data/")
```

you could ALSO write:

```{r}
#| exec: false 
#| eval: false
getPlateLayout()
```

OR:

```{r}
#| exec: false
#| eval: false
getPlateLayout(folder_path = c("plate_layout_1.xlsx", "plate_layout_2.xlsx", "plate_layout_3.xlsx"))
```

## I have multiple Luminex data types I'd like to analyse. How can I do this?

If you have, for example, both Bio-Plex and MAGPIX files and would like to analyse them both, then you can do some clever data manipulation as below: 

*Note that in this example, there is one plate layout that contains all files in it. But the same idea can apply to `readPlateLayout()`. 

**Input your Bio-Plex file/s**: 

Using a reproducible example: 

```{r}
#| message: false
#| warning: false
library(SeroTrackR)
library(tidyverse)

bioplex_raw_plates <- c(
  system.file("extdata", "example_BioPlex_plate1.xlsx", package = "SeroTrackR"),
  system.file("extdata", "example_BioPlex_plate2.xlsx", package = "SeroTrackR")
)
all_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")
```

For your data: 

```{r}
#| exec: false
#| eval: false
bioplex_raw_plates <- c(
  "data/example_BioPlex_plate1.xlsx", 
  "data/example_BioPlex_plate2.xlsx"
)
all_plate_layout <- "data/example_platelayout_1.xlsx" 
```

**Input your MAGPIX file/s**: 

Using a reproducible example: 

```{r}
#| exec: false
#| eval: false 
magpix_raw_plate     <- system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
```

For your data: 

```{r}
#| exec: false
#| eval: false
magpix_raw_plate      <- "data/example_MAGPIX_plate3.csv"
```

**Read the serological data files in**: 

```{r}
#| exec: false
#| eval: false 
# Serological data
bioplex_sero_data <- readSeroData(
  raw_data = bioplex_raw_plates,
  platform = "bioplex"
)
magpix_sero_data <- readSeroData(
  raw_data = magpix_raw_plate,
  platform = "magpix", 
  version = "4.2"
)
```

**Merge the files together**: 

```{r}
#| exec: false
#| eval: false 
sero_data_merged <- NULL
# data_raw 
sero_data_merged$data_raw <- bioplex_sero_data$data_raw %>% 
  bind_rows(magpix_sero_data$data_raw)

# results 
sero_data_merged$results <- bioplex_sero_data$results %>% 
  bind_rows(magpix_sero_data$results)

# counts 
sero_data_merged$counts <- bioplex_sero_data$counts %>% 
  bind_rows(magpix_sero_data$counts)

# blanks
sero_data_merged$blanks <- bioplex_sero_data$blanks %>% 
  bind_rows(magpix_sero_data$blanks)

# stds
sero_data_merged$stds <- bioplex_sero_data$stds %>% 
  bind_rows(magpix_sero_data$stds)

# run 
sero_data_merged$run <- bioplex_sero_data$run %>% 
  bind_rows(magpix_sero_data$run)
```

**Continue the rest of the pipeline**: 

```{r}
#| exec: false
#| eval: false 
plate_list_all  <- readPlateLayout(
  plate_layout = all_plate_layout, 
  sero_data = sero_data_merged
)

qc_results <- runQC(
  sero_data = sero_data_merged, 
  plate_list = all_plate_layout
)

mfi_to_rau_output <- MFItoRAU(
  sero_data = sero_data_merged,
  plate_list = all_plate_layout, 
  qc_results = qc_results, 
  std_point = 10
)

# etc.. 
```

---
title: "Pk/Pv/Pf Serology Tutorial"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Pk/Pv/Pf Serology Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(rmarkdown.html_vignette.check_title = FALSE)
setup <- function() {
  needed <- c("knitr", "rmarkdown", "tidyverse", "kableExtra")
  
  lapply(needed, function(pkg) {
    if (requireNamespace(pkg, quietly = TRUE)) {
      library(pkg, character.only = TRUE)
    }
  })
}

setup()
library(SeroTrackR)
```

For all of these analyses you can run as many plates as you wish. 

### Visualisation of the Pk/Pv/Pf Pipeline

![](../man/figures/SeroTrackR_Pk_Pf_Pv.jpeg)

```{r setup pk analysis, eval=FALSE}
library(SeroTrackR)
library(tidyverse)
```

### 5-Point Standard Curve

#### Step 1: Load your data!

Firstly, we will be using our example data that's in-built in the package. Here replace the `system.file()` argument with the file path for your package.

```{r}
your_raw_data_5std <- c(
  system.file("extdata", "example_MAGPIX_pk_5std_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_pk_5std_plate2.csv", package = "SeroTrackR")
)
your_plate_layout_5std <- system.file("extdata", "example_platelayout_pk_5std.xlsx", package = "SeroTrackR")
```

#### Step 2: Read your data and process MFI to RAU

This function to (a) process raw Serological data and (b) convert MFI to RAU. The `runPlasmoPipeline()` function will output three data frames:

1. All_Results: All columns of every MFI to RAU conversion 
2. MFI_RAU: Just the SampleID, Plate, MFI and RAU values per antigen
3. MFI_RAU_long: SampleID, Plate, MFI, RAU, Antigen, Species (long-format df)

```{r}
results_5stdcurve <- runPlasmoPipeline(
  raw_data = your_raw_data_5std,
  platform = "magpix",
  plate_layout = your_plate_layout_5std,
  panel = "panel1",
  std_point = 5, 
  experiment_name = "5-point standard curve"
)
```

##### Standard Curve Plot

```{r}
results_5stdcurve$std_curve
```

##### Bead Counts QC Plot

```{r}
results_5stdcurve$bead_counts
```

##### Blanks QC Ploat

```{r}
results_5stdcurve$blanks
```

##### MFI to RAU Tables

All results: 

```{r}
results_5stdcurve$mfi_outputs$All_Results %>%
  head() %>% 
  kable()
```

MFI and RAU only: 

```{r}
results_5stdcurve$mfi_outputs$MFI_RAU %>%
  head() %>% 
  kable()
```

MFI and RAU long table: 

```{r}
results_5stdcurve$mfi_outputs$MFI_RAU_long %>%
  head() %>% 
  kable()
```

### 10-Point Standard Curve

These steps are very similar to the 5-point standard curve, except where indicated. 

#### Step 1: Load your data!

```{r}
your_raw_data_10std <- c(
  system.file("extdata", "example_MAGPIX_pk_10std_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_pk_10std_plate2.csv", package = "SeroTrackR")
)
your_plate_layout_10std <- system.file("extdata", "example_platelayout_pk_10std.xlsx", package = "SeroTrackR")
```

#### Step 2: Read your data and process MFI to RAU

```{r}
results_10stdcurve <- runPlasmoPipeline(
  raw_data = your_raw_data_10std,
  platform = "magpix",
  plate_layout = your_plate_layout_10std,
  panel = "panel1",
  std_point = 10, ################################### here make sure you write 10! 
  experiment_name = "10-point standard curve"
)
```

##### Standard Curve Plot

```{r}
results_10stdcurve$std_curve
```

##### Bead Counts QC Plot

```{r}
results_10stdcurve$bead_counts
```

##### Blanks QC Ploat

```{r}
results_10stdcurve$blanks
```

##### MFI to RAU Tables

All results: 

```{r}
results_10stdcurve$mfi_outputs$All_Results %>%
  head() %>% 
  kable()
```

MFI and RAU only: 

```{r}
results_10stdcurve$mfi_outputs$MFI_RAU %>%
  head() %>% 
  kable()
```

MFI and RAU long table: 

```{r}
results_10stdcurve$mfi_outputs$MFI_RAU_long %>%
  head() %>% 
  kable()
```

---
title: "PvSeroApp Serological Assay Quality Control Report"
date: "`r format(Sys.Date(), '%d-%m-%y')`"
header-includes:
   - \usepackage[default]{sourcesanspro}
   - \usepackage[T1]{fontenc}
   - \usepackage{booktabs}
   - \usepackage{longtable}
   - \usepackage{array}
   - \usepackage{multirow}
   - \usepackage{graphicx}
   - \usepackage{float}
   - \usepackage{colortbl}
   - \usepackage{xcolor}
mainfont: SourceSansPro
output: 
  pdf_document: default
params:
  raw_data_filename: NA
  experiment_name: NA
  experiment_notes: NA
  platform: NA
  date: NA
  stdcurve_plot: NA
  plateqc_plot: NA
  blanks_plot: NA
  check_repeats_output: NA
  check_repats_table_pdf: NA
  model_plot: NA
  operator_output: NA
  volume_output: NA
  calibration_output: NA
  machine_output: NA
  plate_list_output: NA
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = F, warning = FALSE, message = FALSE, tidy = TRUE, fig.width=8, fig.height=6, fig.align="center", df_print = "kable")
library(SeroTrackR)
library(dplyr)
library(ggplot2)
library(knitr)
library(kableExtra)
library(rmarkdown)
```

***
### QC Report for experiment: `r params$experiment_name` analysed on `r params$date`
***

#### Plot of Standard Curves

```{r standard curves plot}
params$stdcurve_plot
```
\newpage
#### Plot of Model Results

```{r model results plot, results='asis'}
first_plot <- TRUE                          # Flag to track the first plot
model_plot <- params$model_plot

# Loop through each plot in model_plot and print it
for (plot_name in names(model_plot)) {
  
  if (!first_plot){                         # Insert a page break before the plot but not for the first one 
    cat("\\newpage\n")                      # Page break before each plot
  }
  
  cat(paste0("##### ", plot_name, "\n\n"))  # Adds a title for each plot
  print(model_plot[[plot_name]])            # Render the actual plot
  
  first_plot <- FALSE                       # Update flag after first plot is printed 
}
```
\newpage

#### Plot of Bead Counts

Any wells with < 15 beads/well are indicated in red, and should be double-checked manually.

```{r bead counts plot}
plateqc_plot <- params$plateqc_plot
plateqc_plot + theme(legend.position = "bottom")
```

```{r check repeats table, echo=FALSE, results='asis'}
check_repeats_output <- params$check_repeats_output
check_repats_table_pdf <- params$check_repats_table_pdf

if (is.data.frame(check_repeats_output)) {
  # If it's a table, print it using kable
  tablel <- check_repats_table_pdf %>% 
    knitr::kable(
      format = "latex",
      align = "l",
      booktabs = TRUE,
      longtable = TRUE,
      linesep = "",
      ) %>%
    kableExtra::kable_styling(
        position = "center",
        latex_options = c("striped", "repeat_header"),
        stripe_color = "gray!15"
      )
  print(tablel)
} else {
  # If it's text, print it normally
  cat(check_repeats_output, "\n\n")
}
```
\newpage

#### Plot of Blank Samples

Blank samples should record MFI<50 for each protein. If any proteins are above the dashed line, they should be double-checked manually.

```{r blank samples plot}
blanks_plot <- params$blanks_plot
blanks_plot + theme(legend.position = "bottom")
```
\newpage

#### Experiment Information


**Experiment Notes**: `r params$experiment_notes`

**Platform Chosen**: `r params$platform`

**Raw Data Filename/s**: `r params$raw_data_filename`

**Operator**: `r params$operator_output`

**Acquisition volume**: `r params$volume_output`

**Recent Calibration and Verification results**: 
`r params$calibration_output`

**Machine Serial Number**: `r params$machine_output`

\newpage

#### Plate Layouts

\tiny 

```{r plate layouts, echo = FALSE, warning = FALSE, message = FALSE}
params$plate_list_output
```

---
title: "PvLDH Tutorial"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{PvLDH Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(rmarkdown.html_vignette.check_title = FALSE)
setup <- function() {
  needed <- c("knitr", "rmarkdown", "tidyverse", "kableExtra")
  
  lapply(needed, function(pkg) {
    if (requireNamespace(pkg, quietly = TRUE)) {
      library(pkg, character.only = TRUE)
    }
  })
}

setup()
library(SeroTrackR)

knitr::opts_chunk$set(
  dpi = 72
)
```

## Data Analysis: LDH

For the LDH data analysis, follow the steps below: 

### Using Tutorial Dataset: Load the Data

We will be using the build-in files in the R package for this tutorial, as shown below. 

```{r setup 1, eval=FALSE}
library(SeroTrackR)
library(tidyverse)

your_raw_data <- system.file("extdata", "example_BioPlex_PvLDH_plate1.csv", package = "SeroTrackR")
your_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")
```

To run your OWN data, follow the code below and uncomment (i.e., remove the hashtags): 

```{r exec=FALSE, eval=FALSE}
your_raw_data <- "PATH/TO/YOUR/FILE/plate1.csv"
your_plate_layout <- "PATH/TO/YOUR/FILE/plate_layout.xlsx"
```

### Run Pipeline 

```{r example code for LDH, eval=FALSE}
runLDHpipeline(
  raw_data = your_raw_data,
  plate_layout = your_plate_layout,
  platform = "bioplex",                               # Defaults to "bioplex" but "magpix" also works
  file_path = NULL                                    # Defaults to current folder/working directory
)
```

---
title: "Before You Begin"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Before You Begin}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## File Name Convention 

The following files are required for this app. You might like to organise your folder as follows:

```bash
my_R_project
└── data/
    ├── raw_data_plate1.csv
    ├── raw_data_plate2.csv
    ├── raw_data_plate3.csv
    ├── platelayout.xlsx
└── results/
└── data_processing.qmd
```

### Raw Data Files 

The `raw data` files should **all contain** "plate1", "plate2", "plate3", etc., in the file name. Ensure there are no special characters or spaces between the "plate" and number. To keep things simple, please also do not write any leading 0's before the number e.g., do not write "plate01".

**For the MAGPIX and Intelliflex Machines**: 

You can pre-program the MAGPIX machine so that you can export all the raw data directly from the machine once the plate reading is completed. There is no need to edit the raw data file that comes from the MAGPIX.

Within your plate layout in the MAGPIX, you can use the "U" button for all unknown samples, "B" button for Background or Blank samples, and "S" for Standard Curve samples. For the control wells, please feel free to edit these labels so that the ID is just "B", "S1", "S2", "S3"...."S10".

You can also write in the Operator i.e., who ran the assay! This is useful to track variation in plates between experiments.

![](../man/figures/magpix_raw.jpeg)

**For the Bioplex Machines**: 

Isolate names will tend to be written as 'X1', 'X2', 'X3'... and saved as an .xlsx file. Specifics on Bio-Plex machines will be added shortly.

![](../man/figures/bioplex_raw.jpeg)

### Plate Layout File 

The `plate layout` file should contain **all** of the plate layouts in each tab. For each 96-well plate that you run on the Luminex machine, prepare a plate layout that includes the sample labels that will match your raw data. The application will match the raw data to the corresponding sample based on the plate layout that you import.

Make sure that your sample labels in the plate layout are as follows:

- **Standards**: Labels start with "S" and then a number as required (e.g. S1, S2, S3 or Standard1, Standard2, Standard3).
- **Blanks**: Labels start with "B" and then a number as required if there is more than one blank sample (.e.g 'B1', 'B2', or 'Blank 1', 'Blank2' etc).
- **Unknown Samples**: Label your unknown samples according to your specific sample codes (e.g. ABC001, ABC002).

The package expects standards to start with "S" and blanks to start with "B", but everything else with a label will be considered an unknown sample. If you have other types of samples, for example a positive control, you can use a different sample label to the other unknown study samples (i.e. "PositiveControl" in addition to the "ABC" study codes).

![](../man/figures/plate_layout_1.jpeg)

The standards S1-10 correspond to the following dilution concentrations: 

| S1    | S2    | S3    | S4    | S5    | S6     | S7     | S8     | S9      | S10     |
|-------|-------|-------|-------|-------|--------|--------|--------|---------|---------|
| 1/50  | 1/100 | 1/200 | 1/400 | 1/800 | 1/1600 | 1/3200 | 1/6400 | 1/12800 | 1/25600 |

The standards S1-5 correspond to the following dilution concentrations: 

| S1    | S2    | S3     | S4     | S5      |
|-------|-------|--------|--------|---------|
| 1/50  | 1/250 | 1/1250 | 1/6250 | 1/31250 |

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRepeats.R
\name{getRepeats}
\alias{getRepeats}
\title{Check Beads to Repeat}
\usage{
getRepeats(qc_results, plate_list)
}
\arguments{
\item{qc_results}{Output from `runQC()`.}

\item{plate_list}{Output from `readPlateLayout()`.}
}
\value{
A data frame with wells to "fail", OR if no "fail" found will return
text "No repeats necessary".
}
\description{
This function gets the count data and outputs a table of the isolates to
repeat or a statement to confirm that none need to be repeated.
}
\examples{
\donttest{
# Step 0: Load example raw data and plate layout
raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)
plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")

# Step 1: Read data and plate layout
sero_data   <- readSeroData(raw_data, platform = "magpix")
plate_list  <- readPlateLayout(plate_layout, sero_data)

# Step 2: Process counts
qc_results  <- runQC(sero_data, plate_list)

# Step 3: Identify samples to repeat
repeats_table <- getRepeats(
  qc_results = qc_results,
  plate_list = plate_list
)

# View results
repeats_table
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotBoxPlotClassification.R
\name{plotBoxPlotClassification}
\alias{plotBoxPlotClassification}
\title{Plot Classification}
\usage{
plotBoxPlotClassification(all_classifications, selected_threshold)
}
\arguments{
\item{all_classifications}{Data frame of `classifyResults()` for all
sens_spec thresholds.}

\item{selected_threshold}{String with the threshold.}
}
\value{
Box plots with RAU values for each protein stratified by
classification (ggplot).
}
\description{
One example of data visualisation to detect the median and interquartile
range of the RAU values per antigen for seropositive and seronegative
individuals. Please note that the `classifyResults()` function must
be run first.
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results <- runQC(sero_data, plate_list)

# Step 3: Convert MFI to RAU using ETH beads
mfi_to_rau <- MFItoRAU_Adj(
  sero_data    = sero_data,
  plate_list   = plate_list,
  qc_results   = qc_results
)

# Step 4: Define sens/spec thresholds
sens_spec_all <- c(
  "balanced", "90\% specificity"
)

# Step 5: Classify results across all thresholds
all_classifications <- purrr::map_dfr(sens_spec_all, ~{
  classifyResults(
    mfi_to_rau_output = mfi_to_rau,
    algorithm_type = "antibody_model",
    sens_spec = .x,
    qc_results = qc_results
  ) |>
  as.data.frame() |>
  dplyr::mutate(sens_spec = .x)
})

# Plot classification for a single threshold
plotBoxPlotClassification(all_classifications, "balanced")
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\name{example_BioPlex_plate3_xlsx}
\alias{example_BioPlex_plate3_xlsx}
\title{Example Serological Dataset: Bioplex Plate 3}
\format{
A data frame with 104 rows and 8 columns.
}
\source{
Randomised data
}
\description{
A dataset containing raw MFI values and metadata from a sample plate run (Bioplex).
}
\details{
This file is stored in inst/extdata
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.check_platform}
\alias{.check_platform}
\title{Check Platform}
\usage{
.check_platform(raw_data, platform, file_name)
}
\arguments{
\item{raw_data}{String with the raw data path.}

\item{platform}{"magpix", "bioplex" or "intelliflex".}

\item{file_name}{String with the raw data filename (for error messaging).}
}
\value{
TRUE: if platform == file format, ERROR message when platform does
not equal file format.
}
\description{
This function checks the platform the user has input and whether it aligns
with the correct format as expected. Will report error if NOT aligned.
}
\examples{
your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
.check_platform(raw_data = your_raw_data, platform = "magpix", file_name = basename(your_raw_data))
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.fit_standard_curve}
\alias{.fit_standard_curve}
\title{Fit a standard curve to known mfi and dilution values.}
\usage{
.fit_standard_curve(mfi, dilution, control = NULL)
}
\arguments{
\item{mfi}{Known mfi of samples}

\item{dilution}{Known dilution of samples}

\item{control}{Optional list of control parameters for the underlying call to optim.}
}
\value{
standard curve log logistic
}
\description{
We wish to convert the standard curve samples to a five parameter logistic curve.
This function takes those values and calls optim to determine the fit.
}
\examples{
# This function is typically called within data-processing workflows.
# Workflow-style example (not run on CRAN)

\donttest{

# This block demonstrates how .fit_standard_curve() is typically used
# inside the MFItoRAU_Adj-conversion pipeline.

# Step 1 — Prepare master file (normally from readSeroData)
master_file <- data.frame(
  Location = c("A1","A2","A3"),
  Sample   = c("S1","S2","S3"),
  Plate    = c("Plate1","Plate1","Plate1"),
  Ag1 = c(12000, 8000, 4000),
  Ag2 = c(9000,  5000, 2500)
)

# Convert antigen columns to numeric
L <- master_file |>
  dplyr::mutate(dplyr::across(-c(Location, Sample, Plate), as.numeric))

# Fake plate layout (normally from readPlateLayout)
layout <- list(Plate1 = data.frame(Location = c("A1","A2","A3"), WellType = "STD"))


# Step 2 — Load reference standard curve MFI values (dummy data)
refs <- data.frame(
  std_plate = rep("StdPlate1", 5),
  antigen   = rep("Ag1", 5),
  dilution  = c(1, 1/2, 1/4, 1/8, 1/16),
  eth_mfi   = c(14000, 7000, 3500, 1800, 900),
  png_mfi   = c(15000, 7600, 3800, 1900, 950)
)


# Step 3 — Define optimisation settings
control <- list(
  maxit  = 10000,
  abstol = 1e-8,
  reltol = 1e-6
)


# Step 4 — Fit ETH and PNG curves per standard-plate × antigen
ref_fit <- refs |>
  dplyr::group_by(.data$std_plate, .data$antigen) |>
  tidyr::nest() |>
  dplyr::mutate(
    eth_fit = purrr::map(data, ~ .fit_standard_curve(.x$eth_mfi, .x$dilution, control)),
    png_fit = purrr::map(data, ~ .fit_standard_curve(.x$png_mfi, .x$dilution, control))
  )

ref_fit
}
}
\author{
Eamon Conway
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.relabel_columns}
\alias{.relabel_columns}
\title{Relabel column names to Standardised Naming Convention}
\usage{
.relabel_columns(df)
}
\arguments{
\item{df}{Data frame from `readSeroData()` processing.}
}
\value{
A data fame with columns renamed
}
\description{
This is a helper function to be used inside `readSeroData()` to relabel
columns for each plate.
}
\examples{
\donttest{
your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")

if (
 requireNamespace("dplyr", quietly = TRUE) &&
 requireNamespace("janitor", quietly = TRUE)
) {

  # Read in raw luminex file
  df <- .read_luminex_file(your_raw_data)

  # Get the start and end rows of the data section: start = "Median", end = "Net MFI"
  row1    <- which(df$xPONENT == "Median")
  row2    <- which(df$xPONENT == "Net MFI")

  # Apply data processing pipeline, including .relabel_columns()
  df |>
    dplyr::slice((row1 + 1):(row2 - 1)) |>
    janitor::row_to_names(row_number = 1) |>
    janitor::clean_names() |>
    dplyr::select(dplyr::where(~ !all(is.na(.x)))) |>
    dplyr::filter(dplyr::if_any(dplyr::everything(), ~ !is.na(.x))) |>
    dplyr::mutate(dplyr::across(everything(), ~ gsub("NaN", 0, .))) |>
    .relabel_columns()
}
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{MFItoRAU_Pk}
\alias{MFItoRAU_Pk}
\title{Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
conversion for Pk proteins}
\usage{
MFItoRAU_Pk(processed_Pk, plate_list, std_point, qc_results)
}
\arguments{
\item{processed_Pk}{df$Pk of output `processPkPfPv()`}

\item{plate_list}{Output of `readPlateLayout()`}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.}

\item{qc_results}{Output from `runQC()`.}
}
\value{
Data frame with MFI data, converted RAU data and matched SampleID's.
}
\description{
This function is utilised in the master function `MFItoRAU_Plasmo()`.
}
\author{
Dionne Argyropoulos, Caitlin Bourke
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renderDetailsList.R
\name{renderDetailsList}
\alias{renderDetailsList}
\title{Create a Fluent UI Table}
\usage{
renderDetailsList(df)
}
\arguments{
\item{df}{Any processed data frame}
}
\value{
A table in the Fluent UI format
}
\description{
This function makes the table in a Fluent UI format.
}
\examples{
# Minimal example using a small data frame.
# This example is safe for CRAN because it runs only if
# shiny.fluent and htmltools are installed.

if (requireNamespace("shiny.fluent", quietly = TRUE) &&
    requireNamespace("htmltools", quietly = TRUE)) {

  # Tiny example data frame
  example_df <- data.frame(
    Sample = c("A", "B"),
    Value = c(10, 20),
    stringsAsFactors = FALSE
  )

  # Render Fluent UI DetailsList
  renderDetailsList(example_df)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.read_luminex_file}
\alias{.read_luminex_file}
\title{Helper function to read raw luminex files}
\usage{
.read_luminex_file(file)
}
\arguments{
\item{file}{String with the raw data path.}
}
\value{
raw data frame
}
\description{
Helper function to read raw luminex files
}
\examples{
your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
df <- .read_luminex_file(your_raw_data)
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{MFItoRAU_Adj}
\alias{MFItoRAU_Adj}
\title{Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
conversion based on other standard}
\usage{
MFItoRAU_Adj(sero_data, plate_list, qc_results, std_point = 10, project = NULL)
}
\arguments{
\item{sero_data}{Output from `readSeroData()`.}

\item{plate_list}{Output from `readPlateLayout()`.}

\item{qc_results}{Output from `runQC()`.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.}

\item{project}{Default = NULL. Only write "pkpfpv" if using Pk/Pf/Pv pipeline.}
}
\value{
A list of three data frames:
1. Data frame with  MFI data, converted RAU data and matched SampleID's.
2. Plot information for `plotModel` function.
3. Data frame of RAU data for random forest classification use.
}
\description{
This function fits a 5-parameter logistic standard curve to the dilutions
of the positive controls for each protein and converts the MFI values
into relative antibody units (RAU) written by Eamon Conway.
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results  <- runQC(sero_data, plate_list)

# Step 3: Convert MFI to RAU using ETH beads
mfi_to_rau <- MFItoRAU_Adj(
  sero_data    = sero_data,
  plate_list   = plate_list,
  qc_results   = qc_results
)

}
}
\author{
Eamon Conway, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotStds.R
\name{plotStds}
\alias{plotStds}
\title{Plot Raw Median Fluorescent Intensity of Standard Curve Data}
\usage{
plotStds(sero_data, std_point = 10, location, experiment_name)
}
\arguments{
\item{sero_data}{Output from `readSeroData()`.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve. Default = 10. Value is an integer.}

\item{location}{"PNG" or "ETH" to filter WEHI standard curve data.}

\item{experiment_name}{User-input experiment name.}
}
\value{
- Dot and line plot of standard curves (S1-S10) with PNG or Ethiopia stds
underneath (ggplot).
- WEHI-acceptable standard curve data on background of plot with user data.
}
\description{
This function gets the standards data and plots the standard curves.
}
\examples{
\donttest{
# Example demonstrating how to process bead count data.
# These files are included in the SeroTrackR package under inst/extdata.

your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

# Read in raw MAGPIX data
sero_data <- readSeroData(
  raw_data = your_raw_data,
  platform = "magpix"
)

# Plot Standards
plotStds(
  sero_data = sero_data,
  location = "ETH",
  experiment_name = "experiment1"
)

}

}
\author{
Dionne Argyropoulos, Shazia Ruybal-Pesantez
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.magpix_version_config}
\alias{.magpix_version_config}
\title{Helper function to identify Magpix version}
\usage{
.magpix_version_config(version)
}
\arguments{
\item{version}{String with the raw data path.}
}
\value{
specific column names for filtering for xPONENT software v4.2 and v4.3
}
\description{
Helper function to identify Magpix version
}
\examples{
version = "4.2"
.magpix_version_config(version)

}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPvSeroPipeline.R
\name{runPvSeroPipeline}
\alias{runPvSeroPipeline}
\title{Run PvSero Pipeline from Start to End}
\usage{
runPvSeroPipeline(
  raw_data,
  plate_layout,
  platform = "magpix",
  location,
  experiment_name = "experiment1",
  std_point = 10,
  classify = "Yes",
  algorithm_type = "antibody_model",
  sens_spec = "balanced"
)
}
\arguments{
\item{raw_data}{String with the raw data path.}

\item{plate_layout}{An ".xlsx" file with sheets labelled plate1, plate2... etc.}

\item{platform}{"magpix" or "bioplex". Default = "magpix".}

\item{location}{"PNG" or "ETH" to filter WEHI standard curve data.}

\item{experiment_name}{User-input experiment name.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.}

\item{classify}{"Yes" or "No" depending on whether you would like classification or not. Default = "Yes".}

\item{algorithm_type}{Algorithm: "antibody_model" (PvSEM algorithm; default)}

\item{sens_spec}{User-selected Sensitivity/Specificity threshold: "balanced"
(default) or "90\% specificity".}
}
\value{
classifyResults_output, stdcurve_plot, plateqc_plot, check_repeats_output, blanks_plot, model_plot
}
\description{
A master function combining the entire PvSeroApp pipeline into one command to run in R.
}
\examples{

\donttest{
# Example data supplied with the package
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

plate_layout <- system.file(
  "extdata", "example_platelayout_1.xlsx", package = "SeroTrackR"
)

# Run full pipeline including classification
runPvSeroPipeline(
  raw_data = your_raw_data,
  plate_layout = plate_layout,
  platform = "magpix",
  location = "PNG",
  experiment_name = "experiment1",
  std_point = 10,
  algorithm_type = "antibody_model",
  sens_spec = "balanced",
  classify = "Yes"
)

# Run processing pipeline only (no classification)
runPvSeroPipeline(
  raw_data = your_raw_data,
  plate_layout = plate_layout,
  platform = "magpix",
  location = "PNG",
  experiment_name = "experiment1",
  std_point = 10,
  algorithm_type = "antibody_model",
  sens_spec = "balanced",
  classify = "No"
)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runQC.R
\name{runQC}
\alias{runQC}
\title{Run Quality Control Pipeline}
\usage{
runQC(sero_data, plate_list)
}
\arguments{
\item{sero_data}{Output from `readSeroData()`.}

\item{plate_list}{Output from `readPlateLayout()`.}
}
\value{
processCounts_output, getCounts_output, sampleid_output, getAntigenCounts_output, getCountsQC_output
}
\description{
A master function containing each quality control processing step.
}
\examples{

\donttest{
# Example data supplied with the package
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

your_plate_layout <- system.file(
  "extdata", "example_platelayout_1.xlsx", package = "SeroTrackR"
)

# Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Run full pipeline including classification
runQC(
  sero_data = sero_data,
  plate_list = plate_list
)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{MFItoRAU_LDH}
\alias{MFItoRAU_LDH}
\title{Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
conversion for LDH}
\usage{
MFItoRAU_LDH(sero_data, plate_list, std_point = "PvLDH", file_path = NULL)
}
\arguments{
\item{sero_data}{Output from `readSeroData()` or `readSeroData()`.}

\item{plate_list}{Output from `readPlateLayout()`.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = "PvLDH".}

\item{file_path}{A file path to write the .csv final file. Default: Current working directory.}
}
\value{
A data frame containing the MFI and RAU Dilution values for each sample
}
\description{
This function fits a 5-parameter logistic standard curve to the dilutions
of the positive controls for each protein and converts the MFI values
into relative antibody units (RAU).
}
\examples{
\donttest{
# Example demonstrating multi-plate processing workflow.
# These files are included in the SeroTrackR package under inst/extdata.

your_raw_data <- system.file("extdata", "example_BioPlex_PvLDH_plate1.xlsx", package = "SeroTrackR")

your_plate_layout <- system.file(
  "extdata", "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Read in raw BioPlex data
sero_data <- readSeroData(
  raw_data = your_raw_data,
  platform = "bioplex"
)

# Read matching plate layout
plate_list <- readPlateLayout(
  plate_layout = your_plate_layout,
  sero_data = sero_data
)

# Run MFI to RAU conversion
mfi_outputs <- MFItoRAU_LDH(
  sero_data = sero_data,
  plate_list = plate_list
)

# View All Outputs
mfi_outputs
}
}
\author{
Connie Li Wai Suen, Caitlin Bourke, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.clean_luminex}
\alias{.clean_luminex}
\title{Helper function to process luminex (Magpix/Intelliflex) data}
\usage{
.clean_luminex(df, row1, row2)
}
\arguments{
\item{df}{Raw luminex file}

\item{row1}{Leading row to subset}

\item{row2}{Final row to subset}
}
\value{
Cleaned data fame
}
\description{
Helper function to process luminex (Magpix/Intelliflex) data
}
\examples{
\donttest{

your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
df            <- .read_luminex_file(your_raw_data)
cfg           <- .magpix_version_config("4.2")

row1         <- which(df$xPONENT == "Median")
row2         <- which(df$xPONENT == "Net MFI")

results      <- .clean_luminex(df, row1, row2)

}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.convert_mfi_to_dilution_no_bounds}
\alias{.convert_mfi_to_dilution_no_bounds}
\title{Convert mfi to dilution using known standard curve fit and no bounds}
\usage{
.convert_mfi_to_dilution_no_bounds(mfi, params, min_relative_dilution)
}
\arguments{
\item{mfi}{Known mfi of samples}

\item{params}{Known parameters for five parameter logistic fit.}

\item{min_relative_dilution}{Known minimum value of dilution in the standard curve. Relative means setting S1 to a dilution/RAU/concentration of 1.}
}
\value{
Returns the dilution of each sample in mfi.
}
\description{
Convert mfi to dilution using known standard curve fit and no bounds unless you are below the asymptote of the standard curve.
In this situation we set your value to min_relative_dilution. I dunno argue?
}
\examples{
# This function is generally called inside higher-level analysis workflows.
# Below is a minimal self-contained example using dummy values.

# Dummy five-parameter logistic fit parameters:
# a, b, c, d, e  (with e typically supplied on the log scale)
dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01))

# Example MFI value
mfi_example <- 1500

# Minimum relative dilution from the standard curve
min_rel_dil <- 1

# Convert MFI to dilution without bounds
.convert_mfi_to_dilution_no_bounds(mfi_example, dummy_params, min_rel_dil)
}
\author{
Eamon Conway
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.process_antigen_loglog}
\alias{.process_antigen_loglog}
\title{Helper function to fit a 5-parameter logistic standard curve to dilutions}
\usage{
.process_antigen_loglog(
  subset_data,
  antigen,
  dilution,
  s1_concentration,
  s_final_concentration,
  unknown_letters = c("U", "X")
)
}
\arguments{
\item{subset_data}{Data for one plate.}

\item{antigen}{Data for one antigen.}

\item{dilution}{Set of five or ten.}

\item{s1_concentration}{Concentration of highest dilution.}

\item{s_final_concentration}{Concentration lowest dilution.}

\item{unknown_letters}{Bioplex, Magpix or Intelliflex known unknown letters (Default = U and X).}
}
\value{
A list of the model results data frame and model.
}
\description{
Helper function to fit a 5-parameter logistic standard curve to dilutions
}
\author{
Connie Li Wai Suen, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.clean_bioplex}
\alias{.clean_bioplex}
\title{Helper function to process bioplex data}
\usage{
.clean_bioplex(df)
}
\arguments{
\item{df}{Output from `.read_luminex.file()`}
}
\value{
Cleaned data fame
}
\description{
Helper function to process bioplex data
}
\examples{
your_raw_data <- system.file("extdata", "example_BioPlex_plate1.xlsx", package = "SeroTrackR")
df            <- .read_luminex_file(your_raw_data)
results       <- .clean_bioplex(df)
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classifyResults.R
\name{classifyResults}
\alias{classifyResults}
\title{Random Forest Classification}
\usage{
classifyResults(
  mfi_to_rau_output,
  algorithm_type = "antibody_model",
  sens_spec = "balanced",
  qc_results,
  project = NULL
)
}
\arguments{
\item{mfi_to_rau_output}{Output from `MFItoRAU()` or `MFItoRAU_Adj()`.}

\item{algorithm_type}{Algorithm: "antibody_model" (PvSEM algorithm; default)}

\item{sens_spec}{User-selected Sensitivity/Specificity threshold: "balanced"
(default) or "90\% specificity".}

\item{qc_results}{Output from `runQC()`.}

\item{project}{Default = NULL. Only write "pkpfpv" if using Pk/Pf/Pv pipeline.}
}
\value{
- Data frame with exposure status for every sample.
- Summary table with positive/negative results for each threshold.
}
\description{
This function classifies unknown samples as recently exposed or not
(Note: MFItoRAU() or MFItoRAU_Adj() needs to be run first to convert to
RAU).
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results  <- runQC(sero_data, plate_list)

# Step 3: Convert MFI to RAU using ETH beads
mfi_to_rau <- MFItoRAU_Adj(
  sero_data   = sero_data,
  plate_list  = plate_list,
  qc_results  = qc_results
)

# Step 4: Perform Pv classification
pv_classified <- classifyResults(
  mfi_to_rau_output = mfi_to_rau,
  algorithm_type    = "antibody_model",
  sens_spec         = "balanced",
  qc_results        = qc_results
)
}
}
\author{
Lauren Smith, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\name{example_MAGPIX_pk_10std_plate1_csv}
\alias{example_MAGPIX_pk_10std_plate1_csv}
\title{Example Serological Dataset: MAGPIX Plate 1 Pk Analysis 10-Point Standard Curve
A dataset containing raw MFI values and metadata from a sample plate run (MAGPIX).}
\format{
A data frame with 614 rows and 17 columns.
}
\source{
Randomised data
}
\description{
This file is stored in inst/extdata
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runQC.R
\name{processCounts}
\alias{processCounts}
\title{Process Counts from Raw Serological Data file}
\usage{
processCounts(sero_data)
}
\arguments{
\item{sero_data}{Output from `readSeroData()`.}
}
\value{
Returns a long table of counts with "Warning" category (<15 == 1 and
\eqn{>=}  15 == 0) for downstream wrangling.
}
\description{
A helper function to process counts data.
}
\examples{
\donttest{
# Example demonstrating how to process bead count data.
# These files are included in the SeroTrackR package under inst/extdata.

your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

# Read in raw MAGPIX data
sero_data <- readSeroData(
  raw_data = your_raw_data,
  platform = "magpix"
)

# Process counts
processed_master <- processCounts(sero_data = sero_data)

}

}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renderReport.R
\name{renderReport}
\alias{renderReport}
\title{Render Markdown report}
\usage{
renderReport(input, output, params)
}
\arguments{
\item{input}{Input files}

\item{output}{Output files}

\item{params}{Parameters to generate outputs.}
}
\value{
PDF output.
}
\description{
A short function to render the rmarkdown report on Shiny.
}
\examples{
# Minimal example that renders a temporary Rmd file.
# Safe for CRAN because it only writes to tempdir()
\dontrun{
if (requireNamespace("rmarkdown", quietly = TRUE) &&
    rmarkdown::pandoc_available()) {

  # Create a temporary Rmd that declares params in the YAML
  rmd_file <- tempfile(fileext = ".Rmd")
  writeLines(c(
    "---",
    "title: \"Test Report\"",
    "output: html_document",
    "params:",
    "  value: 0",
    "---",
    "",
    "This is a test report.",
    "",
    "Parameter value: `r params$value`"
  ), con = rmd_file)

  # Output location
  out_file <- tempfile(fileext = ".html")

  # Example parameters to pass in
  example_params <- list(value = 123)

  # Render report
  renderReport(
    input  = rmd_file,
    output = out_file,
    params = example_params
  )

  # Optionally inspect the output path
  out_file
}
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.post_process_bioplex}
\alias{.post_process_bioplex}
\title{Helper function to process bioplex sections}
\usage{
.post_process_bioplex(df)
}
\arguments{
\item{df}{Output from `.read_luminex_file()`}
}
\value{
List of data_raw, results, counts, blanks, stds, run
}
\description{
Helper function to process bioplex sections
}
\author{
Dionne Argyropoulos

your_raw_data <- system.file("extdata", "example_BioPlex_plate1.xlsx", package = "SeroTrackR")
df            <- .read_luminex_file(your_raw_data)
sections      <- .post_process_bioplex(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runLDHPipeline.R
\name{runLDHPipeline}
\alias{runLDHPipeline}
\title{Run LDH Pipeline from Start to End}
\usage{
runLDHPipeline(
  raw_data,
  plate_layout,
  platform = "bioplex",
  dilution = c(1e+06, 333333.33, 111111.11, 37037.04, 12345.68, 4115.23, 1371.74, 457.25,
    152.42, 50.81),
  experiment_name = "experiment1",
  file_path = NULL
)
}
\arguments{
\item{raw_data}{String with the raw data path.}

\item{plate_layout}{An ".xlsx" file with sheets labelled plate1, plate2... etc.}

\item{platform}{"magpix" or "bioplex". Default: "Bioplex"}

\item{dilution}{A list of numbers ranging from S1 to S10. Default: 1000000, 333333.33, 111111.11, 37037.04, 12345.68, 4115.23, 1371.74, 457.25, 152.42, 50.81.}

\item{experiment_name}{User-input experiment name. Default: "experiment1".}

\item{file_path}{A file path to write the .csv final file. Default: Current working directory.}
}
\value{
A data frame containing the MFI and RAU Dilution values for each sample, QC plots for standard curve, bead counts and blanks.
}
\description{
A master function combining the entire LDH pipeline into one command to run in R.
}
\examples{
\donttest{

# Example input files
your_raw_data <- system.file(
  "extdata",
  "example_BioPlex_PvLDH_plate1.xlsx",
  package = "SeroTrackR"
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Run full LDH processing pipeline
runLDHPipeline(
  raw_data      = your_raw_data,        # Vector of raw data files
  plate_layout  = your_plate_layout,    # Plate layout file
)

}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.merge_mfitorau}
\alias{.merge_mfitorau}
\title{Helper function to add Sample IDs to output.}
\usage{
.merge_mfitorau(df, layout, plate_level)
}
\arguments{
\item{df}{Data frame following 5-parameter logistic function applied.}

\item{layout}{Output from `readPlateLayout()`.}

\item{plate_level}{Specific plate of interest.}
}
\value{
Processed data frame with correct Sample IDs.
}
\description{
Helper function to add Sample IDs to output.
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotModel.R
\name{plotModel_Adj}
\alias{plotModel_Adj}
\title{Plot the Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
Results Data based on ETH standard}
\usage{
plotModel_Adj(mfi_to_rau_output, sero_data)
}
\arguments{
\item{mfi_to_rau_output}{Output from `MFItoRAU_Adj()`.}

\item{sero_data}{Output from `readSeroData()`.}
}
\value{
List of dot and line plots of MFI to RAU model standard curve,
with each one representing an individual plate (ggplot).
}
\description{
This function gets the Median Fluorescent Intensity (MFI) to Relative
Antibody Units (RAU) model results data and plots the model fits based on
`MFItoRAU_Adj.`
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results <- runQC(sero_data, plate_list)

# Step 3: Convert MFI to RAU using ETH beads
mfi_to_rau <- MFItoRAU_Adj(
  sero_data = sero_data,
  plate_list = plate_list,
  qc_results = qc_results
)

# Step 4: Plot Model Results
plotModel_Adj(mfi_to_rau, sero_data)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runQC.R
\name{getCountsQC}
\alias{getCountsQC}
\title{Get All Counts Data}
\usage{
getCountsQC(antigen_counts_output, counts_output)
}
\arguments{
\item{antigen_counts_output}{Output from `getAntigenCounts`.}

\item{counts_output}{Output from `getCounts`.}
}
\value{
Joined data frame for all count data.
}
\description{
This function obtains the count data from the raw Median Fluorescent
Intensity (MFI). This function relies on the output of the Antigen-specific
counts (`getAntigenCounts`) and the Well or Sample-specific counts
(`getCounts`).
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
counts      <- processCounts(sero_data)
counts_raw  <- getCounts(counts)
sample_ids  <- getSampleID(counts, plate_list)
antigen_cts <- getAntigenCounts(counts, plate_list)
counts_qc   <- getCountsQC(antigen_cts, counts_raw)

}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.convert_dilution_to_mfi}
\alias{.convert_dilution_to_mfi}
\title{Convert known dilution to mfi from fitted standard curve}
\usage{
.convert_dilution_to_mfi(dilution, params)
}
\arguments{
\item{dilution}{Known dilution of samples}

\item{params}{Known parameters for five parameter logistic fit.}
}
\value{
Returns the predicted mfi of a sample with known dilution.
}
\description{
Convert dilution to predicted mfi using known standard curve fit.
}
\examples{
# This function is typically called internally by higher-level workflows.
# Below is a minimal runnable example using dummy parameters.

# Five-parameter logistic model typically expects parameters in the order:
# a, b, c, d, e  (e often log-transformed)
dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01))

# Example dilution value
dilution_example <- 0.1

# Predict MFI from the dummy standard curve
.convert_dilution_to_mfi(dilution_example, dummy_params)

}
\author{
Eamon Conway
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{MFItoRAU_Plasmo}
\alias{MFItoRAU_Plasmo}
\title{Median Fluorescent Intensity (MFI) to Relative Antibody Units (RAU)
conversion for Pk/Pf/Pv Master Function}
\usage{
MFItoRAU_Plasmo(sero_data, plate_list, panel = "panel1", std_point, qc_results)
}
\arguments{
\item{sero_data}{Output of `readserodata_output()`}

\item{plate_list}{Output of `readPlateLayout()`}

\item{panel}{Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.}

\item{qc_results}{Output from `runQC()`.}
}
\value{
A list of three data frames:
1. Data frame with MFI data, converted RAU data, matched SampleID's, all intermediate dilution conversion factors
2. Data frame with only SampleID's, MFI and RAU data
3. Data frame #2 in long-format
}
\description{
This function leverages `MFItoRAU_Pk()` and `MFItoRAU()` to create a final MFI to RAU
output for Pk/Pf/Pv analyses.
}
\examples{
\donttest{
# Example demonstrating multi-plate 5-standard processing workflow.
# These files are included in the SeroTrackR package under inst/extdata.

your_raw_data_5std <- c(
  system.file("extdata", "example_MAGPIX_pk_5std_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_pk_5std_plate2.csv", package = "SeroTrackR")
)
your_plate_layout_5std <- system.file(
  "extdata", "example_platelayout_pk_5std.xlsx",
  package = "SeroTrackR"
)

# Read in raw MAGPIX data
sero_data <- readSeroData(
  raw_data = your_raw_data_5std,
  platform = "magpix"
)

# Read matching plate layout
plate_list <- readPlateLayout(
  plate_layout = your_plate_layout_5std,
  sero_data = sero_data
)
# Quality control
qc_results  <- runQC(sero_data, plate_list)

# Run MFI to RAU conversion
mfi_outputs               <- MFItoRAU_Plasmo(
  sero_data = sero_data,
  plate_list = plate_list,
  panel = "panel1",
  std_point = 5,
  qc_results = qc_results
)

# View All Outputs
mfi_outputs
}
}
\author{
Dionne Argyropoulos, Caitlin Bourke
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{readSeroData}
\alias{readSeroData}
\title{Read Raw Serological Data}
\usage{
readSeroData(raw_data, platform, version = "4.2", raw_data_filenames = NULL)
}
\arguments{
\item{raw_data}{String with the raw data path.}

\item{platform}{"magpix", "bioplex" or "intelliflex".}

\item{version}{xPONENT software version. For "magpix" can be 4.2 or 4.3. Default: 4.2.}

\item{raw_data_filenames}{String with the raw data filename path.
Default is NA as it can be deduced from raw_data. Needs to be a parameter for the PvSeroApp.}
}
\value{
List of data frames: (i) raw data output, (ii) cleaned all results
(iii) count data, (iv) blanks only, (v) standards only, (vi) run
information.
}
\description{
This function imports the raw data from the Magpix or Bioplex machine
and matches the sample names from the plate layout based on their plate/well
location.
}
\examples{
# Example raw data files (MAGPIX platform)
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

# Read and combine raw serology data
sero_data <- readSeroData(
  raw_data = your_raw_data,
  platform = "magpix"
)
}
\author{
Dionne Argyropoulos, Shazia Ruybal-Pesantez
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runQC.R
\name{getCounts}
\alias{getCounts}
\title{Get Count Data from Raw Median Fluorescent Intensity}
\usage{
getCounts(processed_counts)
}
\arguments{
\item{processed_counts}{Output from `processCounts()`.}
}
\value{
(i) Data frame providing bead counts per well per plate. (ii)
Designates whether wells should be repeated if there are \eqn{<=}  15 beads (repeat)
or if they are sufficient with > 15 beads (sufficient beads).
}
\description{
This function obtains the count data from the raw Median Fluorescent
Intensity (MFI). This is an interim function used for the plotCounts
function. This function relies on the `readAntigens` and `readSeroData` data
processing functions.
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
counts      <- processCounts(sero_data)
counts_raw  <- getCounts(counts)
}
}
\author{
Shazia Ruybal-Pesántez, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGithubRelease.R
\name{getGithubRelease}
\alias{getGithubRelease}
\title{Get GitHub Version}
\usage{
getGithubRelease(repo_owner, repo_name)
}
\arguments{
\item{repo_owner}{GitHub Username}

\item{repo_name}{GitHub Repository Name}
}
\value{
Version tag string
}
\description{
A short function to obtain the github version from the repository. This is
a generalisable function that can be used for any version tags on a repo.
}
\examples{

getGithubRelease(
 repo_owner = "dionnecargy",
 repo_name = "SeroTrackR"
)

}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotRAU.R
\name{plotRAU}
\alias{plotRAU}
\title{Relative Antibody Unit (RAU) Box Plots}
\usage{
plotRAU(mfi_to_rau_output, location)
}
\arguments{
\item{mfi_to_rau_output}{Output from `MFItoRAU()` or `MFItoRAU_Adj()`
.}

\item{location}{"PNG" or "ETH".}
}
\value{
Box plots with RAU values for each protein (ggplot).
}
\description{
Boxplot of the RAU values.
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results <- runQC(sero_data, plate_list)

# Step 3: Convert MFI to RAU using ETH beads
mfi_to_rau <- MFItoRAU_Adj(
  sero_data   = sero_data,
  plate_list  = plate_list,
  qc_results  = qc_results
)

# Step 4: Plot RAU values
plotRAU(mfi_to_rau, "ETH")
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renderQCReport.R
\name{renderQCReport}
\alias{renderQCReport}
\title{Generate QC PDF Report}
\usage{
renderQCReport(
  raw_data,
  plate_layout,
  platform,
  experiment_name = "experiment1",
  date = format(Sys.Date(), "\%Y\%m\%d"),
  experiment_notes = "no notes",
  location,
  std_point = 10,
  path = "."
)
}
\arguments{
\item{raw_data}{A string with the raw data path.}

\item{plate_layout}{A string with the plate layout path.}

\item{platform}{A string: "magpix", "intelliflex", or "bioplex".}

\item{experiment_name}{A string for experiment name.}

\item{date}{A string or Date. Defaults to today's date.}

\item{experiment_notes}{A string of notes. Default is "no notes".}

\item{location}{A string for experiment location: "ETH" or "PNG" accepted.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.}

\item{path}{Output path for the PDF file. Defaults to current working directory.}
}
\value{
Rendered PDF report.
}
\description{
Generate QC PDF Report
}
\examples{
## Not run on CRAN because it requires interactive rendering and can be slow:
\dontrun{
  # Example raw data files (MAGPIX platform)
  your_raw_data <- c(
    system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
    system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
    system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
  )

  # Example plate layout file
  your_plate_layout <- system.file(
    "extdata",
    "example_platelayout_1.xlsx",
    package = "SeroTrackR"
  )

  # Generate the QC PDF report
  renderQCReport(
    raw_data     = your_raw_data,
    plate_layout = your_plate_layout,
    platform     = "magpix",
    location     = "ETH"
  )
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.extract_luminex_sections}
\alias{.extract_luminex_sections}
\title{Helper function to process luminex sections}
\usage{
.extract_luminex_sections(df, cfg, plt)
}
\arguments{
\item{df}{String with the raw data path.}

\item{cfg}{Magpix version output of .magpix_version_config().}

\item{plt}{Platform (magpix, intelliflex)}
}
\value{
List of data_raw, results, counts, blanks, stds, run
}
\description{
Helper function to process luminex sections
}
\examples{
your_raw_data <- system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR")
df            <- .read_luminex_file(your_raw_data)
cfg           <- .magpix_version_config("4.2")
section       <- .extract_luminex_sections(df, cfg, "magpix")
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processPkPfPv.R
\name{processPkPfPv}
\alias{processPkPfPv}
\title{Processing Serological Data for Pk/Pf/Pv MFI to RAU conversion}
\usage{
processPkPfPv(sero_data, plate_list, panel = "panel1")
}
\arguments{
\item{sero_data}{Output of `readSeroData()`}

\item{plate_list}{Output of `readPlateLayout()`}

\item{panel}{Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.}
}
\value{
A list of two data frames:
1. Data frame with Pk antigens
2. Data frame with Pf/Pv antigens
}
\description{
This is a pre-requisite function before running the `MFItoRAU_Plasmo()` so that the
appropriate MFI to RAU conversions can be run for the respective antigens.
}
\examples{
\donttest{
# Example demonstrating multi-plate 5-standard processing workflow.
# These files are included in the SeroTrackR package under inst/extdata.

your_raw_data_5std <- c(
  system.file("extdata", "example_MAGPIX_pk_5std_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_pk_5std_plate2.csv", package = "SeroTrackR")
)

your_plate_layout_5std <- system.file(
  "extdata", "example_platelayout_pk_5std.xlsx",
  package = "SeroTrackR"
)

# Read in raw MAGPIX data
sero_data <- readSeroData(
  raw_data = your_raw_data_5std,
  platform = "magpix"
)

# Read matching plate layout
plate_list <- readPlateLayout(
  plate_layout = your_plate_layout_5std,
  sero_data = sero_data
)

# Process multi-species panel
processed_master <- processPkPfPv(
  sero_data = sero_data,
  plate_list = plate_list,
  panel = "panel1"
)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPlateLayout.R
\name{getPlateLayout}
\alias{getPlateLayout}
\title{Find and create a master plate layout file}
\usage{
getPlateLayout(folder_path = getwd(), output_file = NULL)
}
\arguments{
\item{folder_path}{A string containing your main folder for your project or the plate layout files. Default = current working directory.}

\item{output_file}{A string for the path for your output master file.}
}
\value{
An .xlsx file saved to your current working directory with multiple tabs, one tab for each plate layout.
}
\description{
Join multiple a plate layout files into one master file with multiple tabs
}
\examples{
\donttest{
# Example 1: Create two example 96-well plates in-memory
create_plate <- function(plate_name) {
  rows <- LETTERS[1:8]
  cols <- 1:12
  df <- data.frame(plate = rows)
  for (col in cols) {
    df[[as.character(col)]] <- paste0(rows, col)
  }
  df$plate_id <- plate_name
  df
}

plate1 <- create_plate("Plate1")
plate2 <- create_plate("Plate2")

# Combine plates into a list to simulate getPlateLayout() output
master_layout <- list(
  path = tempfile(fileext = ".xlsx"),  # placeholder path
  data = list(Plate1 = plate1, Plate2 = plate2)
)

# The returned list contains:
# 1. path: the file path to the (simulated) master Excel file
# 2. data: a list of data.frames, one per plate
names(master_layout$data)  # View sheet names

# Example 2: Access individual plates directly
layout_files <- list(plate1, plate2)  # simulate individual Excel sheets

master_layout2 <- list(
  path = tempfile(fileext = ".xlsx"),  # placeholder path
  data = setNames(layout_files, c("Plate1", "Plate2"))
)

# View the resulting plate names
names(master_layout2$data)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotBlanks.R
\name{plotBlanks}
\alias{plotBlanks}
\title{Plot Raw Median Fluorescent Intensity Blanks Data}
\usage{
plotBlanks(sero_data, experiment_name)
}
\arguments{
\item{sero_data}{Output from `readSeroData()`.}

\item{experiment_name}{User-input experiment name.}
}
\value{
Bar plot showing whether MFI values for the blanks for each antigen
per plate is above or below the threshold MFI = 50 (ggplot).
}
\description{
This function gets the blank sample data and plots the blank sample Median
Fluorescent Intensity (MFI) values.
}
\examples{
\donttest{
# Example demonstrating how to process bead count data.
# These files are included in the SeroTrackR package under inst/extdata.

your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

# Read in raw MAGPIX data
sero_data <- readSeroData(
  raw_data = your_raw_data,
  platform = "magpix"
)

# Plot blanks
plotBlanks(
  sero_data = sero_data,
  experiment_name = "experiment1"
)

}

}
\author{
Shazia Ruybal-Pesantez, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renderClassificationTable.R
\name{renderClassificationTable}
\alias{renderClassificationTable}
\title{All Classification Data}
\usage{
renderClassificationTable(mfi_to_rau_output, algorithm_type, qc_results)
}
\arguments{
\item{mfi_to_rau_output}{Output from `MFItoRAU()` or `MFItoRAU_Adj()`.}

\item{algorithm_type}{Algorithm: "antibody_model" (PvSEM algorithm; default)}

\item{qc_results}{Output from `runQC()`.}
}
\value{
A table of all classification outputs.
}
\description{
This function runs the classification algorithm for all possible sensitivity
and specificity options.
}
\examples{
\donttest{

# Step 0: Load in Raw Data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file("extdata", "example_platelayout_1.xlsx", package = "SeroTrackR")

# Step 1: Reading in Raw Data
sero_data     <- readSeroData(raw_data = your_raw_data, "magpix")
plate_list    <- readPlateLayout(
  plate_layout = your_plate_layout,
  sero_data = sero_data
)

# Step 2: Quality Control and MFI to RAU
qc_results <- runQC(sero_data, plate_list)

# Step 4: Run MFI to RAU (e.g., using ETH beads)
mfi_to_rau_output  <- MFItoRAU_Adj(sero_data, plate_list, qc_results)

# Step 5: Render classification table
renderClassificationTable(
  mfi_to_rau_output = mfi_to_rau_output,
  algorithm_type = "antibody_model",
  qc_results = qc_results
)

}

}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readPlateLayout.R
\name{readPlateLayout}
\alias{readPlateLayout}
\title{Read Plate Layout/s}
\usage{
readPlateLayout(plate_layout, sero_data)
}
\arguments{
\item{plate_layout}{An ".xlsx" file with sheets labelled plate1, plate2...
etc..}

\item{sero_data}{Output from `readSeroData()`.}
}
\value{
A list of data frames, with each one representing an individual plate.
}
\description{
This function imports the plate layout. Each sheet of the plate layout
".xlsx" file must contain 13 columns (labelled
Plate, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12) (columns A-M) and 9 rows
(Plate, A, B, C, D, E, F, G, H) (rows 1-9). *Note that the first row/column
i.e., the A1 cell in excel is called "Plate". This function also checks that
the plate sheet labels are consistent with the MAGPIX file input names, as a
check prior to merging downstream.
}
\examples{

# Example input files
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate3.csv", package = "SeroTrackR")
)

your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read and combine serological data
sero_data <- readSeroData(
  raw_data = your_raw_data,
  platform = "magpix"
)
# Step 2: Read plate layout
plate_list <- readPlateLayout(
  plate_layout = your_plate_layout,
  sero_data = sero_data
)
}
\author{
Shazia Ruybal-Pesántez, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SeroTrackR-package.R
\docType{package}
\name{SeroTrackR-package}
\alias{SeroTrackR}
\alias{SeroTrackR-package}
\title{SeroTrackR: Serology-Based Data Analysis and Visualization}
\description{
Data wrangling and cleaning, quality control checks and implementation of machine learning classification algorithm.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/dionnecargy/SeroTrackR}
  \item \url{https://dionnecargy.github.io/SeroTrackR/}
  \item Report bugs at \url{https://github.com/dionnecargy/SeroTrackR/issues}
}

}
\author{
\strong{Maintainer}: Dionne Argyropoulos \email{argyropoulos.d@wehi.edu.au}

Authors:
\itemize{
  \item Dionne Argyropoulos \email{argyropoulos.d@wehi.edu.au}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCounts.R
\name{plotCounts}
\alias{plotCounts}
\title{Plot Bead Count Data}
\usage{
plotCounts(qc_results, experiment_name)
}
\arguments{
\item{qc_results}{Output from `runQC()`.}

\item{experiment_name}{User-input experiment name.}
}
\value{
Tile Plot showing binary result of "sufficient beads" with cut-off
>15 beads and "repeat" less than or equal to 15 beads (ggplot).
}
\description{
This function gets the count data and plots the plate image, creating a new
facet (i.e., panel) for each antigen and each line represents the
different plates so that they can be visualised.
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results  <- runQC(sero_data, plate_list)

# Step 3: Plot Counts
plotCounts(qc_results, "experiment1")
}
}
\author{
Shazia Ruybal-Pesántez, Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotBeadCounts.R
\name{plotBeadCounts}
\alias{plotBeadCounts}
\title{Plot Bead Counts per Plate per Antigen}
\usage{
plotBeadCounts(qc_results)
}
\arguments{
\item{qc_results}{Output from `runQC()`.}
}
\value{
Dot plot with values > 15 threshold coloured in blue (sufficient
beads) and less than or equal to 15 beads coloured in red (repeat)
faceted by each antigen
(ggplot).
}
\description{
Enhances the `plotCounts()` output by providing greater resolution,
displaying antigens per plate, and enabling SampleID name visibility via
hover (transformed to Plotly in server.R)
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
qc_results  <- runQC(sero_data, plate_list)

# Step 3: Plot Bead Counts
plotBeadCounts(qc_results)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renderTwoCols.R
\name{renderTwoCols}
\alias{renderTwoCols}
\title{Create two columns in Fluent UI}
\usage{
renderTwoCols(
  first_col,
  second_col,
  first_width = "50\%",
  second_width = "50\%"
)
}
\arguments{
\item{first_col}{A list of content for the first column.}

\item{second_col}{A list of content for the second column.}

\item{first_width}{Percent width of the column space (default: 50\%).}

\item{second_width}{Percent width of the column space (default: 50\%).}
}
\value{
Fluent UI window with two columns.
}
\description{
This function creates two columns in the Fluent UI format.
}
\examples{
# Minimal example using htmltools elements.
# This example runs without starting a Shiny app and is safe for CRAN.

if (requireNamespace("shiny.fluent", quietly = TRUE) &&
    requireNamespace("htmltools", quietly = TRUE)) {

  # Create simple content for each column
  col1 <- list(htmltools::div("First column content"))
  col2 <- list(htmltools::div("Second column content"))

  # Render two columns with default widths
  renderTwoCols(first_col = col1, second_col = col2)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.convert_mfi_to_dilution}
\alias{.convert_mfi_to_dilution}
\title{Convert mfi to dilution using known standard curve fit.}
\usage{
.convert_mfi_to_dilution(mfi, params, min_relative_dilution)
}
\arguments{
\item{mfi}{Known mfi of samples}

\item{params}{Known parameters for five parameter logistic fit.}

\item{min_relative_dilution}{Known minimum value of dilution in the standard curve. Relative means setting S1 to a dilution/RAU/concentration of 1.}
}
\value{
Returns the dilution of each sample in mfi.
}
\description{
Convert mfi to dilution using known standard curve fit.
}
\examples{
# This function is typically used within larger analysis pipelines.
# Below is a minimal runnable example using dummy values.

# Dummy five-parameter logistic fit parameters:
# a, b, c, d, e  (with e on the log scale)
# Additional placeholders (f, g) included so params[6] and params[7] exist.
dummy_params <- c(a = 10000, b = 1.2, c = 0.05, d = 50, e = log(0.01),
                  f = -5, g = 5)

# Example MFI value
mfi_example <- 1500

# Minimum relative dilution allowed
min_rel_dil <- 1

# Convert MFI to dilution
.convert_mfi_to_dilution(mfi_example, dummy_params, min_rel_dil)
}
\author{
Eamon Conway
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeCard.R
\name{makeCard}
\alias{makeCard}
\title{Make Card in Fluent UI}
\usage{
makeCard(title, id, content, size = 12, style = "")
}
\arguments{
\item{title}{String with the large title that will be printed in the card.}

\item{id}{Identifying tag for use to link.}

\item{content}{A list of content to be rendered.}

\item{size}{A value from 1 to 12 of the width of the screen (default = 12).}

\item{style}{Value for any css styling.}
}
\value{
A "card" in the Fluent UI format with content.
}
\description{
This function imports the makes a card following the Fluent UI format.
}
\examples{
# Minimal example creating a simple Fluent UI card.
# Safe for CRAN: runs only if shiny.fluent, htmltools, and glue are installed.

if (requireNamespace("shiny.fluent", quietly = TRUE) &&
    requireNamespace("htmltools", quietly = TRUE) &&
    requireNamespace("glue", quietly = TRUE)) {

  # Simple card content
  card_content <- list(
    htmltools::div("This is some example text inside the card.")
  )

  # Create a Fluent UI card
  makeCard(
    title   = "Example Card",
    id      = "example-card",
    content = card_content,
    size    = 6
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPlasmoPipeline.R
\name{runPlasmoPipeline}
\alias{runPlasmoPipeline}
\title{Run Pk/Pf/Pv Data Analysis Pipeline from Start to End}
\usage{
runPlasmoPipeline(
  raw_data,
  platform = "magpix",
  plate_layout,
  panel = "panel1",
  std_point,
  experiment_name = "experiment1",
  classify = "Yes",
  algorithm_type = "antibody_model",
  sens_spec = "balanced"
)
}
\arguments{
\item{raw_data}{String with the raw data path.}

\item{platform}{"magpix" or "bioplex". Default: "Bioplex"}

\item{plate_layout}{An ".xlsx" file with sheets labelled plate1, plate2... etc.}

\item{panel}{Panel of Pk/Pf/Pv antigens. Default = "panel1" or user provided csv of Antigens and Species.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve. Value is an integer.}

\item{experiment_name}{User-input experiment name. Default: "experiment1".}

\item{classify}{"Yes" or "No" depending on whether you would like classification or not. Default = "Yes".}

\item{algorithm_type}{Algorithm: "antibody_model" (PvSEM algorithm; default)}

\item{sens_spec}{User-selected Sensitivity/Specificity threshold: "balanced"
(default) or "90\% specificity".}
}
\value{
A data frame containing the MFI and RAU Dilution values for each sample, QC plots for standard curve, bead counts and blanks.
}
\description{
Run Pk/Pf/Pv Data Analysis Pipeline from Start to End
}
\examples{
\donttest{

# Helper to avoid repetition in examples
run_example_std <- function(std_point) {
  # Load raw data for given standard curve
  your_raw_data <- c(
    system.file("extdata",
                paste0("example_MAGPIX_pk_", std_point, "std_plate1.csv"),
                package = "SeroTrackR"),
    system.file("extdata",
                paste0("example_MAGPIX_pk_", std_point, "std_plate2.csv"),
                package = "SeroTrackR")
  )

  layout_file <- system.file(
    "extdata",
    paste0("example_platelayout_pk_", std_point, "std.xlsx"),
    package = "SeroTrackR"
  )

  # Run pipeline
  runPlasmoPipeline(
    raw_data = your_raw_data,
    platform = "magpix",
    plate_layout = layout_file,
    panel = "panel1",
    std_point = std_point,
    experiment_name = paste0(std_point, "-point standard curve")
  )
}

# ---- 5-point standard curve ----
results_5std <- run_example_std(5)

# ---- 10-point standard curve ----
results_10std <- run_example_std(10)

}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFItoRAU.R
\name{.setup_mfitorau_inputs}
\alias{.setup_mfitorau_inputs}
\title{Helper function to set up MFI to RAU function}
\usage{
.setup_mfitorau_inputs(df, plate_list, std_point)
}
\arguments{
\item{df}{Output from `readSeroData()`.}

\item{plate_list}{Output from `readPlateLayout()`.}

\item{std_point}{Standard Point Curve: 5 = 5-point curve, 10 = 10-point curve, "PvLDH" for LDH specific curve. Default = 10. Value is an integer.}
}
\value{
A list of processed sero_data, processed plate layout, antigen names, and parameters for standard curve.
}
\description{
Helper function to set up MFI to RAU function
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Setup MFI to RAU
setup <- .setup_mfitorau_inputs(
  df = sero_data$results,
  plate_list = plate_list,
  std_point = 10
)

}

}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runQC.R
\name{getSampleID}
\alias{getSampleID}
\title{Get SampleID from Plate Layout}
\usage{
getSampleID(processed_counts, plate_list)
}
\arguments{
\item{processed_counts}{Output from `processCounts()`.}

\item{plate_list}{Plate name inside of the plate layout file.}
}
\value{
Returns the corresponding Sample ID for the correct row/column in
the plate layout file. Henceforth "Sample ID" refers to the code in the
plate layout file, while "Sample" is the code in the Luminex file.
}
\description{
A helper function to extract Sample ID based on plate name and row/col
}
\examples{
\donttest{

# Step 0: Load example raw data
your_raw_data <- c(
  system.file("extdata", "example_MAGPIX_plate1.csv", package = "SeroTrackR"),
  system.file("extdata", "example_MAGPIX_plate2.csv", package = "SeroTrackR")
)
your_plate_layout <- system.file(
  "extdata",
  "example_platelayout_1.xlsx",
  package = "SeroTrackR"
)

# Step 1: Read serology data and plate layout
sero_data  <- readSeroData(your_raw_data,"magpix")
plate_list <- readPlateLayout(your_plate_layout, sero_data)

# Step 2: Process counts and perform quality control
counts      <- processCounts(sero_data)
counts_raw  <- getCounts(counts)
sample_ids  <- getSampleID(counts, plate_list)
}
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readSeroData.R
\name{.post_process_luminex}
\alias{.post_process_luminex}
\title{Helper function to process luminex into master_list}
\usage{
.post_process_luminex(sections, file_name, master_list)
}
\arguments{
\item{sections}{Output from `.post_process_bioplex()`.}

\item{file_name}{User input file name.}

\item{master_list}{Intermediary df from `readSeroData()`.}
}
\value{
List of data_raw, results, counts, blanks, stds, run
}
\description{
Helper function to process luminex into master_list
}
\author{
Dionne Argyropoulos
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\name{example_platelayout_pk_5std_xlsx}
\alias{example_platelayout_pk_5std_xlsx}
\title{5-Point Standard Curve Example Plate Layout}
\format{
A data frame with 9 rows and 13 variables.
\describe{
  \item{Plate}{Contains rows labelled "A" to "H"}
  \item{1-12}{Contains columns labelled "1" to "12"}
}
}
\description{
96 well plate map in a wide format used in the lab, used for when 5-point standard curves are required for pk/pf/pv analysis. Contains information of actual Sample ID names in each well.
}
\details{
This file is stored in inst/extdata
}
